
#include "Instruction.h"
#include <iostream>
#include <fstream>
#include <string>
#include <cstdio>

typedef edisassm::Instruction<edisassm::x86_64> insn64_t;
typedef edisassm::Instruction<edisassm::x86>    insn32_t;

struct test_data_t {
	unsigned size;
	const char *bytes;
	const char *result;
	unsigned int flags;
} test32_data[] = {
	// test Mod/RM decoding..
	{2,"\x89\x00", "mov dword ptr [eax], eax", insn32_t::FLAG_NONE},
	{2,"\x89\x01", "mov dword ptr [ecx], eax", insn32_t::FLAG_NONE},
	{2,"\x89\x02", "mov dword ptr [edx], eax", insn32_t::FLAG_NONE},
	{2,"\x89\x03", "mov dword ptr [ebx], eax", insn32_t::FLAG_NONE},
	{2,"\x89\x06", "mov dword ptr [esi], eax", insn32_t::FLAG_NONE},
	{2,"\x89\x07", "mov dword ptr [edi], eax", insn32_t::FLAG_NONE},
	{2,"\x89\x08", "mov dword ptr [eax], ecx", insn32_t::FLAG_NONE},
	{2,"\x89\x09", "mov dword ptr [ecx], ecx", insn32_t::FLAG_NONE},
	{2,"\x89\x0a", "mov dword ptr [edx], ecx", insn32_t::FLAG_NONE},
	{2,"\x89\x0b", "mov dword ptr [ebx], ecx", insn32_t::FLAG_NONE},
	{2,"\x89\x0e", "mov dword ptr [esi], ecx", insn32_t::FLAG_NONE},
	{2,"\x89\x0f", "mov dword ptr [edi], ecx", insn32_t::FLAG_NONE},
	{2,"\x89\x10", "mov dword ptr [eax], edx", insn32_t::FLAG_NONE},
	{2,"\x89\x11", "mov dword ptr [ecx], edx", insn32_t::FLAG_NONE},
	{2,"\x89\x12", "mov dword ptr [edx], edx", insn32_t::FLAG_NONE},
	{2,"\x89\x13", "mov dword ptr [ebx], edx", insn32_t::FLAG_NONE},
	{2,"\x89\x16", "mov dword ptr [esi], edx", insn32_t::FLAG_NONE},
	{2,"\x89\x17", "mov dword ptr [edi], edx", insn32_t::FLAG_NONE},
	{2,"\x89\x18", "mov dword ptr [eax], ebx", insn32_t::FLAG_NONE},
	{2,"\x89\x19", "mov dword ptr [ecx], ebx", insn32_t::FLAG_NONE},
	{2,"\x89\x1a", "mov dword ptr [edx], ebx", insn32_t::FLAG_NONE},
	{2,"\x89\x1b", "mov dword ptr [ebx], ebx", insn32_t::FLAG_NONE},
	{2,"\x89\x1e", "mov dword ptr [esi], ebx", insn32_t::FLAG_NONE},
	{2,"\x89\x1f", "mov dword ptr [edi], ebx", insn32_t::FLAG_NONE},
	{2,"\x89\x20", "mov dword ptr [eax], esp", insn32_t::FLAG_NONE},
	{2,"\x89\x21", "mov dword ptr [ecx], esp", insn32_t::FLAG_NONE},
	{2,"\x89\x22", "mov dword ptr [edx], esp", insn32_t::FLAG_NONE},
	{2,"\x89\x23", "mov dword ptr [ebx], esp", insn32_t::FLAG_NONE},
	{2,"\x89\x26", "mov dword ptr [esi], esp", insn32_t::FLAG_NONE},
	{2,"\x89\x27", "mov dword ptr [edi], esp", insn32_t::FLAG_NONE},
	{2,"\x89\x28", "mov dword ptr [eax], ebp", insn32_t::FLAG_NONE},
	{2,"\x89\x29", "mov dword ptr [ecx], ebp", insn32_t::FLAG_NONE},
	{2,"\x89\x2a", "mov dword ptr [edx], ebp", insn32_t::FLAG_NONE},
	{2,"\x89\x2b", "mov dword ptr [ebx], ebp", insn32_t::FLAG_NONE},
	{2,"\x89\x2e", "mov dword ptr [esi], ebp", insn32_t::FLAG_NONE},
	{2,"\x89\x2f", "mov dword ptr [edi], ebp", insn32_t::FLAG_NONE},
	{2,"\x89\x30", "mov dword ptr [eax], esi", insn32_t::FLAG_NONE},
	{2,"\x89\x31", "mov dword ptr [ecx], esi", insn32_t::FLAG_NONE},
	{2,"\x89\x32", "mov dword ptr [edx], esi", insn32_t::FLAG_NONE},
	{2,"\x89\x33", "mov dword ptr [ebx], esi", insn32_t::FLAG_NONE},
	{2,"\x89\x36", "mov dword ptr [esi], esi", insn32_t::FLAG_NONE},
	{2,"\x89\x37", "mov dword ptr [edi], esi", insn32_t::FLAG_NONE},
	{2,"\x89\x38", "mov dword ptr [eax], edi", insn32_t::FLAG_NONE},
	{2,"\x89\x39", "mov dword ptr [ecx], edi", insn32_t::FLAG_NONE},
	{2,"\x89\x3a", "mov dword ptr [edx], edi", insn32_t::FLAG_NONE},
	{2,"\x89\x3b", "mov dword ptr [ebx], edi", insn32_t::FLAG_NONE},
	{2,"\x89\x3e", "mov dword ptr [esi], edi", insn32_t::FLAG_NONE},
	{2,"\x89\x3f", "mov dword ptr [edi], edi", insn32_t::FLAG_NONE},
	{2,"\x89\xc0", "mov eax, eax", insn32_t::FLAG_NONE},
	{2,"\x89\xc1", "mov ecx, eax", insn32_t::FLAG_NONE},
	{2,"\x89\xc2", "mov edx, eax", insn32_t::FLAG_NONE},
	{2,"\x89\xc3", "mov ebx, eax", insn32_t::FLAG_NONE},
	{2,"\x89\xc4", "mov esp, eax", insn32_t::FLAG_NONE},
	{2,"\x89\xc5", "mov ebp, eax", insn32_t::FLAG_NONE},
	{2,"\x89\xc6", "mov esi, eax", insn32_t::FLAG_NONE},
	{2,"\x89\xc7", "mov edi, eax", insn32_t::FLAG_NONE},
	{2,"\x89\xc8", "mov eax, ecx", insn32_t::FLAG_NONE},
	{2,"\x89\xc9", "mov ecx, ecx", insn32_t::FLAG_NONE},
	{2,"\x89\xca", "mov edx, ecx", insn32_t::FLAG_NONE},
	{2,"\x89\xcb", "mov ebx, ecx", insn32_t::FLAG_NONE},
	{2,"\x89\xcc", "mov esp, ecx", insn32_t::FLAG_NONE},
	{2,"\x89\xcd", "mov ebp, ecx", insn32_t::FLAG_NONE},
	{2,"\x89\xce", "mov esi, ecx", insn32_t::FLAG_NONE},
	{2,"\x89\xcf", "mov edi, ecx", insn32_t::FLAG_NONE},
	{2,"\x89\xd0", "mov eax, edx", insn32_t::FLAG_NONE},
	{2,"\x89\xd1", "mov ecx, edx", insn32_t::FLAG_NONE},
	{2,"\x89\xd2", "mov edx, edx", insn32_t::FLAG_NONE},
	{2,"\x89\xd3", "mov ebx, edx", insn32_t::FLAG_NONE},
	{2,"\x89\xd4", "mov esp, edx", insn32_t::FLAG_NONE},
	{2,"\x89\xd5", "mov ebp, edx", insn32_t::FLAG_NONE},
	{2,"\x89\xd6", "mov esi, edx", insn32_t::FLAG_NONE},
	{2,"\x89\xd7", "mov edi, edx", insn32_t::FLAG_NONE},
	{2,"\x89\xd8", "mov eax, ebx", insn32_t::FLAG_NONE},
	{2,"\x89\xd9", "mov ecx, ebx", insn32_t::FLAG_NONE},
	{2,"\x89\xda", "mov edx, ebx", insn32_t::FLAG_NONE},
	{2,"\x89\xdb", "mov ebx, ebx", insn32_t::FLAG_NONE},
	{2,"\x89\xdc", "mov esp, ebx", insn32_t::FLAG_NONE},
	{2,"\x89\xdd", "mov ebp, ebx", insn32_t::FLAG_NONE},
	{2,"\x89\xde", "mov esi, ebx", insn32_t::FLAG_NONE},
	{2,"\x89\xdf", "mov edi, ebx", insn32_t::FLAG_NONE},
	{2,"\x89\xe0", "mov eax, esp", insn32_t::FLAG_NONE},
	{2,"\x89\xe1", "mov ecx, esp", insn32_t::FLAG_NONE},
	{2,"\x89\xe2", "mov edx, esp", insn32_t::FLAG_NONE},
	{2,"\x89\xe3", "mov ebx, esp", insn32_t::FLAG_NONE},
	{2,"\x89\xe4", "mov esp, esp", insn32_t::FLAG_NONE},
	{2,"\x89\xe5", "mov ebp, esp", insn32_t::FLAG_NONE},
	{2,"\x89\xe6", "mov esi, esp", insn32_t::FLAG_NONE},
	{2,"\x89\xe7", "mov edi, esp", insn32_t::FLAG_NONE},
	{2,"\x89\xe8", "mov eax, ebp", insn32_t::FLAG_NONE},
	{2,"\x89\xe9", "mov ecx, ebp", insn32_t::FLAG_NONE},
	{2,"\x89\xea", "mov edx, ebp", insn32_t::FLAG_NONE},
	{2,"\x89\xeb", "mov ebx, ebp", insn32_t::FLAG_NONE},
	{2,"\x89\xec", "mov esp, ebp", insn32_t::FLAG_NONE},
	{2,"\x89\xed", "mov ebp, ebp", insn32_t::FLAG_NONE},
	{2,"\x89\xee", "mov esi, ebp", insn32_t::FLAG_NONE},
	{2,"\x89\xef", "mov edi, ebp", insn32_t::FLAG_NONE},
	{2,"\x89\xf0", "mov eax, esi", insn32_t::FLAG_NONE},
	{2,"\x89\xf1", "mov ecx, esi", insn32_t::FLAG_NONE},
	{2,"\x89\xf2", "mov edx, esi", insn32_t::FLAG_NONE},
	{2,"\x89\xf3", "mov ebx, esi", insn32_t::FLAG_NONE},
	{2,"\x89\xf4", "mov esp, esi", insn32_t::FLAG_NONE},
	{2,"\x89\xf5", "mov ebp, esi", insn32_t::FLAG_NONE},
	{2,"\x89\xf6", "mov esi, esi", insn32_t::FLAG_NONE},
	{2,"\x89\xf7", "mov edi, esi", insn32_t::FLAG_NONE},
	{2,"\x89\xf8", "mov eax, edi", insn32_t::FLAG_NONE},
	{2,"\x89\xf9", "mov ecx, edi", insn32_t::FLAG_NONE},
	{2,"\x89\xfa", "mov edx, edi", insn32_t::FLAG_NONE},
	{2,"\x89\xfb", "mov ebx, edi", insn32_t::FLAG_NONE},
	{2,"\x89\xfc", "mov esp, edi", insn32_t::FLAG_NONE},
	{2,"\x89\xfd", "mov ebp, edi", insn32_t::FLAG_NONE},
	{2,"\x89\xfe", "mov esi, edi", insn32_t::FLAG_NONE},
	{2,"\x89\xff", "mov edi, edi", insn32_t::FLAG_NONE},
	{2,"\x8b\x00", "mov eax, dword ptr [eax]", insn32_t::FLAG_NONE},
	{2,"\x8b\x01", "mov eax, dword ptr [ecx]", insn32_t::FLAG_NONE},
	{2,"\x8b\x02", "mov eax, dword ptr [edx]", insn32_t::FLAG_NONE},
	{2,"\x8b\x03", "mov eax, dword ptr [ebx]", insn32_t::FLAG_NONE},
	{2,"\x8b\x06", "mov eax, dword ptr [esi]", insn32_t::FLAG_NONE},
	{2,"\x8b\x07", "mov eax, dword ptr [edi]", insn32_t::FLAG_NONE},
	{2,"\x8b\x08", "mov ecx, dword ptr [eax]", insn32_t::FLAG_NONE},
	{2,"\x8b\x09", "mov ecx, dword ptr [ecx]", insn32_t::FLAG_NONE},
	{2,"\x8b\x0a", "mov ecx, dword ptr [edx]", insn32_t::FLAG_NONE},
	{2,"\x8b\x0b", "mov ecx, dword ptr [ebx]", insn32_t::FLAG_NONE},
	{2,"\x8b\x0e", "mov ecx, dword ptr [esi]", insn32_t::FLAG_NONE},
	{2,"\x8b\x0f", "mov ecx, dword ptr [edi]", insn32_t::FLAG_NONE},
	{2,"\x8b\x10", "mov edx, dword ptr [eax]", insn32_t::FLAG_NONE},
	{2,"\x8b\x11", "mov edx, dword ptr [ecx]", insn32_t::FLAG_NONE},
	{2,"\x8b\x12", "mov edx, dword ptr [edx]", insn32_t::FLAG_NONE},
	{2,"\x8b\x13", "mov edx, dword ptr [ebx]", insn32_t::FLAG_NONE},
	{2,"\x8b\x16", "mov edx, dword ptr [esi]", insn32_t::FLAG_NONE},
	{2,"\x8b\x17", "mov edx, dword ptr [edi]", insn32_t::FLAG_NONE},
	{2,"\x8b\x18", "mov ebx, dword ptr [eax]", insn32_t::FLAG_NONE},
	{2,"\x8b\x19", "mov ebx, dword ptr [ecx]", insn32_t::FLAG_NONE},
	{2,"\x8b\x1a", "mov ebx, dword ptr [edx]", insn32_t::FLAG_NONE},
	{2,"\x8b\x1b", "mov ebx, dword ptr [ebx]", insn32_t::FLAG_NONE},
	{2,"\x8b\x1e", "mov ebx, dword ptr [esi]", insn32_t::FLAG_NONE},
	{2,"\x8b\x1f", "mov ebx, dword ptr [edi]", insn32_t::FLAG_NONE},
	{2,"\x8b\x20", "mov esp, dword ptr [eax]", insn32_t::FLAG_NONE},
	{2,"\x8b\x21", "mov esp, dword ptr [ecx]", insn32_t::FLAG_NONE},
	{2,"\x8b\x22", "mov esp, dword ptr [edx]", insn32_t::FLAG_NONE},
	{2,"\x8b\x23", "mov esp, dword ptr [ebx]", insn32_t::FLAG_NONE},
	{2,"\x8b\x26", "mov esp, dword ptr [esi]", insn32_t::FLAG_NONE},
	{2,"\x8b\x27", "mov esp, dword ptr [edi]", insn32_t::FLAG_NONE},
	{2,"\x8b\x28", "mov ebp, dword ptr [eax]", insn32_t::FLAG_NONE},
	{2,"\x8b\x29", "mov ebp, dword ptr [ecx]", insn32_t::FLAG_NONE},
	{2,"\x8b\x2a", "mov ebp, dword ptr [edx]", insn32_t::FLAG_NONE},
	{2,"\x8b\x2b", "mov ebp, dword ptr [ebx]", insn32_t::FLAG_NONE},
	{2,"\x8b\x2e", "mov ebp, dword ptr [esi]", insn32_t::FLAG_NONE},
	{2,"\x8b\x2f", "mov ebp, dword ptr [edi]", insn32_t::FLAG_NONE},
	{2,"\x8b\x30", "mov esi, dword ptr [eax]", insn32_t::FLAG_NONE},
	{2,"\x8b\x31", "mov esi, dword ptr [ecx]", insn32_t::FLAG_NONE},
	{2,"\x8b\x32", "mov esi, dword ptr [edx]", insn32_t::FLAG_NONE},
	{2,"\x8b\x33", "mov esi, dword ptr [ebx]", insn32_t::FLAG_NONE},
	{2,"\x8b\x36", "mov esi, dword ptr [esi]", insn32_t::FLAG_NONE},
	{2,"\x8b\x37", "mov esi, dword ptr [edi]", insn32_t::FLAG_NONE},
	{2,"\x8b\x38", "mov edi, dword ptr [eax]", insn32_t::FLAG_NONE},
	{2,"\x8b\x39", "mov edi, dword ptr [ecx]", insn32_t::FLAG_NONE},
	{2,"\x8b\x3a", "mov edi, dword ptr [edx]", insn32_t::FLAG_NONE},
	{2,"\x8b\x3b", "mov edi, dword ptr [ebx]", insn32_t::FLAG_NONE},
	{2,"\x8b\x3e", "mov edi, dword ptr [esi]", insn32_t::FLAG_NONE},
	{2,"\x8b\x3f", "mov edi, dword ptr [edi]", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x00", "mov dword ptr [eax+eax], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x01", "mov dword ptr [ecx+eax], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x02", "mov dword ptr [edx+eax], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x03", "mov dword ptr [ebx+eax], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x04", "mov dword ptr [esp+eax], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x06", "mov dword ptr [esi+eax], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x07", "mov dword ptr [edi+eax], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x08", "mov dword ptr [eax+ecx], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x09", "mov dword ptr [ecx+ecx], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x0a", "mov dword ptr [edx+ecx], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x0b", "mov dword ptr [ebx+ecx], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x0c", "mov dword ptr [esp+ecx], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x0e", "mov dword ptr [esi+ecx], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x0f", "mov dword ptr [edi+ecx], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x10", "mov dword ptr [eax+edx], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x11", "mov dword ptr [ecx+edx], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x12", "mov dword ptr [edx+edx], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x13", "mov dword ptr [ebx+edx], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x14", "mov dword ptr [esp+edx], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x16", "mov dword ptr [esi+edx], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x17", "mov dword ptr [edi+edx], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x18", "mov dword ptr [eax+ebx], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x19", "mov dword ptr [ecx+ebx], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x1a", "mov dword ptr [edx+ebx], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x1b", "mov dword ptr [ebx+ebx], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x1c", "mov dword ptr [esp+ebx], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x1e", "mov dword ptr [esi+ebx], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x1f", "mov dword ptr [edi+ebx], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x24", "mov dword ptr [esp], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x28", "mov dword ptr [eax+ebp], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x29", "mov dword ptr [ecx+ebp], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x2a", "mov dword ptr [edx+ebp], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x2b", "mov dword ptr [ebx+ebp], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x2c", "mov dword ptr [esp+ebp], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x2e", "mov dword ptr [esi+ebp], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x2f", "mov dword ptr [edi+ebp], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x30", "mov dword ptr [eax+esi], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x31", "mov dword ptr [ecx+esi], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x32", "mov dword ptr [edx+esi], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x33", "mov dword ptr [ebx+esi], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x34", "mov dword ptr [esp+esi], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x36", "mov dword ptr [esi+esi], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x37", "mov dword ptr [edi+esi], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x38", "mov dword ptr [eax+edi], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x39", "mov dword ptr [ecx+edi], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x3a", "mov dword ptr [edx+edi], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x3b", "mov dword ptr [ebx+edi], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x3c", "mov dword ptr [esp+edi], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x3e", "mov dword ptr [esi+edi], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x3f", "mov dword ptr [edi+edi], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x40", "mov dword ptr [eax+eax*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x41", "mov dword ptr [ecx+eax*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x42", "mov dword ptr [edx+eax*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x43", "mov dword ptr [ebx+eax*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x44", "mov dword ptr [esp+eax*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x46", "mov dword ptr [esi+eax*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x47", "mov dword ptr [edi+eax*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x48", "mov dword ptr [eax+ecx*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x49", "mov dword ptr [ecx+ecx*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x4a", "mov dword ptr [edx+ecx*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x4b", "mov dword ptr [ebx+ecx*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x4c", "mov dword ptr [esp+ecx*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x4e", "mov dword ptr [esi+ecx*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x4f", "mov dword ptr [edi+ecx*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x50", "mov dword ptr [eax+edx*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x51", "mov dword ptr [ecx+edx*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x52", "mov dword ptr [edx+edx*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x53", "mov dword ptr [ebx+edx*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x54", "mov dword ptr [esp+edx*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x56", "mov dword ptr [esi+edx*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x57", "mov dword ptr [edi+edx*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x58", "mov dword ptr [eax+ebx*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x59", "mov dword ptr [ecx+ebx*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x5a", "mov dword ptr [edx+ebx*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x5b", "mov dword ptr [ebx+ebx*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x5c", "mov dword ptr [esp+ebx*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x5e", "mov dword ptr [esi+ebx*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x5f", "mov dword ptr [edi+ebx*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x68", "mov dword ptr [eax+ebp*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x69", "mov dword ptr [ecx+ebp*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x6a", "mov dword ptr [edx+ebp*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x6b", "mov dword ptr [ebx+ebp*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x6c", "mov dword ptr [esp+ebp*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x6e", "mov dword ptr [esi+ebp*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x6f", "mov dword ptr [edi+ebp*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x70", "mov dword ptr [eax+esi*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x71", "mov dword ptr [ecx+esi*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x72", "mov dword ptr [edx+esi*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x73", "mov dword ptr [ebx+esi*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x74", "mov dword ptr [esp+esi*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x76", "mov dword ptr [esi+esi*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x77", "mov dword ptr [edi+esi*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x78", "mov dword ptr [eax+edi*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x79", "mov dword ptr [ecx+edi*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x7a", "mov dword ptr [edx+edi*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x7b", "mov dword ptr [ebx+edi*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x7c", "mov dword ptr [esp+edi*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x7e", "mov dword ptr [esi+edi*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x7f", "mov dword ptr [edi+edi*2], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x80", "mov dword ptr [eax+eax*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x81", "mov dword ptr [ecx+eax*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x82", "mov dword ptr [edx+eax*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x83", "mov dword ptr [ebx+eax*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x84", "mov dword ptr [esp+eax*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x86", "mov dword ptr [esi+eax*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x87", "mov dword ptr [edi+eax*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x88", "mov dword ptr [eax+ecx*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x89", "mov dword ptr [ecx+ecx*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x8a", "mov dword ptr [edx+ecx*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x8b", "mov dword ptr [ebx+ecx*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x8c", "mov dword ptr [esp+ecx*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x8e", "mov dword ptr [esi+ecx*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x8f", "mov dword ptr [edi+ecx*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x90", "mov dword ptr [eax+edx*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x91", "mov dword ptr [ecx+edx*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x92", "mov dword ptr [edx+edx*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x93", "mov dword ptr [ebx+edx*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x94", "mov dword ptr [esp+edx*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x96", "mov dword ptr [esi+edx*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x97", "mov dword ptr [edi+edx*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x98", "mov dword ptr [eax+ebx*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x99", "mov dword ptr [ecx+ebx*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x9a", "mov dword ptr [edx+ebx*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x9b", "mov dword ptr [ebx+ebx*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x9c", "mov dword ptr [esp+ebx*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x9e", "mov dword ptr [esi+ebx*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\x9f", "mov dword ptr [edi+ebx*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xa8", "mov dword ptr [eax+ebp*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xa9", "mov dword ptr [ecx+ebp*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xaa", "mov dword ptr [edx+ebp*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xab", "mov dword ptr [ebx+ebp*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xac", "mov dword ptr [esp+ebp*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xae", "mov dword ptr [esi+ebp*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xaf", "mov dword ptr [edi+ebp*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xb0", "mov dword ptr [eax+esi*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xb1", "mov dword ptr [ecx+esi*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xb2", "mov dword ptr [edx+esi*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xb3", "mov dword ptr [ebx+esi*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xb4", "mov dword ptr [esp+esi*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xb6", "mov dword ptr [esi+esi*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xb7", "mov dword ptr [edi+esi*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xb8", "mov dword ptr [eax+edi*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xb9", "mov dword ptr [ecx+edi*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xba", "mov dword ptr [edx+edi*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xbb", "mov dword ptr [ebx+edi*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xbc", "mov dword ptr [esp+edi*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xbe", "mov dword ptr [esi+edi*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xbf", "mov dword ptr [edi+edi*4], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xc0", "mov dword ptr [eax+eax*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xc1", "mov dword ptr [ecx+eax*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xc2", "mov dword ptr [edx+eax*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xc3", "mov dword ptr [ebx+eax*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xc4", "mov dword ptr [esp+eax*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xc6", "mov dword ptr [esi+eax*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xc7", "mov dword ptr [edi+eax*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xc8", "mov dword ptr [eax+ecx*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xc9", "mov dword ptr [ecx+ecx*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xca", "mov dword ptr [edx+ecx*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xcb", "mov dword ptr [ebx+ecx*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xcc", "mov dword ptr [esp+ecx*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xce", "mov dword ptr [esi+ecx*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xcf", "mov dword ptr [edi+ecx*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xd0", "mov dword ptr [eax+edx*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xd1", "mov dword ptr [ecx+edx*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xd2", "mov dword ptr [edx+edx*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xd3", "mov dword ptr [ebx+edx*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xd4", "mov dword ptr [esp+edx*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xd6", "mov dword ptr [esi+edx*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xd7", "mov dword ptr [edi+edx*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xd8", "mov dword ptr [eax+ebx*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xd9", "mov dword ptr [ecx+ebx*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xda", "mov dword ptr [edx+ebx*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xdb", "mov dword ptr [ebx+ebx*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xdc", "mov dword ptr [esp+ebx*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xde", "mov dword ptr [esi+ebx*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xdf", "mov dword ptr [edi+ebx*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xe8", "mov dword ptr [eax+ebp*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xe9", "mov dword ptr [ecx+ebp*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xea", "mov dword ptr [edx+ebp*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xeb", "mov dword ptr [ebx+ebp*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xec", "mov dword ptr [esp+ebp*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xee", "mov dword ptr [esi+ebp*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xef", "mov dword ptr [edi+ebp*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xf0", "mov dword ptr [eax+esi*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xf1", "mov dword ptr [ecx+esi*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xf2", "mov dword ptr [edx+esi*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xf3", "mov dword ptr [ebx+esi*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xf4", "mov dword ptr [esp+esi*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xf6", "mov dword ptr [esi+esi*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xf7", "mov dword ptr [edi+esi*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xf8", "mov dword ptr [eax+edi*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xf9", "mov dword ptr [ecx+edi*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xfa", "mov dword ptr [edx+edi*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xfb", "mov dword ptr [ebx+edi*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xfc", "mov dword ptr [esp+edi*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xfe", "mov dword ptr [esi+edi*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x04\xff", "mov dword ptr [edi+edi*8], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x00", "mov dword ptr [eax+eax], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x01", "mov dword ptr [ecx+eax], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x02", "mov dword ptr [edx+eax], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x03", "mov dword ptr [ebx+eax], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x04", "mov dword ptr [esp+eax], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x06", "mov dword ptr [esi+eax], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x07", "mov dword ptr [edi+eax], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x08", "mov dword ptr [eax+ecx], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x09", "mov dword ptr [ecx+ecx], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x0a", "mov dword ptr [edx+ecx], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x0b", "mov dword ptr [ebx+ecx], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x0c", "mov dword ptr [esp+ecx], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x0e", "mov dword ptr [esi+ecx], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x0f", "mov dword ptr [edi+ecx], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x10", "mov dword ptr [eax+edx], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x11", "mov dword ptr [ecx+edx], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x12", "mov dword ptr [edx+edx], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x13", "mov dword ptr [ebx+edx], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x14", "mov dword ptr [esp+edx], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x16", "mov dword ptr [esi+edx], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x17", "mov dword ptr [edi+edx], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x18", "mov dword ptr [eax+ebx], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x19", "mov dword ptr [ecx+ebx], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x1a", "mov dword ptr [edx+ebx], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x1b", "mov dword ptr [ebx+ebx], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x1c", "mov dword ptr [esp+ebx], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x1e", "mov dword ptr [esi+ebx], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x1f", "mov dword ptr [edi+ebx], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x24", "mov dword ptr [esp], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x28", "mov dword ptr [eax+ebp], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x29", "mov dword ptr [ecx+ebp], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x2a", "mov dword ptr [edx+ebp], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x2b", "mov dword ptr [ebx+ebp], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x2c", "mov dword ptr [esp+ebp], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x2e", "mov dword ptr [esi+ebp], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x2f", "mov dword ptr [edi+ebp], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x30", "mov dword ptr [eax+esi], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x31", "mov dword ptr [ecx+esi], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x32", "mov dword ptr [edx+esi], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x33", "mov dword ptr [ebx+esi], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x34", "mov dword ptr [esp+esi], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x36", "mov dword ptr [esi+esi], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x37", "mov dword ptr [edi+esi], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x38", "mov dword ptr [eax+edi], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x39", "mov dword ptr [ecx+edi], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x3a", "mov dword ptr [edx+edi], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x3b", "mov dword ptr [ebx+edi], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x3c", "mov dword ptr [esp+edi], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x3e", "mov dword ptr [esi+edi], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x3f", "mov dword ptr [edi+edi], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x40", "mov dword ptr [eax+eax*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x41", "mov dword ptr [ecx+eax*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x42", "mov dword ptr [edx+eax*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x43", "mov dword ptr [ebx+eax*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x44", "mov dword ptr [esp+eax*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x46", "mov dword ptr [esi+eax*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x47", "mov dword ptr [edi+eax*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x48", "mov dword ptr [eax+ecx*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x49", "mov dword ptr [ecx+ecx*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x4a", "mov dword ptr [edx+ecx*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x4b", "mov dword ptr [ebx+ecx*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x4c", "mov dword ptr [esp+ecx*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x4e", "mov dword ptr [esi+ecx*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x4f", "mov dword ptr [edi+ecx*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x50", "mov dword ptr [eax+edx*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x51", "mov dword ptr [ecx+edx*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x52", "mov dword ptr [edx+edx*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x53", "mov dword ptr [ebx+edx*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x54", "mov dword ptr [esp+edx*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x56", "mov dword ptr [esi+edx*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x57", "mov dword ptr [edi+edx*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x58", "mov dword ptr [eax+ebx*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x59", "mov dword ptr [ecx+ebx*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x5a", "mov dword ptr [edx+ebx*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x5b", "mov dword ptr [ebx+ebx*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x5c", "mov dword ptr [esp+ebx*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x5e", "mov dword ptr [esi+ebx*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x5f", "mov dword ptr [edi+ebx*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x68", "mov dword ptr [eax+ebp*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x69", "mov dword ptr [ecx+ebp*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x6a", "mov dword ptr [edx+ebp*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x6b", "mov dword ptr [ebx+ebp*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x6c", "mov dword ptr [esp+ebp*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x6e", "mov dword ptr [esi+ebp*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x6f", "mov dword ptr [edi+ebp*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x70", "mov dword ptr [eax+esi*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x71", "mov dword ptr [ecx+esi*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x72", "mov dword ptr [edx+esi*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x73", "mov dword ptr [ebx+esi*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x74", "mov dword ptr [esp+esi*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x76", "mov dword ptr [esi+esi*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x77", "mov dword ptr [edi+esi*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x78", "mov dword ptr [eax+edi*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x79", "mov dword ptr [ecx+edi*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x7a", "mov dword ptr [edx+edi*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x7b", "mov dword ptr [ebx+edi*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x7c", "mov dword ptr [esp+edi*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x7e", "mov dword ptr [esi+edi*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x7f", "mov dword ptr [edi+edi*2], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x80", "mov dword ptr [eax+eax*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x81", "mov dword ptr [ecx+eax*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x82", "mov dword ptr [edx+eax*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x83", "mov dword ptr [ebx+eax*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x84", "mov dword ptr [esp+eax*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x86", "mov dword ptr [esi+eax*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x87", "mov dword ptr [edi+eax*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x88", "mov dword ptr [eax+ecx*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x89", "mov dword ptr [ecx+ecx*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x8a", "mov dword ptr [edx+ecx*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x8b", "mov dword ptr [ebx+ecx*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x8c", "mov dword ptr [esp+ecx*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x8e", "mov dword ptr [esi+ecx*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x8f", "mov dword ptr [edi+ecx*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x90", "mov dword ptr [eax+edx*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x91", "mov dword ptr [ecx+edx*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x92", "mov dword ptr [edx+edx*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x93", "mov dword ptr [ebx+edx*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x94", "mov dword ptr [esp+edx*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x96", "mov dword ptr [esi+edx*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x97", "mov dword ptr [edi+edx*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x98", "mov dword ptr [eax+ebx*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x99", "mov dword ptr [ecx+ebx*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x9a", "mov dword ptr [edx+ebx*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x9b", "mov dword ptr [ebx+ebx*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x9c", "mov dword ptr [esp+ebx*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x9e", "mov dword ptr [esi+ebx*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\x9f", "mov dword ptr [edi+ebx*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xa8", "mov dword ptr [eax+ebp*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xa9", "mov dword ptr [ecx+ebp*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xaa", "mov dword ptr [edx+ebp*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xab", "mov dword ptr [ebx+ebp*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xac", "mov dword ptr [esp+ebp*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xae", "mov dword ptr [esi+ebp*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xaf", "mov dword ptr [edi+ebp*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xb0", "mov dword ptr [eax+esi*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xb1", "mov dword ptr [ecx+esi*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xb2", "mov dword ptr [edx+esi*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xb3", "mov dword ptr [ebx+esi*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xb4", "mov dword ptr [esp+esi*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xb6", "mov dword ptr [esi+esi*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xb7", "mov dword ptr [edi+esi*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xb8", "mov dword ptr [eax+edi*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xb9", "mov dword ptr [ecx+edi*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xba", "mov dword ptr [edx+edi*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xbb", "mov dword ptr [ebx+edi*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xbc", "mov dword ptr [esp+edi*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xbe", "mov dword ptr [esi+edi*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xbf", "mov dword ptr [edi+edi*4], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xc0", "mov dword ptr [eax+eax*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xc1", "mov dword ptr [ecx+eax*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xc2", "mov dword ptr [edx+eax*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xc3", "mov dword ptr [ebx+eax*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xc4", "mov dword ptr [esp+eax*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xc6", "mov dword ptr [esi+eax*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xc7", "mov dword ptr [edi+eax*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xc8", "mov dword ptr [eax+ecx*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xc9", "mov dword ptr [ecx+ecx*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xca", "mov dword ptr [edx+ecx*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xcb", "mov dword ptr [ebx+ecx*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xcc", "mov dword ptr [esp+ecx*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xce", "mov dword ptr [esi+ecx*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xcf", "mov dword ptr [edi+ecx*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xd0", "mov dword ptr [eax+edx*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xd1", "mov dword ptr [ecx+edx*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xd2", "mov dword ptr [edx+edx*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xd3", "mov dword ptr [ebx+edx*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xd4", "mov dword ptr [esp+edx*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xd6", "mov dword ptr [esi+edx*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xd7", "mov dword ptr [edi+edx*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xd8", "mov dword ptr [eax+ebx*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xd9", "mov dword ptr [ecx+ebx*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xda", "mov dword ptr [edx+ebx*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xdb", "mov dword ptr [ebx+ebx*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xdc", "mov dword ptr [esp+ebx*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xde", "mov dword ptr [esi+ebx*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xdf", "mov dword ptr [edi+ebx*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xe8", "mov dword ptr [eax+ebp*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xe9", "mov dword ptr [ecx+ebp*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xea", "mov dword ptr [edx+ebp*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xeb", "mov dword ptr [ebx+ebp*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xec", "mov dword ptr [esp+ebp*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xee", "mov dword ptr [esi+ebp*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xef", "mov dword ptr [edi+ebp*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xf0", "mov dword ptr [eax+esi*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xf1", "mov dword ptr [ecx+esi*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xf2", "mov dword ptr [edx+esi*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xf3", "mov dword ptr [ebx+esi*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xf4", "mov dword ptr [esp+esi*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xf6", "mov dword ptr [esi+esi*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xf7", "mov dword ptr [edi+esi*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xf8", "mov dword ptr [eax+edi*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xf9", "mov dword ptr [ecx+edi*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xfa", "mov dword ptr [edx+edi*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xfb", "mov dword ptr [ebx+edi*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xfc", "mov dword ptr [esp+edi*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xfe", "mov dword ptr [esi+edi*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x0c\xff", "mov dword ptr [edi+edi*8], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x00", "mov dword ptr [eax+eax], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x01", "mov dword ptr [ecx+eax], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x02", "mov dword ptr [edx+eax], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x03", "mov dword ptr [ebx+eax], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x04", "mov dword ptr [esp+eax], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x06", "mov dword ptr [esi+eax], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x07", "mov dword ptr [edi+eax], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x08", "mov dword ptr [eax+ecx], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x09", "mov dword ptr [ecx+ecx], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x0a", "mov dword ptr [edx+ecx], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x0b", "mov dword ptr [ebx+ecx], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x0c", "mov dword ptr [esp+ecx], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x0e", "mov dword ptr [esi+ecx], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x0f", "mov dword ptr [edi+ecx], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x10", "mov dword ptr [eax+edx], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x11", "mov dword ptr [ecx+edx], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x12", "mov dword ptr [edx+edx], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x13", "mov dword ptr [ebx+edx], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x14", "mov dword ptr [esp+edx], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x16", "mov dword ptr [esi+edx], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x17", "mov dword ptr [edi+edx], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x18", "mov dword ptr [eax+ebx], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x19", "mov dword ptr [ecx+ebx], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x1a", "mov dword ptr [edx+ebx], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x1b", "mov dword ptr [ebx+ebx], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x1c", "mov dword ptr [esp+ebx], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x1e", "mov dword ptr [esi+ebx], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x1f", "mov dword ptr [edi+ebx], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x24", "mov dword ptr [esp], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x28", "mov dword ptr [eax+ebp], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x29", "mov dword ptr [ecx+ebp], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x2a", "mov dword ptr [edx+ebp], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x2b", "mov dword ptr [ebx+ebp], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x2c", "mov dword ptr [esp+ebp], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x2e", "mov dword ptr [esi+ebp], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x2f", "mov dword ptr [edi+ebp], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x30", "mov dword ptr [eax+esi], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x31", "mov dword ptr [ecx+esi], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x32", "mov dword ptr [edx+esi], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x33", "mov dword ptr [ebx+esi], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x34", "mov dword ptr [esp+esi], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x36", "mov dword ptr [esi+esi], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x37", "mov dword ptr [edi+esi], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x38", "mov dword ptr [eax+edi], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x39", "mov dword ptr [ecx+edi], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x3a", "mov dword ptr [edx+edi], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x3b", "mov dword ptr [ebx+edi], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x3c", "mov dword ptr [esp+edi], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x3e", "mov dword ptr [esi+edi], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x3f", "mov dword ptr [edi+edi], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x40", "mov dword ptr [eax+eax*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x41", "mov dword ptr [ecx+eax*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x42", "mov dword ptr [edx+eax*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x43", "mov dword ptr [ebx+eax*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x44", "mov dword ptr [esp+eax*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x46", "mov dword ptr [esi+eax*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x47", "mov dword ptr [edi+eax*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x48", "mov dword ptr [eax+ecx*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x49", "mov dword ptr [ecx+ecx*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x4a", "mov dword ptr [edx+ecx*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x4b", "mov dword ptr [ebx+ecx*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x4c", "mov dword ptr [esp+ecx*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x4e", "mov dword ptr [esi+ecx*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x4f", "mov dword ptr [edi+ecx*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x50", "mov dword ptr [eax+edx*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x51", "mov dword ptr [ecx+edx*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x52", "mov dword ptr [edx+edx*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x53", "mov dword ptr [ebx+edx*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x54", "mov dword ptr [esp+edx*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x56", "mov dword ptr [esi+edx*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x57", "mov dword ptr [edi+edx*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x58", "mov dword ptr [eax+ebx*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x59", "mov dword ptr [ecx+ebx*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x5a", "mov dword ptr [edx+ebx*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x5b", "mov dword ptr [ebx+ebx*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x5c", "mov dword ptr [esp+ebx*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x5e", "mov dword ptr [esi+ebx*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x5f", "mov dword ptr [edi+ebx*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x68", "mov dword ptr [eax+ebp*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x69", "mov dword ptr [ecx+ebp*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x6a", "mov dword ptr [edx+ebp*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x6b", "mov dword ptr [ebx+ebp*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x6c", "mov dword ptr [esp+ebp*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x6e", "mov dword ptr [esi+ebp*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x6f", "mov dword ptr [edi+ebp*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x70", "mov dword ptr [eax+esi*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x71", "mov dword ptr [ecx+esi*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x72", "mov dword ptr [edx+esi*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x73", "mov dword ptr [ebx+esi*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x74", "mov dword ptr [esp+esi*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x76", "mov dword ptr [esi+esi*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x77", "mov dword ptr [edi+esi*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x78", "mov dword ptr [eax+edi*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x79", "mov dword ptr [ecx+edi*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x7a", "mov dword ptr [edx+edi*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x7b", "mov dword ptr [ebx+edi*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x7c", "mov dword ptr [esp+edi*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x7e", "mov dword ptr [esi+edi*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x7f", "mov dword ptr [edi+edi*2], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x80", "mov dword ptr [eax+eax*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x81", "mov dword ptr [ecx+eax*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x82", "mov dword ptr [edx+eax*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x83", "mov dword ptr [ebx+eax*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x84", "mov dword ptr [esp+eax*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x86", "mov dword ptr [esi+eax*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x87", "mov dword ptr [edi+eax*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x88", "mov dword ptr [eax+ecx*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x89", "mov dword ptr [ecx+ecx*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x8a", "mov dword ptr [edx+ecx*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x8b", "mov dword ptr [ebx+ecx*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x8c", "mov dword ptr [esp+ecx*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x8e", "mov dword ptr [esi+ecx*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x8f", "mov dword ptr [edi+ecx*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x90", "mov dword ptr [eax+edx*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x91", "mov dword ptr [ecx+edx*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x92", "mov dword ptr [edx+edx*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x93", "mov dword ptr [ebx+edx*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x94", "mov dword ptr [esp+edx*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x96", "mov dword ptr [esi+edx*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x97", "mov dword ptr [edi+edx*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x98", "mov dword ptr [eax+ebx*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x99", "mov dword ptr [ecx+ebx*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x9a", "mov dword ptr [edx+ebx*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x9b", "mov dword ptr [ebx+ebx*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x9c", "mov dword ptr [esp+ebx*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x9e", "mov dword ptr [esi+ebx*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\x9f", "mov dword ptr [edi+ebx*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xa8", "mov dword ptr [eax+ebp*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xa9", "mov dword ptr [ecx+ebp*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xaa", "mov dword ptr [edx+ebp*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xab", "mov dword ptr [ebx+ebp*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xac", "mov dword ptr [esp+ebp*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xae", "mov dword ptr [esi+ebp*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xaf", "mov dword ptr [edi+ebp*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xb0", "mov dword ptr [eax+esi*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xb1", "mov dword ptr [ecx+esi*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xb2", "mov dword ptr [edx+esi*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xb3", "mov dword ptr [ebx+esi*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xb4", "mov dword ptr [esp+esi*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xb6", "mov dword ptr [esi+esi*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xb7", "mov dword ptr [edi+esi*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xb8", "mov dword ptr [eax+edi*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xb9", "mov dword ptr [ecx+edi*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xba", "mov dword ptr [edx+edi*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xbb", "mov dword ptr [ebx+edi*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xbc", "mov dword ptr [esp+edi*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xbe", "mov dword ptr [esi+edi*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xbf", "mov dword ptr [edi+edi*4], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xc0", "mov dword ptr [eax+eax*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xc1", "mov dword ptr [ecx+eax*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xc2", "mov dword ptr [edx+eax*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xc3", "mov dword ptr [ebx+eax*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xc4", "mov dword ptr [esp+eax*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xc6", "mov dword ptr [esi+eax*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xc7", "mov dword ptr [edi+eax*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xc8", "mov dword ptr [eax+ecx*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xc9", "mov dword ptr [ecx+ecx*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xca", "mov dword ptr [edx+ecx*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xcb", "mov dword ptr [ebx+ecx*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xcc", "mov dword ptr [esp+ecx*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xce", "mov dword ptr [esi+ecx*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xcf", "mov dword ptr [edi+ecx*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xd0", "mov dword ptr [eax+edx*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xd1", "mov dword ptr [ecx+edx*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xd2", "mov dword ptr [edx+edx*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xd3", "mov dword ptr [ebx+edx*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xd4", "mov dword ptr [esp+edx*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xd6", "mov dword ptr [esi+edx*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xd7", "mov dword ptr [edi+edx*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xd8", "mov dword ptr [eax+ebx*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xd9", "mov dword ptr [ecx+ebx*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xda", "mov dword ptr [edx+ebx*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xdb", "mov dword ptr [ebx+ebx*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xdc", "mov dword ptr [esp+ebx*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xde", "mov dword ptr [esi+ebx*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xdf", "mov dword ptr [edi+ebx*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xe8", "mov dword ptr [eax+ebp*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xe9", "mov dword ptr [ecx+ebp*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xea", "mov dword ptr [edx+ebp*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xeb", "mov dword ptr [ebx+ebp*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xec", "mov dword ptr [esp+ebp*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xee", "mov dword ptr [esi+ebp*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xef", "mov dword ptr [edi+ebp*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xf0", "mov dword ptr [eax+esi*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xf1", "mov dword ptr [ecx+esi*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xf2", "mov dword ptr [edx+esi*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xf3", "mov dword ptr [ebx+esi*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xf4", "mov dword ptr [esp+esi*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xf6", "mov dword ptr [esi+esi*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xf7", "mov dword ptr [edi+esi*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xf8", "mov dword ptr [eax+edi*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xf9", "mov dword ptr [ecx+edi*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xfa", "mov dword ptr [edx+edi*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xfb", "mov dword ptr [ebx+edi*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xfc", "mov dword ptr [esp+edi*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xfe", "mov dword ptr [esi+edi*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x14\xff", "mov dword ptr [edi+edi*8], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x00", "mov dword ptr [eax+eax], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x01", "mov dword ptr [ecx+eax], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x02", "mov dword ptr [edx+eax], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x03", "mov dword ptr [ebx+eax], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x04", "mov dword ptr [esp+eax], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x06", "mov dword ptr [esi+eax], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x07", "mov dword ptr [edi+eax], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x08", "mov dword ptr [eax+ecx], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x09", "mov dword ptr [ecx+ecx], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x0a", "mov dword ptr [edx+ecx], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x0b", "mov dword ptr [ebx+ecx], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x0c", "mov dword ptr [esp+ecx], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x0e", "mov dword ptr [esi+ecx], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x0f", "mov dword ptr [edi+ecx], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x10", "mov dword ptr [eax+edx], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x11", "mov dword ptr [ecx+edx], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x12", "mov dword ptr [edx+edx], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x13", "mov dword ptr [ebx+edx], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x14", "mov dword ptr [esp+edx], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x16", "mov dword ptr [esi+edx], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x17", "mov dword ptr [edi+edx], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x18", "mov dword ptr [eax+ebx], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x19", "mov dword ptr [ecx+ebx], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x1a", "mov dword ptr [edx+ebx], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x1b", "mov dword ptr [ebx+ebx], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x1c", "mov dword ptr [esp+ebx], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x1e", "mov dword ptr [esi+ebx], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x1f", "mov dword ptr [edi+ebx], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x24", "mov dword ptr [esp], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x28", "mov dword ptr [eax+ebp], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x29", "mov dword ptr [ecx+ebp], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x2a", "mov dword ptr [edx+ebp], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x2b", "mov dword ptr [ebx+ebp], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x2c", "mov dword ptr [esp+ebp], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x2e", "mov dword ptr [esi+ebp], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x2f", "mov dword ptr [edi+ebp], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x30", "mov dword ptr [eax+esi], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x31", "mov dword ptr [ecx+esi], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x32", "mov dword ptr [edx+esi], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x33", "mov dword ptr [ebx+esi], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x34", "mov dword ptr [esp+esi], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x36", "mov dword ptr [esi+esi], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x37", "mov dword ptr [edi+esi], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x38", "mov dword ptr [eax+edi], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x39", "mov dword ptr [ecx+edi], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x3a", "mov dword ptr [edx+edi], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x3b", "mov dword ptr [ebx+edi], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x3c", "mov dword ptr [esp+edi], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x3e", "mov dword ptr [esi+edi], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x3f", "mov dword ptr [edi+edi], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x40", "mov dword ptr [eax+eax*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x41", "mov dword ptr [ecx+eax*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x42", "mov dword ptr [edx+eax*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x43", "mov dword ptr [ebx+eax*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x44", "mov dword ptr [esp+eax*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x46", "mov dword ptr [esi+eax*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x47", "mov dword ptr [edi+eax*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x48", "mov dword ptr [eax+ecx*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x49", "mov dword ptr [ecx+ecx*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x4a", "mov dword ptr [edx+ecx*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x4b", "mov dword ptr [ebx+ecx*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x4c", "mov dword ptr [esp+ecx*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x4e", "mov dword ptr [esi+ecx*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x4f", "mov dword ptr [edi+ecx*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x50", "mov dword ptr [eax+edx*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x51", "mov dword ptr [ecx+edx*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x52", "mov dword ptr [edx+edx*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x53", "mov dword ptr [ebx+edx*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x54", "mov dword ptr [esp+edx*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x56", "mov dword ptr [esi+edx*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x57", "mov dword ptr [edi+edx*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x58", "mov dword ptr [eax+ebx*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x59", "mov dword ptr [ecx+ebx*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x5a", "mov dword ptr [edx+ebx*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x5b", "mov dword ptr [ebx+ebx*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x5c", "mov dword ptr [esp+ebx*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x5e", "mov dword ptr [esi+ebx*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x5f", "mov dword ptr [edi+ebx*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x68", "mov dword ptr [eax+ebp*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x69", "mov dword ptr [ecx+ebp*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x6a", "mov dword ptr [edx+ebp*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x6b", "mov dword ptr [ebx+ebp*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x6c", "mov dword ptr [esp+ebp*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x6e", "mov dword ptr [esi+ebp*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x6f", "mov dword ptr [edi+ebp*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x70", "mov dword ptr [eax+esi*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x71", "mov dword ptr [ecx+esi*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x72", "mov dword ptr [edx+esi*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x73", "mov dword ptr [ebx+esi*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x74", "mov dword ptr [esp+esi*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x76", "mov dword ptr [esi+esi*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x77", "mov dword ptr [edi+esi*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x78", "mov dword ptr [eax+edi*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x79", "mov dword ptr [ecx+edi*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x7a", "mov dword ptr [edx+edi*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x7b", "mov dword ptr [ebx+edi*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x7c", "mov dword ptr [esp+edi*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x7e", "mov dword ptr [esi+edi*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x7f", "mov dword ptr [edi+edi*2], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x80", "mov dword ptr [eax+eax*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x81", "mov dword ptr [ecx+eax*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x82", "mov dword ptr [edx+eax*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x83", "mov dword ptr [ebx+eax*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x84", "mov dword ptr [esp+eax*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x86", "mov dword ptr [esi+eax*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x87", "mov dword ptr [edi+eax*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x88", "mov dword ptr [eax+ecx*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x89", "mov dword ptr [ecx+ecx*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x8a", "mov dword ptr [edx+ecx*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x8b", "mov dword ptr [ebx+ecx*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x8c", "mov dword ptr [esp+ecx*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x8e", "mov dword ptr [esi+ecx*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x8f", "mov dword ptr [edi+ecx*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x90", "mov dword ptr [eax+edx*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x91", "mov dword ptr [ecx+edx*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x92", "mov dword ptr [edx+edx*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x93", "mov dword ptr [ebx+edx*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x94", "mov dword ptr [esp+edx*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x96", "mov dword ptr [esi+edx*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x97", "mov dword ptr [edi+edx*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x98", "mov dword ptr [eax+ebx*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x99", "mov dword ptr [ecx+ebx*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x9a", "mov dword ptr [edx+ebx*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x9b", "mov dword ptr [ebx+ebx*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x9c", "mov dword ptr [esp+ebx*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x9e", "mov dword ptr [esi+ebx*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\x9f", "mov dword ptr [edi+ebx*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xa8", "mov dword ptr [eax+ebp*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xa9", "mov dword ptr [ecx+ebp*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xaa", "mov dword ptr [edx+ebp*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xab", "mov dword ptr [ebx+ebp*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xac", "mov dword ptr [esp+ebp*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xae", "mov dword ptr [esi+ebp*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xaf", "mov dword ptr [edi+ebp*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xb0", "mov dword ptr [eax+esi*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xb1", "mov dword ptr [ecx+esi*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xb2", "mov dword ptr [edx+esi*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xb3", "mov dword ptr [ebx+esi*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xb4", "mov dword ptr [esp+esi*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xb6", "mov dword ptr [esi+esi*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xb7", "mov dword ptr [edi+esi*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xb8", "mov dword ptr [eax+edi*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xb9", "mov dword ptr [ecx+edi*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xba", "mov dword ptr [edx+edi*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xbb", "mov dword ptr [ebx+edi*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xbc", "mov dword ptr [esp+edi*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xbe", "mov dword ptr [esi+edi*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xbf", "mov dword ptr [edi+edi*4], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xc0", "mov dword ptr [eax+eax*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xc1", "mov dword ptr [ecx+eax*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xc2", "mov dword ptr [edx+eax*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xc3", "mov dword ptr [ebx+eax*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xc4", "mov dword ptr [esp+eax*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xc6", "mov dword ptr [esi+eax*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xc7", "mov dword ptr [edi+eax*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xc8", "mov dword ptr [eax+ecx*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xc9", "mov dword ptr [ecx+ecx*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xca", "mov dword ptr [edx+ecx*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xcb", "mov dword ptr [ebx+ecx*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xcc", "mov dword ptr [esp+ecx*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xce", "mov dword ptr [esi+ecx*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xcf", "mov dword ptr [edi+ecx*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xd0", "mov dword ptr [eax+edx*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xd1", "mov dword ptr [ecx+edx*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xd2", "mov dword ptr [edx+edx*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xd3", "mov dword ptr [ebx+edx*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xd4", "mov dword ptr [esp+edx*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xd6", "mov dword ptr [esi+edx*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xd7", "mov dword ptr [edi+edx*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xd8", "mov dword ptr [eax+ebx*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xd9", "mov dword ptr [ecx+ebx*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xda", "mov dword ptr [edx+ebx*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xdb", "mov dword ptr [ebx+ebx*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xdc", "mov dword ptr [esp+ebx*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xde", "mov dword ptr [esi+ebx*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xdf", "mov dword ptr [edi+ebx*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xe8", "mov dword ptr [eax+ebp*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xe9", "mov dword ptr [ecx+ebp*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xea", "mov dword ptr [edx+ebp*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xeb", "mov dword ptr [ebx+ebp*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xec", "mov dword ptr [esp+ebp*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xee", "mov dword ptr [esi+ebp*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xef", "mov dword ptr [edi+ebp*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xf0", "mov dword ptr [eax+esi*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xf1", "mov dword ptr [ecx+esi*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xf2", "mov dword ptr [edx+esi*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xf3", "mov dword ptr [ebx+esi*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xf4", "mov dword ptr [esp+esi*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xf6", "mov dword ptr [esi+esi*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xf7", "mov dword ptr [edi+esi*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xf8", "mov dword ptr [eax+edi*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xf9", "mov dword ptr [ecx+edi*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xfa", "mov dword ptr [edx+edi*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xfb", "mov dword ptr [ebx+edi*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xfc", "mov dword ptr [esp+edi*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xfe", "mov dword ptr [esi+edi*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x1c\xff", "mov dword ptr [edi+edi*8], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x00", "mov dword ptr [eax+eax], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x01", "mov dword ptr [ecx+eax], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x02", "mov dword ptr [edx+eax], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x03", "mov dword ptr [ebx+eax], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x04", "mov dword ptr [esp+eax], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x06", "mov dword ptr [esi+eax], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x07", "mov dword ptr [edi+eax], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x08", "mov dword ptr [eax+ecx], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x09", "mov dword ptr [ecx+ecx], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x0a", "mov dword ptr [edx+ecx], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x0b", "mov dword ptr [ebx+ecx], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x0c", "mov dword ptr [esp+ecx], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x0e", "mov dword ptr [esi+ecx], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x0f", "mov dword ptr [edi+ecx], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x10", "mov dword ptr [eax+edx], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x11", "mov dword ptr [ecx+edx], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x12", "mov dword ptr [edx+edx], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x13", "mov dword ptr [ebx+edx], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x14", "mov dword ptr [esp+edx], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x16", "mov dword ptr [esi+edx], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x17", "mov dword ptr [edi+edx], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x18", "mov dword ptr [eax+ebx], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x19", "mov dword ptr [ecx+ebx], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x1a", "mov dword ptr [edx+ebx], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x1b", "mov dword ptr [ebx+ebx], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x1c", "mov dword ptr [esp+ebx], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x1e", "mov dword ptr [esi+ebx], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x1f", "mov dword ptr [edi+ebx], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x24", "mov dword ptr [esp], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x28", "mov dword ptr [eax+ebp], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x29", "mov dword ptr [ecx+ebp], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x2a", "mov dword ptr [edx+ebp], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x2b", "mov dword ptr [ebx+ebp], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x2c", "mov dword ptr [esp+ebp], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x2e", "mov dword ptr [esi+ebp], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x2f", "mov dword ptr [edi+ebp], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x30", "mov dword ptr [eax+esi], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x31", "mov dword ptr [ecx+esi], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x32", "mov dword ptr [edx+esi], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x33", "mov dword ptr [ebx+esi], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x34", "mov dword ptr [esp+esi], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x36", "mov dword ptr [esi+esi], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x37", "mov dword ptr [edi+esi], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x38", "mov dword ptr [eax+edi], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x39", "mov dword ptr [ecx+edi], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x3a", "mov dword ptr [edx+edi], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x3b", "mov dword ptr [ebx+edi], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x3c", "mov dword ptr [esp+edi], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x3e", "mov dword ptr [esi+edi], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x3f", "mov dword ptr [edi+edi], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x40", "mov dword ptr [eax+eax*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x41", "mov dword ptr [ecx+eax*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x42", "mov dword ptr [edx+eax*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x43", "mov dword ptr [ebx+eax*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x44", "mov dword ptr [esp+eax*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x46", "mov dword ptr [esi+eax*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x47", "mov dword ptr [edi+eax*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x48", "mov dword ptr [eax+ecx*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x49", "mov dword ptr [ecx+ecx*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x4a", "mov dword ptr [edx+ecx*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x4b", "mov dword ptr [ebx+ecx*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x4c", "mov dword ptr [esp+ecx*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x4e", "mov dword ptr [esi+ecx*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x4f", "mov dword ptr [edi+ecx*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x50", "mov dword ptr [eax+edx*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x51", "mov dword ptr [ecx+edx*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x52", "mov dword ptr [edx+edx*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x53", "mov dword ptr [ebx+edx*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x54", "mov dword ptr [esp+edx*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x56", "mov dword ptr [esi+edx*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x57", "mov dword ptr [edi+edx*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x58", "mov dword ptr [eax+ebx*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x59", "mov dword ptr [ecx+ebx*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x5a", "mov dword ptr [edx+ebx*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x5b", "mov dword ptr [ebx+ebx*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x5c", "mov dword ptr [esp+ebx*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x5e", "mov dword ptr [esi+ebx*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x5f", "mov dword ptr [edi+ebx*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x68", "mov dword ptr [eax+ebp*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x69", "mov dword ptr [ecx+ebp*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x6a", "mov dword ptr [edx+ebp*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x6b", "mov dword ptr [ebx+ebp*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x6c", "mov dword ptr [esp+ebp*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x6e", "mov dword ptr [esi+ebp*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x6f", "mov dword ptr [edi+ebp*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x70", "mov dword ptr [eax+esi*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x71", "mov dword ptr [ecx+esi*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x72", "mov dword ptr [edx+esi*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x73", "mov dword ptr [ebx+esi*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x74", "mov dword ptr [esp+esi*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x76", "mov dword ptr [esi+esi*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x77", "mov dword ptr [edi+esi*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x78", "mov dword ptr [eax+edi*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x79", "mov dword ptr [ecx+edi*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x7a", "mov dword ptr [edx+edi*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x7b", "mov dword ptr [ebx+edi*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x7c", "mov dword ptr [esp+edi*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x7e", "mov dword ptr [esi+edi*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x7f", "mov dword ptr [edi+edi*2], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x80", "mov dword ptr [eax+eax*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x81", "mov dword ptr [ecx+eax*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x82", "mov dword ptr [edx+eax*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x83", "mov dword ptr [ebx+eax*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x84", "mov dword ptr [esp+eax*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x86", "mov dword ptr [esi+eax*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x87", "mov dword ptr [edi+eax*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x88", "mov dword ptr [eax+ecx*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x89", "mov dword ptr [ecx+ecx*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x8a", "mov dword ptr [edx+ecx*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x8b", "mov dword ptr [ebx+ecx*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x8c", "mov dword ptr [esp+ecx*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x8e", "mov dword ptr [esi+ecx*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x8f", "mov dword ptr [edi+ecx*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x90", "mov dword ptr [eax+edx*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x91", "mov dword ptr [ecx+edx*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x92", "mov dword ptr [edx+edx*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x93", "mov dword ptr [ebx+edx*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x94", "mov dword ptr [esp+edx*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x96", "mov dword ptr [esi+edx*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x97", "mov dword ptr [edi+edx*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x98", "mov dword ptr [eax+ebx*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x99", "mov dword ptr [ecx+ebx*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x9a", "mov dword ptr [edx+ebx*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x9b", "mov dword ptr [ebx+ebx*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x9c", "mov dword ptr [esp+ebx*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x9e", "mov dword ptr [esi+ebx*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\x9f", "mov dword ptr [edi+ebx*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xa8", "mov dword ptr [eax+ebp*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xa9", "mov dword ptr [ecx+ebp*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xaa", "mov dword ptr [edx+ebp*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xab", "mov dword ptr [ebx+ebp*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xac", "mov dword ptr [esp+ebp*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xae", "mov dword ptr [esi+ebp*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xaf", "mov dword ptr [edi+ebp*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xb0", "mov dword ptr [eax+esi*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xb1", "mov dword ptr [ecx+esi*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xb2", "mov dword ptr [edx+esi*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xb3", "mov dword ptr [ebx+esi*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xb4", "mov dword ptr [esp+esi*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xb6", "mov dword ptr [esi+esi*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xb7", "mov dword ptr [edi+esi*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xb8", "mov dword ptr [eax+edi*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xb9", "mov dword ptr [ecx+edi*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xba", "mov dword ptr [edx+edi*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xbb", "mov dword ptr [ebx+edi*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xbc", "mov dword ptr [esp+edi*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xbe", "mov dword ptr [esi+edi*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xbf", "mov dword ptr [edi+edi*4], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xc0", "mov dword ptr [eax+eax*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xc1", "mov dword ptr [ecx+eax*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xc2", "mov dword ptr [edx+eax*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xc3", "mov dword ptr [ebx+eax*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xc4", "mov dword ptr [esp+eax*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xc6", "mov dword ptr [esi+eax*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xc7", "mov dword ptr [edi+eax*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xc8", "mov dword ptr [eax+ecx*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xc9", "mov dword ptr [ecx+ecx*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xca", "mov dword ptr [edx+ecx*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xcb", "mov dword ptr [ebx+ecx*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xcc", "mov dword ptr [esp+ecx*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xce", "mov dword ptr [esi+ecx*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xcf", "mov dword ptr [edi+ecx*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xd0", "mov dword ptr [eax+edx*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xd1", "mov dword ptr [ecx+edx*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xd2", "mov dword ptr [edx+edx*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xd3", "mov dword ptr [ebx+edx*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xd4", "mov dword ptr [esp+edx*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xd6", "mov dword ptr [esi+edx*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xd7", "mov dword ptr [edi+edx*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xd8", "mov dword ptr [eax+ebx*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xd9", "mov dword ptr [ecx+ebx*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xda", "mov dword ptr [edx+ebx*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xdb", "mov dword ptr [ebx+ebx*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xdc", "mov dword ptr [esp+ebx*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xde", "mov dword ptr [esi+ebx*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xdf", "mov dword ptr [edi+ebx*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xe8", "mov dword ptr [eax+ebp*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xe9", "mov dword ptr [ecx+ebp*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xea", "mov dword ptr [edx+ebp*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xeb", "mov dword ptr [ebx+ebp*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xec", "mov dword ptr [esp+ebp*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xee", "mov dword ptr [esi+ebp*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xef", "mov dword ptr [edi+ebp*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xf0", "mov dword ptr [eax+esi*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xf1", "mov dword ptr [ecx+esi*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xf2", "mov dword ptr [edx+esi*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xf3", "mov dword ptr [ebx+esi*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xf4", "mov dword ptr [esp+esi*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xf6", "mov dword ptr [esi+esi*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xf7", "mov dword ptr [edi+esi*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xf8", "mov dword ptr [eax+edi*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xf9", "mov dword ptr [ecx+edi*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xfa", "mov dword ptr [edx+edi*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xfb", "mov dword ptr [ebx+edi*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xfc", "mov dword ptr [esp+edi*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xfe", "mov dword ptr [esi+edi*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x24\xff", "mov dword ptr [edi+edi*8], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x00", "mov dword ptr [eax+eax], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x01", "mov dword ptr [ecx+eax], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x02", "mov dword ptr [edx+eax], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x03", "mov dword ptr [ebx+eax], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x04", "mov dword ptr [esp+eax], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x06", "mov dword ptr [esi+eax], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x07", "mov dword ptr [edi+eax], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x08", "mov dword ptr [eax+ecx], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x09", "mov dword ptr [ecx+ecx], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x0a", "mov dword ptr [edx+ecx], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x0b", "mov dword ptr [ebx+ecx], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x0c", "mov dword ptr [esp+ecx], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x0e", "mov dword ptr [esi+ecx], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x0f", "mov dword ptr [edi+ecx], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x10", "mov dword ptr [eax+edx], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x11", "mov dword ptr [ecx+edx], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x12", "mov dword ptr [edx+edx], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x13", "mov dword ptr [ebx+edx], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x14", "mov dword ptr [esp+edx], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x16", "mov dword ptr [esi+edx], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x17", "mov dword ptr [edi+edx], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x18", "mov dword ptr [eax+ebx], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x19", "mov dword ptr [ecx+ebx], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x1a", "mov dword ptr [edx+ebx], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x1b", "mov dword ptr [ebx+ebx], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x1c", "mov dword ptr [esp+ebx], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x1e", "mov dword ptr [esi+ebx], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x1f", "mov dword ptr [edi+ebx], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x24", "mov dword ptr [esp], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x28", "mov dword ptr [eax+ebp], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x29", "mov dword ptr [ecx+ebp], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x2a", "mov dword ptr [edx+ebp], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x2b", "mov dword ptr [ebx+ebp], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x2c", "mov dword ptr [esp+ebp], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x2e", "mov dword ptr [esi+ebp], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x2f", "mov dword ptr [edi+ebp], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x30", "mov dword ptr [eax+esi], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x31", "mov dword ptr [ecx+esi], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x32", "mov dword ptr [edx+esi], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x33", "mov dword ptr [ebx+esi], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x34", "mov dword ptr [esp+esi], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x36", "mov dword ptr [esi+esi], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x37", "mov dword ptr [edi+esi], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x38", "mov dword ptr [eax+edi], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x39", "mov dword ptr [ecx+edi], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x3a", "mov dword ptr [edx+edi], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x3b", "mov dword ptr [ebx+edi], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x3c", "mov dword ptr [esp+edi], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x3e", "mov dword ptr [esi+edi], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x3f", "mov dword ptr [edi+edi], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x40", "mov dword ptr [eax+eax*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x41", "mov dword ptr [ecx+eax*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x42", "mov dword ptr [edx+eax*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x43", "mov dword ptr [ebx+eax*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x44", "mov dword ptr [esp+eax*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x46", "mov dword ptr [esi+eax*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x47", "mov dword ptr [edi+eax*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x48", "mov dword ptr [eax+ecx*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x49", "mov dword ptr [ecx+ecx*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x4a", "mov dword ptr [edx+ecx*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x4b", "mov dword ptr [ebx+ecx*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x4c", "mov dword ptr [esp+ecx*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x4e", "mov dword ptr [esi+ecx*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x4f", "mov dword ptr [edi+ecx*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x50", "mov dword ptr [eax+edx*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x51", "mov dword ptr [ecx+edx*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x52", "mov dword ptr [edx+edx*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x53", "mov dword ptr [ebx+edx*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x54", "mov dword ptr [esp+edx*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x56", "mov dword ptr [esi+edx*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x57", "mov dword ptr [edi+edx*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x58", "mov dword ptr [eax+ebx*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x59", "mov dword ptr [ecx+ebx*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x5a", "mov dword ptr [edx+ebx*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x5b", "mov dword ptr [ebx+ebx*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x5c", "mov dword ptr [esp+ebx*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x5e", "mov dword ptr [esi+ebx*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x5f", "mov dword ptr [edi+ebx*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x68", "mov dword ptr [eax+ebp*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x69", "mov dword ptr [ecx+ebp*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x6a", "mov dword ptr [edx+ebp*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x6b", "mov dword ptr [ebx+ebp*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x6c", "mov dword ptr [esp+ebp*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x6e", "mov dword ptr [esi+ebp*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x6f", "mov dword ptr [edi+ebp*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x70", "mov dword ptr [eax+esi*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x71", "mov dword ptr [ecx+esi*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x72", "mov dword ptr [edx+esi*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x73", "mov dword ptr [ebx+esi*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x74", "mov dword ptr [esp+esi*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x76", "mov dword ptr [esi+esi*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x77", "mov dword ptr [edi+esi*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x78", "mov dword ptr [eax+edi*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x79", "mov dword ptr [ecx+edi*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x7a", "mov dword ptr [edx+edi*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x7b", "mov dword ptr [ebx+edi*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x7c", "mov dword ptr [esp+edi*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x7e", "mov dword ptr [esi+edi*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x7f", "mov dword ptr [edi+edi*2], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x80", "mov dword ptr [eax+eax*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x81", "mov dword ptr [ecx+eax*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x82", "mov dword ptr [edx+eax*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x83", "mov dword ptr [ebx+eax*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x84", "mov dword ptr [esp+eax*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x86", "mov dword ptr [esi+eax*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x87", "mov dword ptr [edi+eax*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x88", "mov dword ptr [eax+ecx*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x89", "mov dword ptr [ecx+ecx*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x8a", "mov dword ptr [edx+ecx*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x8b", "mov dword ptr [ebx+ecx*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x8c", "mov dword ptr [esp+ecx*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x8e", "mov dword ptr [esi+ecx*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x8f", "mov dword ptr [edi+ecx*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x90", "mov dword ptr [eax+edx*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x91", "mov dword ptr [ecx+edx*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x92", "mov dword ptr [edx+edx*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x93", "mov dword ptr [ebx+edx*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x94", "mov dword ptr [esp+edx*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x96", "mov dword ptr [esi+edx*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x97", "mov dword ptr [edi+edx*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x98", "mov dword ptr [eax+ebx*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x99", "mov dword ptr [ecx+ebx*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x9a", "mov dword ptr [edx+ebx*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x9b", "mov dword ptr [ebx+ebx*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x9c", "mov dword ptr [esp+ebx*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x9e", "mov dword ptr [esi+ebx*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\x9f", "mov dword ptr [edi+ebx*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xa8", "mov dword ptr [eax+ebp*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xa9", "mov dword ptr [ecx+ebp*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xaa", "mov dword ptr [edx+ebp*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xab", "mov dword ptr [ebx+ebp*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xac", "mov dword ptr [esp+ebp*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xae", "mov dword ptr [esi+ebp*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xaf", "mov dword ptr [edi+ebp*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xb0", "mov dword ptr [eax+esi*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xb1", "mov dword ptr [ecx+esi*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xb2", "mov dword ptr [edx+esi*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xb3", "mov dword ptr [ebx+esi*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xb4", "mov dword ptr [esp+esi*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xb6", "mov dword ptr [esi+esi*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xb7", "mov dword ptr [edi+esi*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xb8", "mov dword ptr [eax+edi*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xb9", "mov dword ptr [ecx+edi*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xba", "mov dword ptr [edx+edi*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xbb", "mov dword ptr [ebx+edi*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xbc", "mov dword ptr [esp+edi*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xbe", "mov dword ptr [esi+edi*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xbf", "mov dword ptr [edi+edi*4], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xc0", "mov dword ptr [eax+eax*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xc1", "mov dword ptr [ecx+eax*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xc2", "mov dword ptr [edx+eax*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xc3", "mov dword ptr [ebx+eax*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xc4", "mov dword ptr [esp+eax*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xc6", "mov dword ptr [esi+eax*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xc7", "mov dword ptr [edi+eax*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xc8", "mov dword ptr [eax+ecx*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xc9", "mov dword ptr [ecx+ecx*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xca", "mov dword ptr [edx+ecx*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xcb", "mov dword ptr [ebx+ecx*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xcc", "mov dword ptr [esp+ecx*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xce", "mov dword ptr [esi+ecx*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xcf", "mov dword ptr [edi+ecx*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xd0", "mov dword ptr [eax+edx*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xd1", "mov dword ptr [ecx+edx*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xd2", "mov dword ptr [edx+edx*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xd3", "mov dword ptr [ebx+edx*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xd4", "mov dword ptr [esp+edx*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xd6", "mov dword ptr [esi+edx*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xd7", "mov dword ptr [edi+edx*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xd8", "mov dword ptr [eax+ebx*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xd9", "mov dword ptr [ecx+ebx*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xda", "mov dword ptr [edx+ebx*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xdb", "mov dword ptr [ebx+ebx*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xdc", "mov dword ptr [esp+ebx*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xde", "mov dword ptr [esi+ebx*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xdf", "mov dword ptr [edi+ebx*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xe8", "mov dword ptr [eax+ebp*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xe9", "mov dword ptr [ecx+ebp*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xea", "mov dword ptr [edx+ebp*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xeb", "mov dword ptr [ebx+ebp*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xec", "mov dword ptr [esp+ebp*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xee", "mov dword ptr [esi+ebp*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xef", "mov dword ptr [edi+ebp*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xf0", "mov dword ptr [eax+esi*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xf1", "mov dword ptr [ecx+esi*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xf2", "mov dword ptr [edx+esi*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xf3", "mov dword ptr [ebx+esi*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xf4", "mov dword ptr [esp+esi*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xf6", "mov dword ptr [esi+esi*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xf7", "mov dword ptr [edi+esi*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xf8", "mov dword ptr [eax+edi*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xf9", "mov dword ptr [ecx+edi*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xfa", "mov dword ptr [edx+edi*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xfb", "mov dword ptr [ebx+edi*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xfc", "mov dword ptr [esp+edi*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xfe", "mov dword ptr [esi+edi*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x2c\xff", "mov dword ptr [edi+edi*8], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x00", "mov dword ptr [eax+eax], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x01", "mov dword ptr [ecx+eax], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x02", "mov dword ptr [edx+eax], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x03", "mov dword ptr [ebx+eax], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x04", "mov dword ptr [esp+eax], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x06", "mov dword ptr [esi+eax], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x07", "mov dword ptr [edi+eax], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x08", "mov dword ptr [eax+ecx], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x09", "mov dword ptr [ecx+ecx], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x0a", "mov dword ptr [edx+ecx], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x0b", "mov dword ptr [ebx+ecx], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x0c", "mov dword ptr [esp+ecx], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x0e", "mov dword ptr [esi+ecx], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x0f", "mov dword ptr [edi+ecx], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x10", "mov dword ptr [eax+edx], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x11", "mov dword ptr [ecx+edx], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x12", "mov dword ptr [edx+edx], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x13", "mov dword ptr [ebx+edx], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x14", "mov dword ptr [esp+edx], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x16", "mov dword ptr [esi+edx], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x17", "mov dword ptr [edi+edx], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x18", "mov dword ptr [eax+ebx], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x19", "mov dword ptr [ecx+ebx], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x1a", "mov dword ptr [edx+ebx], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x1b", "mov dword ptr [ebx+ebx], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x1c", "mov dword ptr [esp+ebx], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x1e", "mov dword ptr [esi+ebx], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x1f", "mov dword ptr [edi+ebx], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x24", "mov dword ptr [esp], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x28", "mov dword ptr [eax+ebp], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x29", "mov dword ptr [ecx+ebp], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x2a", "mov dword ptr [edx+ebp], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x2b", "mov dword ptr [ebx+ebp], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x2c", "mov dword ptr [esp+ebp], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x2e", "mov dword ptr [esi+ebp], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x2f", "mov dword ptr [edi+ebp], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x30", "mov dword ptr [eax+esi], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x31", "mov dword ptr [ecx+esi], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x32", "mov dword ptr [edx+esi], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x33", "mov dword ptr [ebx+esi], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x34", "mov dword ptr [esp+esi], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x36", "mov dword ptr [esi+esi], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x37", "mov dword ptr [edi+esi], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x38", "mov dword ptr [eax+edi], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x39", "mov dword ptr [ecx+edi], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x3a", "mov dword ptr [edx+edi], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x3b", "mov dword ptr [ebx+edi], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x3c", "mov dword ptr [esp+edi], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x3e", "mov dword ptr [esi+edi], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x3f", "mov dword ptr [edi+edi], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x40", "mov dword ptr [eax+eax*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x41", "mov dword ptr [ecx+eax*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x42", "mov dword ptr [edx+eax*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x43", "mov dword ptr [ebx+eax*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x44", "mov dword ptr [esp+eax*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x46", "mov dword ptr [esi+eax*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x47", "mov dword ptr [edi+eax*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x48", "mov dword ptr [eax+ecx*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x49", "mov dword ptr [ecx+ecx*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x4a", "mov dword ptr [edx+ecx*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x4b", "mov dword ptr [ebx+ecx*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x4c", "mov dword ptr [esp+ecx*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x4e", "mov dword ptr [esi+ecx*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x4f", "mov dword ptr [edi+ecx*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x50", "mov dword ptr [eax+edx*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x51", "mov dword ptr [ecx+edx*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x52", "mov dword ptr [edx+edx*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x53", "mov dword ptr [ebx+edx*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x54", "mov dword ptr [esp+edx*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x56", "mov dword ptr [esi+edx*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x57", "mov dword ptr [edi+edx*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x58", "mov dword ptr [eax+ebx*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x59", "mov dword ptr [ecx+ebx*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x5a", "mov dword ptr [edx+ebx*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x5b", "mov dword ptr [ebx+ebx*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x5c", "mov dword ptr [esp+ebx*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x5e", "mov dword ptr [esi+ebx*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x5f", "mov dword ptr [edi+ebx*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x68", "mov dword ptr [eax+ebp*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x69", "mov dword ptr [ecx+ebp*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x6a", "mov dword ptr [edx+ebp*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x6b", "mov dword ptr [ebx+ebp*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x6c", "mov dword ptr [esp+ebp*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x6e", "mov dword ptr [esi+ebp*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x6f", "mov dword ptr [edi+ebp*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x70", "mov dword ptr [eax+esi*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x71", "mov dword ptr [ecx+esi*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x72", "mov dword ptr [edx+esi*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x73", "mov dword ptr [ebx+esi*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x74", "mov dword ptr [esp+esi*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x76", "mov dword ptr [esi+esi*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x77", "mov dword ptr [edi+esi*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x78", "mov dword ptr [eax+edi*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x79", "mov dword ptr [ecx+edi*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x7a", "mov dword ptr [edx+edi*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x7b", "mov dword ptr [ebx+edi*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x7c", "mov dword ptr [esp+edi*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x7e", "mov dword ptr [esi+edi*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x7f", "mov dword ptr [edi+edi*2], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x80", "mov dword ptr [eax+eax*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x81", "mov dword ptr [ecx+eax*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x82", "mov dword ptr [edx+eax*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x83", "mov dword ptr [ebx+eax*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x84", "mov dword ptr [esp+eax*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x86", "mov dword ptr [esi+eax*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x87", "mov dword ptr [edi+eax*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x88", "mov dword ptr [eax+ecx*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x89", "mov dword ptr [ecx+ecx*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x8a", "mov dword ptr [edx+ecx*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x8b", "mov dword ptr [ebx+ecx*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x8c", "mov dword ptr [esp+ecx*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x8e", "mov dword ptr [esi+ecx*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x8f", "mov dword ptr [edi+ecx*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x90", "mov dword ptr [eax+edx*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x91", "mov dword ptr [ecx+edx*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x92", "mov dword ptr [edx+edx*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x93", "mov dword ptr [ebx+edx*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x94", "mov dword ptr [esp+edx*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x96", "mov dword ptr [esi+edx*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x97", "mov dword ptr [edi+edx*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x98", "mov dword ptr [eax+ebx*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x99", "mov dword ptr [ecx+ebx*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x9a", "mov dword ptr [edx+ebx*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x9b", "mov dword ptr [ebx+ebx*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x9c", "mov dword ptr [esp+ebx*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x9e", "mov dword ptr [esi+ebx*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\x9f", "mov dword ptr [edi+ebx*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xa8", "mov dword ptr [eax+ebp*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xa9", "mov dword ptr [ecx+ebp*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xaa", "mov dword ptr [edx+ebp*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xab", "mov dword ptr [ebx+ebp*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xac", "mov dword ptr [esp+ebp*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xae", "mov dword ptr [esi+ebp*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xaf", "mov dword ptr [edi+ebp*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xb0", "mov dword ptr [eax+esi*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xb1", "mov dword ptr [ecx+esi*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xb2", "mov dword ptr [edx+esi*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xb3", "mov dword ptr [ebx+esi*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xb4", "mov dword ptr [esp+esi*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xb6", "mov dword ptr [esi+esi*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xb7", "mov dword ptr [edi+esi*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xb8", "mov dword ptr [eax+edi*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xb9", "mov dword ptr [ecx+edi*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xba", "mov dword ptr [edx+edi*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xbb", "mov dword ptr [ebx+edi*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xbc", "mov dword ptr [esp+edi*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xbe", "mov dword ptr [esi+edi*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xbf", "mov dword ptr [edi+edi*4], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xc0", "mov dword ptr [eax+eax*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xc1", "mov dword ptr [ecx+eax*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xc2", "mov dword ptr [edx+eax*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xc3", "mov dword ptr [ebx+eax*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xc4", "mov dword ptr [esp+eax*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xc6", "mov dword ptr [esi+eax*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xc7", "mov dword ptr [edi+eax*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xc8", "mov dword ptr [eax+ecx*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xc9", "mov dword ptr [ecx+ecx*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xca", "mov dword ptr [edx+ecx*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xcb", "mov dword ptr [ebx+ecx*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xcc", "mov dword ptr [esp+ecx*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xce", "mov dword ptr [esi+ecx*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xcf", "mov dword ptr [edi+ecx*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xd0", "mov dword ptr [eax+edx*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xd1", "mov dword ptr [ecx+edx*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xd2", "mov dword ptr [edx+edx*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xd3", "mov dword ptr [ebx+edx*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xd4", "mov dword ptr [esp+edx*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xd6", "mov dword ptr [esi+edx*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xd7", "mov dword ptr [edi+edx*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xd8", "mov dword ptr [eax+ebx*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xd9", "mov dword ptr [ecx+ebx*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xda", "mov dword ptr [edx+ebx*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xdb", "mov dword ptr [ebx+ebx*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xdc", "mov dword ptr [esp+ebx*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xde", "mov dword ptr [esi+ebx*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xdf", "mov dword ptr [edi+ebx*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xe8", "mov dword ptr [eax+ebp*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xe9", "mov dword ptr [ecx+ebp*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xea", "mov dword ptr [edx+ebp*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xeb", "mov dword ptr [ebx+ebp*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xec", "mov dword ptr [esp+ebp*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xee", "mov dword ptr [esi+ebp*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xef", "mov dword ptr [edi+ebp*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xf0", "mov dword ptr [eax+esi*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xf1", "mov dword ptr [ecx+esi*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xf2", "mov dword ptr [edx+esi*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xf3", "mov dword ptr [ebx+esi*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xf4", "mov dword ptr [esp+esi*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xf6", "mov dword ptr [esi+esi*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xf7", "mov dword ptr [edi+esi*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xf8", "mov dword ptr [eax+edi*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xf9", "mov dword ptr [ecx+edi*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xfa", "mov dword ptr [edx+edi*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xfb", "mov dword ptr [ebx+edi*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xfc", "mov dword ptr [esp+edi*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xfe", "mov dword ptr [esi+edi*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x34\xff", "mov dword ptr [edi+edi*8], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x00", "mov dword ptr [eax+eax], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x01", "mov dword ptr [ecx+eax], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x02", "mov dword ptr [edx+eax], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x03", "mov dword ptr [ebx+eax], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x04", "mov dword ptr [esp+eax], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x06", "mov dword ptr [esi+eax], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x07", "mov dword ptr [edi+eax], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x08", "mov dword ptr [eax+ecx], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x09", "mov dword ptr [ecx+ecx], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x0a", "mov dword ptr [edx+ecx], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x0b", "mov dword ptr [ebx+ecx], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x0c", "mov dword ptr [esp+ecx], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x0e", "mov dword ptr [esi+ecx], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x0f", "mov dword ptr [edi+ecx], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x10", "mov dword ptr [eax+edx], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x11", "mov dword ptr [ecx+edx], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x12", "mov dword ptr [edx+edx], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x13", "mov dword ptr [ebx+edx], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x14", "mov dword ptr [esp+edx], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x16", "mov dword ptr [esi+edx], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x17", "mov dword ptr [edi+edx], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x18", "mov dword ptr [eax+ebx], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x19", "mov dword ptr [ecx+ebx], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x1a", "mov dword ptr [edx+ebx], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x1b", "mov dword ptr [ebx+ebx], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x1c", "mov dword ptr [esp+ebx], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x1e", "mov dword ptr [esi+ebx], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x1f", "mov dword ptr [edi+ebx], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x24", "mov dword ptr [esp], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x28", "mov dword ptr [eax+ebp], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x29", "mov dword ptr [ecx+ebp], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x2a", "mov dword ptr [edx+ebp], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x2b", "mov dword ptr [ebx+ebp], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x2c", "mov dword ptr [esp+ebp], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x2e", "mov dword ptr [esi+ebp], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x2f", "mov dword ptr [edi+ebp], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x30", "mov dword ptr [eax+esi], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x31", "mov dword ptr [ecx+esi], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x32", "mov dword ptr [edx+esi], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x33", "mov dword ptr [ebx+esi], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x34", "mov dword ptr [esp+esi], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x36", "mov dword ptr [esi+esi], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x37", "mov dword ptr [edi+esi], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x38", "mov dword ptr [eax+edi], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x39", "mov dword ptr [ecx+edi], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x3a", "mov dword ptr [edx+edi], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x3b", "mov dword ptr [ebx+edi], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x3c", "mov dword ptr [esp+edi], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x3e", "mov dword ptr [esi+edi], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x3f", "mov dword ptr [edi+edi], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x40", "mov dword ptr [eax+eax*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x41", "mov dword ptr [ecx+eax*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x42", "mov dword ptr [edx+eax*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x43", "mov dword ptr [ebx+eax*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x44", "mov dword ptr [esp+eax*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x46", "mov dword ptr [esi+eax*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x47", "mov dword ptr [edi+eax*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x48", "mov dword ptr [eax+ecx*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x49", "mov dword ptr [ecx+ecx*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x4a", "mov dword ptr [edx+ecx*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x4b", "mov dword ptr [ebx+ecx*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x4c", "mov dword ptr [esp+ecx*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x4e", "mov dword ptr [esi+ecx*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x4f", "mov dword ptr [edi+ecx*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x50", "mov dword ptr [eax+edx*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x51", "mov dword ptr [ecx+edx*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x52", "mov dword ptr [edx+edx*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x53", "mov dword ptr [ebx+edx*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x54", "mov dword ptr [esp+edx*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x56", "mov dword ptr [esi+edx*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x57", "mov dword ptr [edi+edx*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x58", "mov dword ptr [eax+ebx*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x59", "mov dword ptr [ecx+ebx*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x5a", "mov dword ptr [edx+ebx*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x5b", "mov dword ptr [ebx+ebx*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x5c", "mov dword ptr [esp+ebx*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x5e", "mov dword ptr [esi+ebx*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x5f", "mov dword ptr [edi+ebx*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x68", "mov dword ptr [eax+ebp*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x69", "mov dword ptr [ecx+ebp*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x6a", "mov dword ptr [edx+ebp*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x6b", "mov dword ptr [ebx+ebp*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x6c", "mov dword ptr [esp+ebp*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x6e", "mov dword ptr [esi+ebp*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x6f", "mov dword ptr [edi+ebp*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x70", "mov dword ptr [eax+esi*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x71", "mov dword ptr [ecx+esi*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x72", "mov dword ptr [edx+esi*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x73", "mov dword ptr [ebx+esi*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x74", "mov dword ptr [esp+esi*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x76", "mov dword ptr [esi+esi*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x77", "mov dword ptr [edi+esi*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x78", "mov dword ptr [eax+edi*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x79", "mov dword ptr [ecx+edi*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x7a", "mov dword ptr [edx+edi*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x7b", "mov dword ptr [ebx+edi*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x7c", "mov dword ptr [esp+edi*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x7e", "mov dword ptr [esi+edi*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x7f", "mov dword ptr [edi+edi*2], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x80", "mov dword ptr [eax+eax*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x81", "mov dword ptr [ecx+eax*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x82", "mov dword ptr [edx+eax*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x83", "mov dword ptr [ebx+eax*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x84", "mov dword ptr [esp+eax*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x86", "mov dword ptr [esi+eax*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x87", "mov dword ptr [edi+eax*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x88", "mov dword ptr [eax+ecx*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x89", "mov dword ptr [ecx+ecx*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x8a", "mov dword ptr [edx+ecx*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x8b", "mov dword ptr [ebx+ecx*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x8c", "mov dword ptr [esp+ecx*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x8e", "mov dword ptr [esi+ecx*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x8f", "mov dword ptr [edi+ecx*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x90", "mov dword ptr [eax+edx*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x91", "mov dword ptr [ecx+edx*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x92", "mov dword ptr [edx+edx*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x93", "mov dword ptr [ebx+edx*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x94", "mov dword ptr [esp+edx*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x96", "mov dword ptr [esi+edx*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x97", "mov dword ptr [edi+edx*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x98", "mov dword ptr [eax+ebx*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x99", "mov dword ptr [ecx+ebx*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x9a", "mov dword ptr [edx+ebx*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x9b", "mov dword ptr [ebx+ebx*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x9c", "mov dword ptr [esp+ebx*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x9e", "mov dword ptr [esi+ebx*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\x9f", "mov dword ptr [edi+ebx*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xa8", "mov dword ptr [eax+ebp*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xa9", "mov dword ptr [ecx+ebp*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xaa", "mov dword ptr [edx+ebp*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xab", "mov dword ptr [ebx+ebp*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xac", "mov dword ptr [esp+ebp*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xae", "mov dword ptr [esi+ebp*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xaf", "mov dword ptr [edi+ebp*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xb0", "mov dword ptr [eax+esi*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xb1", "mov dword ptr [ecx+esi*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xb2", "mov dword ptr [edx+esi*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xb3", "mov dword ptr [ebx+esi*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xb4", "mov dword ptr [esp+esi*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xb6", "mov dword ptr [esi+esi*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xb7", "mov dword ptr [edi+esi*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xb8", "mov dword ptr [eax+edi*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xb9", "mov dword ptr [ecx+edi*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xba", "mov dword ptr [edx+edi*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xbb", "mov dword ptr [ebx+edi*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xbc", "mov dword ptr [esp+edi*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xbe", "mov dword ptr [esi+edi*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xbf", "mov dword ptr [edi+edi*4], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xc0", "mov dword ptr [eax+eax*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xc1", "mov dword ptr [ecx+eax*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xc2", "mov dword ptr [edx+eax*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xc3", "mov dword ptr [ebx+eax*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xc4", "mov dword ptr [esp+eax*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xc6", "mov dword ptr [esi+eax*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xc7", "mov dword ptr [edi+eax*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xc8", "mov dword ptr [eax+ecx*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xc9", "mov dword ptr [ecx+ecx*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xca", "mov dword ptr [edx+ecx*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xcb", "mov dword ptr [ebx+ecx*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xcc", "mov dword ptr [esp+ecx*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xce", "mov dword ptr [esi+ecx*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xcf", "mov dword ptr [edi+ecx*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xd0", "mov dword ptr [eax+edx*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xd1", "mov dword ptr [ecx+edx*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xd2", "mov dword ptr [edx+edx*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xd3", "mov dword ptr [ebx+edx*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xd4", "mov dword ptr [esp+edx*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xd6", "mov dword ptr [esi+edx*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xd7", "mov dword ptr [edi+edx*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xd8", "mov dword ptr [eax+ebx*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xd9", "mov dword ptr [ecx+ebx*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xda", "mov dword ptr [edx+ebx*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xdb", "mov dword ptr [ebx+ebx*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xdc", "mov dword ptr [esp+ebx*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xde", "mov dword ptr [esi+ebx*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xdf", "mov dword ptr [edi+ebx*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xe8", "mov dword ptr [eax+ebp*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xe9", "mov dword ptr [ecx+ebp*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xea", "mov dword ptr [edx+ebp*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xeb", "mov dword ptr [ebx+ebp*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xec", "mov dword ptr [esp+ebp*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xee", "mov dword ptr [esi+ebp*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xef", "mov dword ptr [edi+ebp*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xf0", "mov dword ptr [eax+esi*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xf1", "mov dword ptr [ecx+esi*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xf2", "mov dword ptr [edx+esi*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xf3", "mov dword ptr [ebx+esi*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xf4", "mov dword ptr [esp+esi*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xf6", "mov dword ptr [esi+esi*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xf7", "mov dword ptr [edi+esi*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xf8", "mov dword ptr [eax+edi*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xf9", "mov dword ptr [ecx+edi*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xfa", "mov dword ptr [edx+edi*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xfb", "mov dword ptr [ebx+edi*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xfc", "mov dword ptr [esp+edi*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xfe", "mov dword ptr [esi+edi*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x3c\xff", "mov dword ptr [edi+edi*8], edi", insn32_t::FLAG_NONE},
	{3,"\x89\x45\x00", "mov dword ptr [ebp], eax", insn32_t::FLAG_NONE},
	{3,"\x89\x4d\x00", "mov dword ptr [ebp], ecx", insn32_t::FLAG_NONE},
	{3,"\x89\x55\x00", "mov dword ptr [ebp], edx", insn32_t::FLAG_NONE},
	{3,"\x89\x5d\x00", "mov dword ptr [ebp], ebx", insn32_t::FLAG_NONE},
	{3,"\x89\x65\x00", "mov dword ptr [ebp], esp", insn32_t::FLAG_NONE},
	{3,"\x89\x6d\x00", "mov dword ptr [ebp], ebp", insn32_t::FLAG_NONE},
	{3,"\x89\x75\x00", "mov dword ptr [ebp], esi", insn32_t::FLAG_NONE},
	{3,"\x89\x7d\x00", "mov dword ptr [ebp], edi", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x00", "mov eax, dword ptr [eax+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x01", "mov eax, dword ptr [ecx+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x02", "mov eax, dword ptr [edx+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x03", "mov eax, dword ptr [ebx+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x04", "mov eax, dword ptr [esp+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x06", "mov eax, dword ptr [esi+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x07", "mov eax, dword ptr [edi+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x08", "mov eax, dword ptr [eax+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x09", "mov eax, dword ptr [ecx+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x0a", "mov eax, dword ptr [edx+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x0b", "mov eax, dword ptr [ebx+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x0c", "mov eax, dword ptr [esp+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x0e", "mov eax, dword ptr [esi+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x0f", "mov eax, dword ptr [edi+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x10", "mov eax, dword ptr [eax+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x11", "mov eax, dword ptr [ecx+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x12", "mov eax, dword ptr [edx+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x13", "mov eax, dword ptr [ebx+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x14", "mov eax, dword ptr [esp+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x16", "mov eax, dword ptr [esi+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x17", "mov eax, dword ptr [edi+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x18", "mov eax, dword ptr [eax+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x19", "mov eax, dword ptr [ecx+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x1a", "mov eax, dword ptr [edx+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x1b", "mov eax, dword ptr [ebx+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x1c", "mov eax, dword ptr [esp+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x1e", "mov eax, dword ptr [esi+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x1f", "mov eax, dword ptr [edi+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x24", "mov eax, dword ptr [esp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x28", "mov eax, dword ptr [eax+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x29", "mov eax, dword ptr [ecx+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x2a", "mov eax, dword ptr [edx+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x2b", "mov eax, dword ptr [ebx+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x2c", "mov eax, dword ptr [esp+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x2e", "mov eax, dword ptr [esi+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x2f", "mov eax, dword ptr [edi+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x30", "mov eax, dword ptr [eax+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x31", "mov eax, dword ptr [ecx+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x32", "mov eax, dword ptr [edx+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x33", "mov eax, dword ptr [ebx+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x34", "mov eax, dword ptr [esp+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x36", "mov eax, dword ptr [esi+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x37", "mov eax, dword ptr [edi+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x38", "mov eax, dword ptr [eax+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x39", "mov eax, dword ptr [ecx+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x3a", "mov eax, dword ptr [edx+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x3b", "mov eax, dword ptr [ebx+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x3c", "mov eax, dword ptr [esp+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x3e", "mov eax, dword ptr [esi+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x3f", "mov eax, dword ptr [edi+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x40", "mov eax, dword ptr [eax+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x41", "mov eax, dword ptr [ecx+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x42", "mov eax, dword ptr [edx+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x43", "mov eax, dword ptr [ebx+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x44", "mov eax, dword ptr [esp+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x46", "mov eax, dword ptr [esi+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x47", "mov eax, dword ptr [edi+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x48", "mov eax, dword ptr [eax+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x49", "mov eax, dword ptr [ecx+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x4a", "mov eax, dword ptr [edx+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x4b", "mov eax, dword ptr [ebx+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x4c", "mov eax, dword ptr [esp+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x4e", "mov eax, dword ptr [esi+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x4f", "mov eax, dword ptr [edi+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x50", "mov eax, dword ptr [eax+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x51", "mov eax, dword ptr [ecx+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x52", "mov eax, dword ptr [edx+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x53", "mov eax, dword ptr [ebx+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x54", "mov eax, dword ptr [esp+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x56", "mov eax, dword ptr [esi+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x57", "mov eax, dword ptr [edi+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x58", "mov eax, dword ptr [eax+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x59", "mov eax, dword ptr [ecx+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x5a", "mov eax, dword ptr [edx+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x5b", "mov eax, dword ptr [ebx+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x5c", "mov eax, dword ptr [esp+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x5e", "mov eax, dword ptr [esi+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x5f", "mov eax, dword ptr [edi+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x68", "mov eax, dword ptr [eax+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x69", "mov eax, dword ptr [ecx+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x6a", "mov eax, dword ptr [edx+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x6b", "mov eax, dword ptr [ebx+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x6c", "mov eax, dword ptr [esp+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x6e", "mov eax, dword ptr [esi+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x6f", "mov eax, dword ptr [edi+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x70", "mov eax, dword ptr [eax+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x71", "mov eax, dword ptr [ecx+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x72", "mov eax, dword ptr [edx+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x73", "mov eax, dword ptr [ebx+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x74", "mov eax, dword ptr [esp+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x76", "mov eax, dword ptr [esi+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x77", "mov eax, dword ptr [edi+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x78", "mov eax, dword ptr [eax+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x79", "mov eax, dword ptr [ecx+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x7a", "mov eax, dword ptr [edx+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x7b", "mov eax, dword ptr [ebx+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x7c", "mov eax, dword ptr [esp+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x7e", "mov eax, dword ptr [esi+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x7f", "mov eax, dword ptr [edi+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x80", "mov eax, dword ptr [eax+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x81", "mov eax, dword ptr [ecx+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x82", "mov eax, dword ptr [edx+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x83", "mov eax, dword ptr [ebx+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x84", "mov eax, dword ptr [esp+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x86", "mov eax, dword ptr [esi+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x87", "mov eax, dword ptr [edi+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x88", "mov eax, dword ptr [eax+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x89", "mov eax, dword ptr [ecx+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x8a", "mov eax, dword ptr [edx+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x8b", "mov eax, dword ptr [ebx+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x8c", "mov eax, dword ptr [esp+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x8e", "mov eax, dword ptr [esi+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x8f", "mov eax, dword ptr [edi+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x90", "mov eax, dword ptr [eax+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x91", "mov eax, dword ptr [ecx+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x92", "mov eax, dword ptr [edx+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x93", "mov eax, dword ptr [ebx+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x94", "mov eax, dword ptr [esp+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x96", "mov eax, dword ptr [esi+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x97", "mov eax, dword ptr [edi+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x98", "mov eax, dword ptr [eax+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x99", "mov eax, dword ptr [ecx+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x9a", "mov eax, dword ptr [edx+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x9b", "mov eax, dword ptr [ebx+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x9c", "mov eax, dword ptr [esp+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x9e", "mov eax, dword ptr [esi+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\x9f", "mov eax, dword ptr [edi+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xa8", "mov eax, dword ptr [eax+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xa9", "mov eax, dword ptr [ecx+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xaa", "mov eax, dword ptr [edx+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xab", "mov eax, dword ptr [ebx+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xac", "mov eax, dword ptr [esp+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xae", "mov eax, dword ptr [esi+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xaf", "mov eax, dword ptr [edi+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xb0", "mov eax, dword ptr [eax+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xb1", "mov eax, dword ptr [ecx+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xb2", "mov eax, dword ptr [edx+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xb3", "mov eax, dword ptr [ebx+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xb4", "mov eax, dword ptr [esp+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xb6", "mov eax, dword ptr [esi+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xb7", "mov eax, dword ptr [edi+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xb8", "mov eax, dword ptr [eax+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xb9", "mov eax, dword ptr [ecx+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xba", "mov eax, dword ptr [edx+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xbb", "mov eax, dword ptr [ebx+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xbc", "mov eax, dword ptr [esp+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xbe", "mov eax, dword ptr [esi+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xbf", "mov eax, dword ptr [edi+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xc0", "mov eax, dword ptr [eax+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xc1", "mov eax, dword ptr [ecx+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xc2", "mov eax, dword ptr [edx+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xc3", "mov eax, dword ptr [ebx+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xc4", "mov eax, dword ptr [esp+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xc6", "mov eax, dword ptr [esi+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xc7", "mov eax, dword ptr [edi+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xc8", "mov eax, dword ptr [eax+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xc9", "mov eax, dword ptr [ecx+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xca", "mov eax, dword ptr [edx+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xcb", "mov eax, dword ptr [ebx+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xcc", "mov eax, dword ptr [esp+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xce", "mov eax, dword ptr [esi+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xcf", "mov eax, dword ptr [edi+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xd0", "mov eax, dword ptr [eax+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xd1", "mov eax, dword ptr [ecx+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xd2", "mov eax, dword ptr [edx+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xd3", "mov eax, dword ptr [ebx+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xd4", "mov eax, dword ptr [esp+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xd6", "mov eax, dword ptr [esi+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xd7", "mov eax, dword ptr [edi+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xd8", "mov eax, dword ptr [eax+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xd9", "mov eax, dword ptr [ecx+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xda", "mov eax, dword ptr [edx+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xdb", "mov eax, dword ptr [ebx+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xdc", "mov eax, dword ptr [esp+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xde", "mov eax, dword ptr [esi+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xdf", "mov eax, dword ptr [edi+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xe8", "mov eax, dword ptr [eax+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xe9", "mov eax, dword ptr [ecx+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xea", "mov eax, dword ptr [edx+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xeb", "mov eax, dword ptr [ebx+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xec", "mov eax, dword ptr [esp+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xee", "mov eax, dword ptr [esi+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xef", "mov eax, dword ptr [edi+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xf0", "mov eax, dword ptr [eax+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xf1", "mov eax, dword ptr [ecx+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xf2", "mov eax, dword ptr [edx+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xf3", "mov eax, dword ptr [ebx+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xf4", "mov eax, dword ptr [esp+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xf6", "mov eax, dword ptr [esi+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xf7", "mov eax, dword ptr [edi+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xf8", "mov eax, dword ptr [eax+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xf9", "mov eax, dword ptr [ecx+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xfa", "mov eax, dword ptr [edx+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xfb", "mov eax, dword ptr [ebx+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xfc", "mov eax, dword ptr [esp+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xfe", "mov eax, dword ptr [esi+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x04\xff", "mov eax, dword ptr [edi+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x00", "mov ecx, dword ptr [eax+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x01", "mov ecx, dword ptr [ecx+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x02", "mov ecx, dword ptr [edx+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x03", "mov ecx, dword ptr [ebx+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x04", "mov ecx, dword ptr [esp+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x06", "mov ecx, dword ptr [esi+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x07", "mov ecx, dword ptr [edi+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x08", "mov ecx, dword ptr [eax+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x09", "mov ecx, dword ptr [ecx+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x0a", "mov ecx, dword ptr [edx+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x0b", "mov ecx, dword ptr [ebx+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x0c", "mov ecx, dword ptr [esp+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x0e", "mov ecx, dword ptr [esi+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x0f", "mov ecx, dword ptr [edi+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x10", "mov ecx, dword ptr [eax+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x11", "mov ecx, dword ptr [ecx+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x12", "mov ecx, dword ptr [edx+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x13", "mov ecx, dword ptr [ebx+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x14", "mov ecx, dword ptr [esp+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x16", "mov ecx, dword ptr [esi+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x17", "mov ecx, dword ptr [edi+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x18", "mov ecx, dword ptr [eax+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x19", "mov ecx, dword ptr [ecx+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x1a", "mov ecx, dword ptr [edx+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x1b", "mov ecx, dword ptr [ebx+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x1c", "mov ecx, dword ptr [esp+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x1e", "mov ecx, dword ptr [esi+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x1f", "mov ecx, dword ptr [edi+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x24", "mov ecx, dword ptr [esp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x28", "mov ecx, dword ptr [eax+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x29", "mov ecx, dword ptr [ecx+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x2a", "mov ecx, dword ptr [edx+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x2b", "mov ecx, dword ptr [ebx+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x2c", "mov ecx, dword ptr [esp+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x2e", "mov ecx, dword ptr [esi+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x2f", "mov ecx, dword ptr [edi+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x30", "mov ecx, dword ptr [eax+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x31", "mov ecx, dword ptr [ecx+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x32", "mov ecx, dword ptr [edx+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x33", "mov ecx, dword ptr [ebx+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x34", "mov ecx, dword ptr [esp+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x36", "mov ecx, dword ptr [esi+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x37", "mov ecx, dword ptr [edi+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x38", "mov ecx, dword ptr [eax+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x39", "mov ecx, dword ptr [ecx+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x3a", "mov ecx, dword ptr [edx+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x3b", "mov ecx, dword ptr [ebx+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x3c", "mov ecx, dword ptr [esp+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x3e", "mov ecx, dword ptr [esi+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x3f", "mov ecx, dword ptr [edi+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x40", "mov ecx, dword ptr [eax+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x41", "mov ecx, dword ptr [ecx+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x42", "mov ecx, dword ptr [edx+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x43", "mov ecx, dword ptr [ebx+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x44", "mov ecx, dword ptr [esp+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x46", "mov ecx, dword ptr [esi+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x47", "mov ecx, dword ptr [edi+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x48", "mov ecx, dword ptr [eax+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x49", "mov ecx, dword ptr [ecx+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x4a", "mov ecx, dword ptr [edx+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x4b", "mov ecx, dword ptr [ebx+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x4c", "mov ecx, dword ptr [esp+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x4e", "mov ecx, dword ptr [esi+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x4f", "mov ecx, dword ptr [edi+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x50", "mov ecx, dword ptr [eax+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x51", "mov ecx, dword ptr [ecx+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x52", "mov ecx, dword ptr [edx+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x53", "mov ecx, dword ptr [ebx+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x54", "mov ecx, dword ptr [esp+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x56", "mov ecx, dword ptr [esi+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x57", "mov ecx, dword ptr [edi+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x58", "mov ecx, dword ptr [eax+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x59", "mov ecx, dword ptr [ecx+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x5a", "mov ecx, dword ptr [edx+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x5b", "mov ecx, dword ptr [ebx+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x5c", "mov ecx, dword ptr [esp+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x5e", "mov ecx, dword ptr [esi+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x5f", "mov ecx, dword ptr [edi+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x68", "mov ecx, dword ptr [eax+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x69", "mov ecx, dword ptr [ecx+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x6a", "mov ecx, dword ptr [edx+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x6b", "mov ecx, dword ptr [ebx+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x6c", "mov ecx, dword ptr [esp+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x6e", "mov ecx, dword ptr [esi+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x6f", "mov ecx, dword ptr [edi+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x70", "mov ecx, dword ptr [eax+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x71", "mov ecx, dword ptr [ecx+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x72", "mov ecx, dword ptr [edx+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x73", "mov ecx, dword ptr [ebx+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x74", "mov ecx, dword ptr [esp+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x76", "mov ecx, dword ptr [esi+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x77", "mov ecx, dword ptr [edi+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x78", "mov ecx, dword ptr [eax+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x79", "mov ecx, dword ptr [ecx+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x7a", "mov ecx, dword ptr [edx+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x7b", "mov ecx, dword ptr [ebx+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x7c", "mov ecx, dword ptr [esp+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x7e", "mov ecx, dword ptr [esi+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x7f", "mov ecx, dword ptr [edi+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x80", "mov ecx, dword ptr [eax+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x81", "mov ecx, dword ptr [ecx+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x82", "mov ecx, dword ptr [edx+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x83", "mov ecx, dword ptr [ebx+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x84", "mov ecx, dword ptr [esp+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x86", "mov ecx, dword ptr [esi+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x87", "mov ecx, dword ptr [edi+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x88", "mov ecx, dword ptr [eax+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x89", "mov ecx, dword ptr [ecx+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x8a", "mov ecx, dword ptr [edx+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x8b", "mov ecx, dword ptr [ebx+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x8c", "mov ecx, dword ptr [esp+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x8e", "mov ecx, dword ptr [esi+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x8f", "mov ecx, dword ptr [edi+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x90", "mov ecx, dword ptr [eax+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x91", "mov ecx, dword ptr [ecx+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x92", "mov ecx, dword ptr [edx+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x93", "mov ecx, dword ptr [ebx+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x94", "mov ecx, dword ptr [esp+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x96", "mov ecx, dword ptr [esi+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x97", "mov ecx, dword ptr [edi+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x98", "mov ecx, dword ptr [eax+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x99", "mov ecx, dword ptr [ecx+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x9a", "mov ecx, dword ptr [edx+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x9b", "mov ecx, dword ptr [ebx+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x9c", "mov ecx, dword ptr [esp+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x9e", "mov ecx, dword ptr [esi+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\x9f", "mov ecx, dword ptr [edi+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xa8", "mov ecx, dword ptr [eax+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xa9", "mov ecx, dword ptr [ecx+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xaa", "mov ecx, dword ptr [edx+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xab", "mov ecx, dword ptr [ebx+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xac", "mov ecx, dword ptr [esp+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xae", "mov ecx, dword ptr [esi+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xaf", "mov ecx, dword ptr [edi+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xb0", "mov ecx, dword ptr [eax+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xb1", "mov ecx, dword ptr [ecx+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xb2", "mov ecx, dword ptr [edx+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xb3", "mov ecx, dword ptr [ebx+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xb4", "mov ecx, dword ptr [esp+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xb6", "mov ecx, dword ptr [esi+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xb7", "mov ecx, dword ptr [edi+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xb8", "mov ecx, dword ptr [eax+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xb9", "mov ecx, dword ptr [ecx+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xba", "mov ecx, dword ptr [edx+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xbb", "mov ecx, dword ptr [ebx+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xbc", "mov ecx, dword ptr [esp+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xbe", "mov ecx, dword ptr [esi+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xbf", "mov ecx, dword ptr [edi+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xc0", "mov ecx, dword ptr [eax+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xc1", "mov ecx, dword ptr [ecx+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xc2", "mov ecx, dword ptr [edx+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xc3", "mov ecx, dword ptr [ebx+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xc4", "mov ecx, dword ptr [esp+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xc6", "mov ecx, dword ptr [esi+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xc7", "mov ecx, dword ptr [edi+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xc8", "mov ecx, dword ptr [eax+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xc9", "mov ecx, dword ptr [ecx+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xca", "mov ecx, dword ptr [edx+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xcb", "mov ecx, dword ptr [ebx+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xcc", "mov ecx, dword ptr [esp+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xce", "mov ecx, dword ptr [esi+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xcf", "mov ecx, dword ptr [edi+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xd0", "mov ecx, dword ptr [eax+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xd1", "mov ecx, dword ptr [ecx+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xd2", "mov ecx, dword ptr [edx+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xd3", "mov ecx, dword ptr [ebx+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xd4", "mov ecx, dword ptr [esp+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xd6", "mov ecx, dword ptr [esi+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xd7", "mov ecx, dword ptr [edi+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xd8", "mov ecx, dword ptr [eax+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xd9", "mov ecx, dword ptr [ecx+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xda", "mov ecx, dword ptr [edx+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xdb", "mov ecx, dword ptr [ebx+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xdc", "mov ecx, dword ptr [esp+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xde", "mov ecx, dword ptr [esi+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xdf", "mov ecx, dword ptr [edi+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xe8", "mov ecx, dword ptr [eax+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xe9", "mov ecx, dword ptr [ecx+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xea", "mov ecx, dword ptr [edx+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xeb", "mov ecx, dword ptr [ebx+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xec", "mov ecx, dword ptr [esp+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xee", "mov ecx, dword ptr [esi+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xef", "mov ecx, dword ptr [edi+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xf0", "mov ecx, dword ptr [eax+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xf1", "mov ecx, dword ptr [ecx+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xf2", "mov ecx, dword ptr [edx+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xf3", "mov ecx, dword ptr [ebx+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xf4", "mov ecx, dword ptr [esp+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xf6", "mov ecx, dword ptr [esi+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xf7", "mov ecx, dword ptr [edi+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xf8", "mov ecx, dword ptr [eax+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xf9", "mov ecx, dword ptr [ecx+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xfa", "mov ecx, dword ptr [edx+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xfb", "mov ecx, dword ptr [ebx+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xfc", "mov ecx, dword ptr [esp+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xfe", "mov ecx, dword ptr [esi+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x0c\xff", "mov ecx, dword ptr [edi+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x00", "mov edx, dword ptr [eax+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x01", "mov edx, dword ptr [ecx+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x02", "mov edx, dword ptr [edx+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x03", "mov edx, dword ptr [ebx+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x04", "mov edx, dword ptr [esp+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x06", "mov edx, dword ptr [esi+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x07", "mov edx, dword ptr [edi+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x08", "mov edx, dword ptr [eax+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x09", "mov edx, dword ptr [ecx+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x0a", "mov edx, dword ptr [edx+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x0b", "mov edx, dword ptr [ebx+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x0c", "mov edx, dword ptr [esp+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x0e", "mov edx, dword ptr [esi+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x0f", "mov edx, dword ptr [edi+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x10", "mov edx, dword ptr [eax+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x11", "mov edx, dword ptr [ecx+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x12", "mov edx, dword ptr [edx+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x13", "mov edx, dword ptr [ebx+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x14", "mov edx, dword ptr [esp+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x16", "mov edx, dword ptr [esi+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x17", "mov edx, dword ptr [edi+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x18", "mov edx, dword ptr [eax+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x19", "mov edx, dword ptr [ecx+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x1a", "mov edx, dword ptr [edx+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x1b", "mov edx, dword ptr [ebx+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x1c", "mov edx, dword ptr [esp+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x1e", "mov edx, dword ptr [esi+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x1f", "mov edx, dword ptr [edi+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x24", "mov edx, dword ptr [esp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x28", "mov edx, dword ptr [eax+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x29", "mov edx, dword ptr [ecx+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x2a", "mov edx, dword ptr [edx+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x2b", "mov edx, dword ptr [ebx+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x2c", "mov edx, dword ptr [esp+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x2e", "mov edx, dword ptr [esi+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x2f", "mov edx, dword ptr [edi+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x30", "mov edx, dword ptr [eax+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x31", "mov edx, dword ptr [ecx+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x32", "mov edx, dword ptr [edx+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x33", "mov edx, dword ptr [ebx+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x34", "mov edx, dword ptr [esp+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x36", "mov edx, dword ptr [esi+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x37", "mov edx, dword ptr [edi+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x38", "mov edx, dword ptr [eax+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x39", "mov edx, dword ptr [ecx+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x3a", "mov edx, dword ptr [edx+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x3b", "mov edx, dword ptr [ebx+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x3c", "mov edx, dword ptr [esp+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x3e", "mov edx, dword ptr [esi+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x3f", "mov edx, dword ptr [edi+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x40", "mov edx, dword ptr [eax+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x41", "mov edx, dword ptr [ecx+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x42", "mov edx, dword ptr [edx+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x43", "mov edx, dword ptr [ebx+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x44", "mov edx, dword ptr [esp+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x46", "mov edx, dword ptr [esi+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x47", "mov edx, dword ptr [edi+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x48", "mov edx, dword ptr [eax+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x49", "mov edx, dword ptr [ecx+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x4a", "mov edx, dword ptr [edx+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x4b", "mov edx, dword ptr [ebx+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x4c", "mov edx, dword ptr [esp+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x4e", "mov edx, dword ptr [esi+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x4f", "mov edx, dword ptr [edi+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x50", "mov edx, dword ptr [eax+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x51", "mov edx, dword ptr [ecx+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x52", "mov edx, dword ptr [edx+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x53", "mov edx, dword ptr [ebx+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x54", "mov edx, dword ptr [esp+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x56", "mov edx, dword ptr [esi+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x57", "mov edx, dword ptr [edi+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x58", "mov edx, dword ptr [eax+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x59", "mov edx, dword ptr [ecx+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x5a", "mov edx, dword ptr [edx+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x5b", "mov edx, dword ptr [ebx+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x5c", "mov edx, dword ptr [esp+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x5e", "mov edx, dword ptr [esi+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x5f", "mov edx, dword ptr [edi+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x68", "mov edx, dword ptr [eax+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x69", "mov edx, dword ptr [ecx+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x6a", "mov edx, dword ptr [edx+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x6b", "mov edx, dword ptr [ebx+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x6c", "mov edx, dword ptr [esp+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x6e", "mov edx, dword ptr [esi+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x6f", "mov edx, dword ptr [edi+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x70", "mov edx, dword ptr [eax+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x71", "mov edx, dword ptr [ecx+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x72", "mov edx, dword ptr [edx+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x73", "mov edx, dword ptr [ebx+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x74", "mov edx, dword ptr [esp+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x76", "mov edx, dword ptr [esi+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x77", "mov edx, dword ptr [edi+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x78", "mov edx, dword ptr [eax+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x79", "mov edx, dword ptr [ecx+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x7a", "mov edx, dword ptr [edx+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x7b", "mov edx, dword ptr [ebx+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x7c", "mov edx, dword ptr [esp+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x7e", "mov edx, dword ptr [esi+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x7f", "mov edx, dword ptr [edi+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x80", "mov edx, dword ptr [eax+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x81", "mov edx, dword ptr [ecx+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x82", "mov edx, dword ptr [edx+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x83", "mov edx, dword ptr [ebx+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x84", "mov edx, dword ptr [esp+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x86", "mov edx, dword ptr [esi+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x87", "mov edx, dword ptr [edi+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x88", "mov edx, dword ptr [eax+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x89", "mov edx, dword ptr [ecx+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x8a", "mov edx, dword ptr [edx+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x8b", "mov edx, dword ptr [ebx+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x8c", "mov edx, dword ptr [esp+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x8e", "mov edx, dword ptr [esi+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x8f", "mov edx, dword ptr [edi+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x90", "mov edx, dword ptr [eax+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x91", "mov edx, dword ptr [ecx+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x92", "mov edx, dword ptr [edx+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x93", "mov edx, dword ptr [ebx+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x94", "mov edx, dword ptr [esp+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x96", "mov edx, dword ptr [esi+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x97", "mov edx, dword ptr [edi+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x98", "mov edx, dword ptr [eax+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x99", "mov edx, dword ptr [ecx+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x9a", "mov edx, dword ptr [edx+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x9b", "mov edx, dword ptr [ebx+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x9c", "mov edx, dword ptr [esp+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x9e", "mov edx, dword ptr [esi+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\x9f", "mov edx, dword ptr [edi+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xa8", "mov edx, dword ptr [eax+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xa9", "mov edx, dword ptr [ecx+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xaa", "mov edx, dword ptr [edx+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xab", "mov edx, dword ptr [ebx+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xac", "mov edx, dword ptr [esp+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xae", "mov edx, dword ptr [esi+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xaf", "mov edx, dword ptr [edi+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xb0", "mov edx, dword ptr [eax+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xb1", "mov edx, dword ptr [ecx+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xb2", "mov edx, dword ptr [edx+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xb3", "mov edx, dword ptr [ebx+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xb4", "mov edx, dword ptr [esp+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xb6", "mov edx, dword ptr [esi+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xb7", "mov edx, dword ptr [edi+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xb8", "mov edx, dword ptr [eax+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xb9", "mov edx, dword ptr [ecx+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xba", "mov edx, dword ptr [edx+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xbb", "mov edx, dword ptr [ebx+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xbc", "mov edx, dword ptr [esp+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xbe", "mov edx, dword ptr [esi+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xbf", "mov edx, dword ptr [edi+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xc0", "mov edx, dword ptr [eax+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xc1", "mov edx, dword ptr [ecx+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xc2", "mov edx, dword ptr [edx+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xc3", "mov edx, dword ptr [ebx+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xc4", "mov edx, dword ptr [esp+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xc6", "mov edx, dword ptr [esi+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xc7", "mov edx, dword ptr [edi+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xc8", "mov edx, dword ptr [eax+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xc9", "mov edx, dword ptr [ecx+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xca", "mov edx, dword ptr [edx+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xcb", "mov edx, dword ptr [ebx+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xcc", "mov edx, dword ptr [esp+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xce", "mov edx, dword ptr [esi+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xcf", "mov edx, dword ptr [edi+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xd0", "mov edx, dword ptr [eax+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xd1", "mov edx, dword ptr [ecx+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xd2", "mov edx, dword ptr [edx+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xd3", "mov edx, dword ptr [ebx+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xd4", "mov edx, dword ptr [esp+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xd6", "mov edx, dword ptr [esi+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xd7", "mov edx, dword ptr [edi+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xd8", "mov edx, dword ptr [eax+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xd9", "mov edx, dword ptr [ecx+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xda", "mov edx, dword ptr [edx+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xdb", "mov edx, dword ptr [ebx+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xdc", "mov edx, dword ptr [esp+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xde", "mov edx, dword ptr [esi+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xdf", "mov edx, dword ptr [edi+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xe8", "mov edx, dword ptr [eax+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xe9", "mov edx, dword ptr [ecx+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xea", "mov edx, dword ptr [edx+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xeb", "mov edx, dword ptr [ebx+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xec", "mov edx, dword ptr [esp+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xee", "mov edx, dword ptr [esi+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xef", "mov edx, dword ptr [edi+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xf0", "mov edx, dword ptr [eax+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xf1", "mov edx, dword ptr [ecx+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xf2", "mov edx, dword ptr [edx+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xf3", "mov edx, dword ptr [ebx+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xf4", "mov edx, dword ptr [esp+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xf6", "mov edx, dword ptr [esi+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xf7", "mov edx, dword ptr [edi+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xf8", "mov edx, dword ptr [eax+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xf9", "mov edx, dword ptr [ecx+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xfa", "mov edx, dword ptr [edx+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xfb", "mov edx, dword ptr [ebx+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xfc", "mov edx, dword ptr [esp+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xfe", "mov edx, dword ptr [esi+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x14\xff", "mov edx, dword ptr [edi+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x00", "mov ebx, dword ptr [eax+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x01", "mov ebx, dword ptr [ecx+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x02", "mov ebx, dword ptr [edx+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x03", "mov ebx, dword ptr [ebx+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x04", "mov ebx, dword ptr [esp+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x06", "mov ebx, dword ptr [esi+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x07", "mov ebx, dword ptr [edi+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x08", "mov ebx, dword ptr [eax+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x09", "mov ebx, dword ptr [ecx+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x0a", "mov ebx, dword ptr [edx+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x0b", "mov ebx, dword ptr [ebx+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x0c", "mov ebx, dword ptr [esp+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x0e", "mov ebx, dword ptr [esi+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x0f", "mov ebx, dword ptr [edi+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x10", "mov ebx, dword ptr [eax+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x11", "mov ebx, dword ptr [ecx+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x12", "mov ebx, dword ptr [edx+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x13", "mov ebx, dword ptr [ebx+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x14", "mov ebx, dword ptr [esp+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x16", "mov ebx, dword ptr [esi+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x17", "mov ebx, dword ptr [edi+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x18", "mov ebx, dword ptr [eax+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x19", "mov ebx, dword ptr [ecx+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x1a", "mov ebx, dword ptr [edx+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x1b", "mov ebx, dword ptr [ebx+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x1c", "mov ebx, dword ptr [esp+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x1e", "mov ebx, dword ptr [esi+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x1f", "mov ebx, dword ptr [edi+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x24", "mov ebx, dword ptr [esp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x28", "mov ebx, dword ptr [eax+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x29", "mov ebx, dword ptr [ecx+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x2a", "mov ebx, dword ptr [edx+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x2b", "mov ebx, dword ptr [ebx+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x2c", "mov ebx, dword ptr [esp+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x2e", "mov ebx, dword ptr [esi+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x2f", "mov ebx, dword ptr [edi+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x30", "mov ebx, dword ptr [eax+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x31", "mov ebx, dword ptr [ecx+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x32", "mov ebx, dword ptr [edx+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x33", "mov ebx, dword ptr [ebx+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x34", "mov ebx, dword ptr [esp+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x36", "mov ebx, dword ptr [esi+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x37", "mov ebx, dword ptr [edi+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x38", "mov ebx, dword ptr [eax+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x39", "mov ebx, dword ptr [ecx+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x3a", "mov ebx, dword ptr [edx+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x3b", "mov ebx, dword ptr [ebx+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x3c", "mov ebx, dword ptr [esp+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x3e", "mov ebx, dword ptr [esi+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x3f", "mov ebx, dword ptr [edi+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x40", "mov ebx, dword ptr [eax+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x41", "mov ebx, dword ptr [ecx+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x42", "mov ebx, dword ptr [edx+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x43", "mov ebx, dword ptr [ebx+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x44", "mov ebx, dword ptr [esp+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x46", "mov ebx, dword ptr [esi+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x47", "mov ebx, dword ptr [edi+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x48", "mov ebx, dword ptr [eax+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x49", "mov ebx, dword ptr [ecx+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x4a", "mov ebx, dword ptr [edx+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x4b", "mov ebx, dword ptr [ebx+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x4c", "mov ebx, dword ptr [esp+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x4e", "mov ebx, dword ptr [esi+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x4f", "mov ebx, dword ptr [edi+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x50", "mov ebx, dword ptr [eax+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x51", "mov ebx, dword ptr [ecx+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x52", "mov ebx, dword ptr [edx+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x53", "mov ebx, dword ptr [ebx+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x54", "mov ebx, dword ptr [esp+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x56", "mov ebx, dword ptr [esi+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x57", "mov ebx, dword ptr [edi+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x58", "mov ebx, dword ptr [eax+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x59", "mov ebx, dword ptr [ecx+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x5a", "mov ebx, dword ptr [edx+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x5b", "mov ebx, dword ptr [ebx+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x5c", "mov ebx, dword ptr [esp+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x5e", "mov ebx, dword ptr [esi+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x5f", "mov ebx, dword ptr [edi+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x68", "mov ebx, dword ptr [eax+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x69", "mov ebx, dword ptr [ecx+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x6a", "mov ebx, dword ptr [edx+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x6b", "mov ebx, dword ptr [ebx+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x6c", "mov ebx, dword ptr [esp+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x6e", "mov ebx, dword ptr [esi+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x6f", "mov ebx, dword ptr [edi+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x70", "mov ebx, dword ptr [eax+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x71", "mov ebx, dword ptr [ecx+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x72", "mov ebx, dword ptr [edx+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x73", "mov ebx, dword ptr [ebx+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x74", "mov ebx, dword ptr [esp+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x76", "mov ebx, dword ptr [esi+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x77", "mov ebx, dword ptr [edi+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x78", "mov ebx, dword ptr [eax+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x79", "mov ebx, dword ptr [ecx+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x7a", "mov ebx, dword ptr [edx+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x7b", "mov ebx, dword ptr [ebx+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x7c", "mov ebx, dword ptr [esp+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x7e", "mov ebx, dword ptr [esi+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x7f", "mov ebx, dword ptr [edi+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x80", "mov ebx, dword ptr [eax+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x81", "mov ebx, dword ptr [ecx+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x82", "mov ebx, dword ptr [edx+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x83", "mov ebx, dword ptr [ebx+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x84", "mov ebx, dword ptr [esp+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x86", "mov ebx, dword ptr [esi+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x87", "mov ebx, dword ptr [edi+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x88", "mov ebx, dword ptr [eax+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x89", "mov ebx, dword ptr [ecx+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x8a", "mov ebx, dword ptr [edx+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x8b", "mov ebx, dword ptr [ebx+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x8c", "mov ebx, dword ptr [esp+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x8e", "mov ebx, dword ptr [esi+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x8f", "mov ebx, dword ptr [edi+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x90", "mov ebx, dword ptr [eax+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x91", "mov ebx, dword ptr [ecx+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x92", "mov ebx, dword ptr [edx+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x93", "mov ebx, dword ptr [ebx+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x94", "mov ebx, dword ptr [esp+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x96", "mov ebx, dword ptr [esi+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x97", "mov ebx, dword ptr [edi+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x98", "mov ebx, dword ptr [eax+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x99", "mov ebx, dword ptr [ecx+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x9a", "mov ebx, dword ptr [edx+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x9b", "mov ebx, dword ptr [ebx+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x9c", "mov ebx, dword ptr [esp+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x9e", "mov ebx, dword ptr [esi+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\x9f", "mov ebx, dword ptr [edi+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xa8", "mov ebx, dword ptr [eax+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xa9", "mov ebx, dword ptr [ecx+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xaa", "mov ebx, dword ptr [edx+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xab", "mov ebx, dword ptr [ebx+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xac", "mov ebx, dword ptr [esp+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xae", "mov ebx, dword ptr [esi+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xaf", "mov ebx, dword ptr [edi+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xb0", "mov ebx, dword ptr [eax+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xb1", "mov ebx, dword ptr [ecx+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xb2", "mov ebx, dword ptr [edx+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xb3", "mov ebx, dword ptr [ebx+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xb4", "mov ebx, dword ptr [esp+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xb6", "mov ebx, dword ptr [esi+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xb7", "mov ebx, dword ptr [edi+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xb8", "mov ebx, dword ptr [eax+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xb9", "mov ebx, dword ptr [ecx+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xba", "mov ebx, dword ptr [edx+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xbb", "mov ebx, dword ptr [ebx+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xbc", "mov ebx, dword ptr [esp+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xbe", "mov ebx, dword ptr [esi+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xbf", "mov ebx, dword ptr [edi+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xc0", "mov ebx, dword ptr [eax+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xc1", "mov ebx, dword ptr [ecx+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xc2", "mov ebx, dword ptr [edx+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xc3", "mov ebx, dword ptr [ebx+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xc4", "mov ebx, dword ptr [esp+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xc6", "mov ebx, dword ptr [esi+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xc7", "mov ebx, dword ptr [edi+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xc8", "mov ebx, dword ptr [eax+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xc9", "mov ebx, dword ptr [ecx+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xca", "mov ebx, dword ptr [edx+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xcb", "mov ebx, dword ptr [ebx+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xcc", "mov ebx, dword ptr [esp+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xce", "mov ebx, dword ptr [esi+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xcf", "mov ebx, dword ptr [edi+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xd0", "mov ebx, dword ptr [eax+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xd1", "mov ebx, dword ptr [ecx+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xd2", "mov ebx, dword ptr [edx+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xd3", "mov ebx, dword ptr [ebx+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xd4", "mov ebx, dword ptr [esp+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xd6", "mov ebx, dword ptr [esi+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xd7", "mov ebx, dword ptr [edi+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xd8", "mov ebx, dword ptr [eax+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xd9", "mov ebx, dword ptr [ecx+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xda", "mov ebx, dword ptr [edx+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xdb", "mov ebx, dword ptr [ebx+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xdc", "mov ebx, dword ptr [esp+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xde", "mov ebx, dword ptr [esi+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xdf", "mov ebx, dword ptr [edi+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xe8", "mov ebx, dword ptr [eax+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xe9", "mov ebx, dword ptr [ecx+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xea", "mov ebx, dword ptr [edx+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xeb", "mov ebx, dword ptr [ebx+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xec", "mov ebx, dword ptr [esp+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xee", "mov ebx, dword ptr [esi+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xef", "mov ebx, dword ptr [edi+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xf0", "mov ebx, dword ptr [eax+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xf1", "mov ebx, dword ptr [ecx+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xf2", "mov ebx, dword ptr [edx+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xf3", "mov ebx, dword ptr [ebx+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xf4", "mov ebx, dword ptr [esp+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xf6", "mov ebx, dword ptr [esi+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xf7", "mov ebx, dword ptr [edi+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xf8", "mov ebx, dword ptr [eax+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xf9", "mov ebx, dword ptr [ecx+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xfa", "mov ebx, dword ptr [edx+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xfb", "mov ebx, dword ptr [ebx+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xfc", "mov ebx, dword ptr [esp+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xfe", "mov ebx, dword ptr [esi+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x1c\xff", "mov ebx, dword ptr [edi+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x00", "mov esp, dword ptr [eax+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x01", "mov esp, dword ptr [ecx+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x02", "mov esp, dword ptr [edx+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x03", "mov esp, dword ptr [ebx+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x04", "mov esp, dword ptr [esp+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x06", "mov esp, dword ptr [esi+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x07", "mov esp, dword ptr [edi+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x08", "mov esp, dword ptr [eax+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x09", "mov esp, dword ptr [ecx+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x0a", "mov esp, dword ptr [edx+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x0b", "mov esp, dword ptr [ebx+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x0c", "mov esp, dword ptr [esp+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x0e", "mov esp, dword ptr [esi+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x0f", "mov esp, dword ptr [edi+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x10", "mov esp, dword ptr [eax+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x11", "mov esp, dword ptr [ecx+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x12", "mov esp, dword ptr [edx+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x13", "mov esp, dword ptr [ebx+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x14", "mov esp, dword ptr [esp+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x16", "mov esp, dword ptr [esi+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x17", "mov esp, dword ptr [edi+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x18", "mov esp, dword ptr [eax+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x19", "mov esp, dword ptr [ecx+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x1a", "mov esp, dword ptr [edx+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x1b", "mov esp, dword ptr [ebx+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x1c", "mov esp, dword ptr [esp+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x1e", "mov esp, dword ptr [esi+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x1f", "mov esp, dword ptr [edi+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x24", "mov esp, dword ptr [esp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x28", "mov esp, dword ptr [eax+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x29", "mov esp, dword ptr [ecx+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x2a", "mov esp, dword ptr [edx+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x2b", "mov esp, dword ptr [ebx+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x2c", "mov esp, dword ptr [esp+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x2e", "mov esp, dword ptr [esi+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x2f", "mov esp, dword ptr [edi+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x30", "mov esp, dword ptr [eax+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x31", "mov esp, dword ptr [ecx+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x32", "mov esp, dword ptr [edx+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x33", "mov esp, dword ptr [ebx+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x34", "mov esp, dword ptr [esp+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x36", "mov esp, dword ptr [esi+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x37", "mov esp, dword ptr [edi+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x38", "mov esp, dword ptr [eax+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x39", "mov esp, dword ptr [ecx+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x3a", "mov esp, dword ptr [edx+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x3b", "mov esp, dword ptr [ebx+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x3c", "mov esp, dword ptr [esp+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x3e", "mov esp, dword ptr [esi+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x3f", "mov esp, dword ptr [edi+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x40", "mov esp, dword ptr [eax+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x41", "mov esp, dword ptr [ecx+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x42", "mov esp, dword ptr [edx+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x43", "mov esp, dword ptr [ebx+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x44", "mov esp, dword ptr [esp+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x46", "mov esp, dword ptr [esi+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x47", "mov esp, dword ptr [edi+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x48", "mov esp, dword ptr [eax+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x49", "mov esp, dword ptr [ecx+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x4a", "mov esp, dword ptr [edx+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x4b", "mov esp, dword ptr [ebx+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x4c", "mov esp, dword ptr [esp+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x4e", "mov esp, dword ptr [esi+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x4f", "mov esp, dword ptr [edi+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x50", "mov esp, dword ptr [eax+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x51", "mov esp, dword ptr [ecx+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x52", "mov esp, dword ptr [edx+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x53", "mov esp, dword ptr [ebx+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x54", "mov esp, dword ptr [esp+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x56", "mov esp, dword ptr [esi+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x57", "mov esp, dword ptr [edi+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x58", "mov esp, dword ptr [eax+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x59", "mov esp, dword ptr [ecx+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x5a", "mov esp, dword ptr [edx+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x5b", "mov esp, dword ptr [ebx+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x5c", "mov esp, dword ptr [esp+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x5e", "mov esp, dword ptr [esi+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x5f", "mov esp, dword ptr [edi+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x68", "mov esp, dword ptr [eax+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x69", "mov esp, dword ptr [ecx+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x6a", "mov esp, dword ptr [edx+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x6b", "mov esp, dword ptr [ebx+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x6c", "mov esp, dword ptr [esp+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x6e", "mov esp, dword ptr [esi+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x6f", "mov esp, dword ptr [edi+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x70", "mov esp, dword ptr [eax+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x71", "mov esp, dword ptr [ecx+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x72", "mov esp, dword ptr [edx+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x73", "mov esp, dword ptr [ebx+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x74", "mov esp, dword ptr [esp+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x76", "mov esp, dword ptr [esi+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x77", "mov esp, dword ptr [edi+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x78", "mov esp, dword ptr [eax+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x79", "mov esp, dword ptr [ecx+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x7a", "mov esp, dword ptr [edx+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x7b", "mov esp, dword ptr [ebx+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x7c", "mov esp, dword ptr [esp+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x7e", "mov esp, dword ptr [esi+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x7f", "mov esp, dword ptr [edi+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x80", "mov esp, dword ptr [eax+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x81", "mov esp, dword ptr [ecx+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x82", "mov esp, dword ptr [edx+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x83", "mov esp, dword ptr [ebx+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x84", "mov esp, dword ptr [esp+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x86", "mov esp, dword ptr [esi+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x87", "mov esp, dword ptr [edi+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x88", "mov esp, dword ptr [eax+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x89", "mov esp, dword ptr [ecx+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x8a", "mov esp, dword ptr [edx+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x8b", "mov esp, dword ptr [ebx+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x8c", "mov esp, dword ptr [esp+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x8e", "mov esp, dword ptr [esi+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x8f", "mov esp, dword ptr [edi+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x90", "mov esp, dword ptr [eax+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x91", "mov esp, dword ptr [ecx+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x92", "mov esp, dword ptr [edx+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x93", "mov esp, dword ptr [ebx+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x94", "mov esp, dword ptr [esp+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x96", "mov esp, dword ptr [esi+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x97", "mov esp, dword ptr [edi+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x98", "mov esp, dword ptr [eax+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x99", "mov esp, dword ptr [ecx+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x9a", "mov esp, dword ptr [edx+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x9b", "mov esp, dword ptr [ebx+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x9c", "mov esp, dword ptr [esp+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x9e", "mov esp, dword ptr [esi+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\x9f", "mov esp, dword ptr [edi+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xa8", "mov esp, dword ptr [eax+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xa9", "mov esp, dword ptr [ecx+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xaa", "mov esp, dword ptr [edx+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xab", "mov esp, dword ptr [ebx+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xac", "mov esp, dword ptr [esp+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xae", "mov esp, dword ptr [esi+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xaf", "mov esp, dword ptr [edi+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xb0", "mov esp, dword ptr [eax+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xb1", "mov esp, dword ptr [ecx+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xb2", "mov esp, dword ptr [edx+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xb3", "mov esp, dword ptr [ebx+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xb4", "mov esp, dword ptr [esp+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xb6", "mov esp, dword ptr [esi+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xb7", "mov esp, dword ptr [edi+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xb8", "mov esp, dword ptr [eax+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xb9", "mov esp, dword ptr [ecx+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xba", "mov esp, dword ptr [edx+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xbb", "mov esp, dword ptr [ebx+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xbc", "mov esp, dword ptr [esp+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xbe", "mov esp, dword ptr [esi+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xbf", "mov esp, dword ptr [edi+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xc0", "mov esp, dword ptr [eax+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xc1", "mov esp, dword ptr [ecx+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xc2", "mov esp, dword ptr [edx+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xc3", "mov esp, dword ptr [ebx+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xc4", "mov esp, dword ptr [esp+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xc6", "mov esp, dword ptr [esi+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xc7", "mov esp, dword ptr [edi+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xc8", "mov esp, dword ptr [eax+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xc9", "mov esp, dword ptr [ecx+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xca", "mov esp, dword ptr [edx+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xcb", "mov esp, dword ptr [ebx+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xcc", "mov esp, dword ptr [esp+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xce", "mov esp, dword ptr [esi+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xcf", "mov esp, dword ptr [edi+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xd0", "mov esp, dword ptr [eax+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xd1", "mov esp, dword ptr [ecx+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xd2", "mov esp, dword ptr [edx+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xd3", "mov esp, dword ptr [ebx+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xd4", "mov esp, dword ptr [esp+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xd6", "mov esp, dword ptr [esi+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xd7", "mov esp, dword ptr [edi+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xd8", "mov esp, dword ptr [eax+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xd9", "mov esp, dword ptr [ecx+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xda", "mov esp, dword ptr [edx+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xdb", "mov esp, dword ptr [ebx+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xdc", "mov esp, dword ptr [esp+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xde", "mov esp, dword ptr [esi+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xdf", "mov esp, dword ptr [edi+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xe8", "mov esp, dword ptr [eax+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xe9", "mov esp, dword ptr [ecx+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xea", "mov esp, dword ptr [edx+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xeb", "mov esp, dword ptr [ebx+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xec", "mov esp, dword ptr [esp+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xee", "mov esp, dword ptr [esi+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xef", "mov esp, dword ptr [edi+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xf0", "mov esp, dword ptr [eax+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xf1", "mov esp, dword ptr [ecx+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xf2", "mov esp, dword ptr [edx+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xf3", "mov esp, dword ptr [ebx+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xf4", "mov esp, dword ptr [esp+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xf6", "mov esp, dword ptr [esi+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xf7", "mov esp, dword ptr [edi+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xf8", "mov esp, dword ptr [eax+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xf9", "mov esp, dword ptr [ecx+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xfa", "mov esp, dword ptr [edx+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xfb", "mov esp, dword ptr [ebx+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xfc", "mov esp, dword ptr [esp+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xfe", "mov esp, dword ptr [esi+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x24\xff", "mov esp, dword ptr [edi+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x00", "mov ebp, dword ptr [eax+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x01", "mov ebp, dword ptr [ecx+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x02", "mov ebp, dword ptr [edx+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x03", "mov ebp, dword ptr [ebx+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x04", "mov ebp, dword ptr [esp+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x06", "mov ebp, dword ptr [esi+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x07", "mov ebp, dword ptr [edi+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x08", "mov ebp, dword ptr [eax+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x09", "mov ebp, dword ptr [ecx+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x0a", "mov ebp, dword ptr [edx+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x0b", "mov ebp, dword ptr [ebx+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x0c", "mov ebp, dword ptr [esp+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x0e", "mov ebp, dword ptr [esi+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x0f", "mov ebp, dword ptr [edi+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x10", "mov ebp, dword ptr [eax+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x11", "mov ebp, dword ptr [ecx+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x12", "mov ebp, dword ptr [edx+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x13", "mov ebp, dword ptr [ebx+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x14", "mov ebp, dword ptr [esp+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x16", "mov ebp, dword ptr [esi+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x17", "mov ebp, dword ptr [edi+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x18", "mov ebp, dword ptr [eax+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x19", "mov ebp, dword ptr [ecx+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x1a", "mov ebp, dword ptr [edx+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x1b", "mov ebp, dword ptr [ebx+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x1c", "mov ebp, dword ptr [esp+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x1e", "mov ebp, dword ptr [esi+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x1f", "mov ebp, dword ptr [edi+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x24", "mov ebp, dword ptr [esp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x28", "mov ebp, dword ptr [eax+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x29", "mov ebp, dword ptr [ecx+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x2a", "mov ebp, dword ptr [edx+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x2b", "mov ebp, dword ptr [ebx+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x2c", "mov ebp, dword ptr [esp+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x2e", "mov ebp, dword ptr [esi+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x2f", "mov ebp, dword ptr [edi+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x30", "mov ebp, dword ptr [eax+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x31", "mov ebp, dword ptr [ecx+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x32", "mov ebp, dword ptr [edx+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x33", "mov ebp, dword ptr [ebx+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x34", "mov ebp, dword ptr [esp+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x36", "mov ebp, dword ptr [esi+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x37", "mov ebp, dword ptr [edi+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x38", "mov ebp, dword ptr [eax+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x39", "mov ebp, dword ptr [ecx+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x3a", "mov ebp, dword ptr [edx+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x3b", "mov ebp, dword ptr [ebx+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x3c", "mov ebp, dword ptr [esp+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x3e", "mov ebp, dword ptr [esi+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x3f", "mov ebp, dword ptr [edi+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x40", "mov ebp, dword ptr [eax+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x41", "mov ebp, dword ptr [ecx+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x42", "mov ebp, dword ptr [edx+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x43", "mov ebp, dword ptr [ebx+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x44", "mov ebp, dword ptr [esp+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x46", "mov ebp, dword ptr [esi+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x47", "mov ebp, dword ptr [edi+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x48", "mov ebp, dword ptr [eax+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x49", "mov ebp, dword ptr [ecx+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x4a", "mov ebp, dword ptr [edx+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x4b", "mov ebp, dword ptr [ebx+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x4c", "mov ebp, dword ptr [esp+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x4e", "mov ebp, dword ptr [esi+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x4f", "mov ebp, dword ptr [edi+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x50", "mov ebp, dword ptr [eax+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x51", "mov ebp, dword ptr [ecx+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x52", "mov ebp, dword ptr [edx+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x53", "mov ebp, dword ptr [ebx+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x54", "mov ebp, dword ptr [esp+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x56", "mov ebp, dword ptr [esi+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x57", "mov ebp, dword ptr [edi+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x58", "mov ebp, dword ptr [eax+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x59", "mov ebp, dword ptr [ecx+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x5a", "mov ebp, dword ptr [edx+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x5b", "mov ebp, dword ptr [ebx+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x5c", "mov ebp, dword ptr [esp+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x5e", "mov ebp, dword ptr [esi+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x5f", "mov ebp, dword ptr [edi+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x68", "mov ebp, dword ptr [eax+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x69", "mov ebp, dword ptr [ecx+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x6a", "mov ebp, dword ptr [edx+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x6b", "mov ebp, dword ptr [ebx+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x6c", "mov ebp, dword ptr [esp+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x6e", "mov ebp, dword ptr [esi+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x6f", "mov ebp, dword ptr [edi+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x70", "mov ebp, dword ptr [eax+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x71", "mov ebp, dword ptr [ecx+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x72", "mov ebp, dword ptr [edx+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x73", "mov ebp, dword ptr [ebx+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x74", "mov ebp, dword ptr [esp+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x76", "mov ebp, dword ptr [esi+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x77", "mov ebp, dword ptr [edi+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x78", "mov ebp, dword ptr [eax+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x79", "mov ebp, dword ptr [ecx+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x7a", "mov ebp, dword ptr [edx+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x7b", "mov ebp, dword ptr [ebx+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x7c", "mov ebp, dword ptr [esp+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x7e", "mov ebp, dword ptr [esi+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x7f", "mov ebp, dword ptr [edi+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x80", "mov ebp, dword ptr [eax+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x81", "mov ebp, dword ptr [ecx+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x82", "mov ebp, dword ptr [edx+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x83", "mov ebp, dword ptr [ebx+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x84", "mov ebp, dword ptr [esp+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x86", "mov ebp, dword ptr [esi+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x87", "mov ebp, dword ptr [edi+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x88", "mov ebp, dword ptr [eax+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x89", "mov ebp, dword ptr [ecx+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x8a", "mov ebp, dword ptr [edx+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x8b", "mov ebp, dword ptr [ebx+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x8c", "mov ebp, dword ptr [esp+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x8e", "mov ebp, dword ptr [esi+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x8f", "mov ebp, dword ptr [edi+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x90", "mov ebp, dword ptr [eax+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x91", "mov ebp, dword ptr [ecx+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x92", "mov ebp, dword ptr [edx+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x93", "mov ebp, dword ptr [ebx+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x94", "mov ebp, dword ptr [esp+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x96", "mov ebp, dword ptr [esi+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x97", "mov ebp, dword ptr [edi+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x98", "mov ebp, dword ptr [eax+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x99", "mov ebp, dword ptr [ecx+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x9a", "mov ebp, dword ptr [edx+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x9b", "mov ebp, dword ptr [ebx+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x9c", "mov ebp, dword ptr [esp+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x9e", "mov ebp, dword ptr [esi+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\x9f", "mov ebp, dword ptr [edi+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xa8", "mov ebp, dword ptr [eax+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xa9", "mov ebp, dword ptr [ecx+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xaa", "mov ebp, dword ptr [edx+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xab", "mov ebp, dword ptr [ebx+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xac", "mov ebp, dword ptr [esp+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xae", "mov ebp, dword ptr [esi+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xaf", "mov ebp, dword ptr [edi+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xb0", "mov ebp, dword ptr [eax+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xb1", "mov ebp, dword ptr [ecx+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xb2", "mov ebp, dword ptr [edx+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xb3", "mov ebp, dword ptr [ebx+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xb4", "mov ebp, dword ptr [esp+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xb6", "mov ebp, dword ptr [esi+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xb7", "mov ebp, dword ptr [edi+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xb8", "mov ebp, dword ptr [eax+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xb9", "mov ebp, dword ptr [ecx+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xba", "mov ebp, dword ptr [edx+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xbb", "mov ebp, dword ptr [ebx+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xbc", "mov ebp, dword ptr [esp+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xbe", "mov ebp, dword ptr [esi+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xbf", "mov ebp, dword ptr [edi+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xc0", "mov ebp, dword ptr [eax+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xc1", "mov ebp, dword ptr [ecx+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xc2", "mov ebp, dword ptr [edx+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xc3", "mov ebp, dword ptr [ebx+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xc4", "mov ebp, dword ptr [esp+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xc6", "mov ebp, dword ptr [esi+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xc7", "mov ebp, dword ptr [edi+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xc8", "mov ebp, dword ptr [eax+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xc9", "mov ebp, dword ptr [ecx+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xca", "mov ebp, dword ptr [edx+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xcb", "mov ebp, dword ptr [ebx+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xcc", "mov ebp, dword ptr [esp+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xce", "mov ebp, dword ptr [esi+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xcf", "mov ebp, dword ptr [edi+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xd0", "mov ebp, dword ptr [eax+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xd1", "mov ebp, dword ptr [ecx+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xd2", "mov ebp, dword ptr [edx+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xd3", "mov ebp, dword ptr [ebx+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xd4", "mov ebp, dword ptr [esp+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xd6", "mov ebp, dword ptr [esi+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xd7", "mov ebp, dword ptr [edi+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xd8", "mov ebp, dword ptr [eax+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xd9", "mov ebp, dword ptr [ecx+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xda", "mov ebp, dword ptr [edx+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xdb", "mov ebp, dword ptr [ebx+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xdc", "mov ebp, dword ptr [esp+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xde", "mov ebp, dword ptr [esi+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xdf", "mov ebp, dword ptr [edi+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xe8", "mov ebp, dword ptr [eax+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xe9", "mov ebp, dword ptr [ecx+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xea", "mov ebp, dword ptr [edx+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xeb", "mov ebp, dword ptr [ebx+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xec", "mov ebp, dword ptr [esp+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xee", "mov ebp, dword ptr [esi+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xef", "mov ebp, dword ptr [edi+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xf0", "mov ebp, dword ptr [eax+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xf1", "mov ebp, dword ptr [ecx+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xf2", "mov ebp, dword ptr [edx+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xf3", "mov ebp, dword ptr [ebx+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xf4", "mov ebp, dword ptr [esp+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xf6", "mov ebp, dword ptr [esi+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xf7", "mov ebp, dword ptr [edi+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xf8", "mov ebp, dword ptr [eax+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xf9", "mov ebp, dword ptr [ecx+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xfa", "mov ebp, dword ptr [edx+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xfb", "mov ebp, dword ptr [ebx+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xfc", "mov ebp, dword ptr [esp+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xfe", "mov ebp, dword ptr [esi+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x2c\xff", "mov ebp, dword ptr [edi+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x00", "mov esi, dword ptr [eax+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x01", "mov esi, dword ptr [ecx+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x02", "mov esi, dword ptr [edx+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x03", "mov esi, dword ptr [ebx+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x04", "mov esi, dword ptr [esp+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x06", "mov esi, dword ptr [esi+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x07", "mov esi, dword ptr [edi+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x08", "mov esi, dword ptr [eax+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x09", "mov esi, dword ptr [ecx+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x0a", "mov esi, dword ptr [edx+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x0b", "mov esi, dword ptr [ebx+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x0c", "mov esi, dword ptr [esp+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x0e", "mov esi, dword ptr [esi+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x0f", "mov esi, dword ptr [edi+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x10", "mov esi, dword ptr [eax+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x11", "mov esi, dword ptr [ecx+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x12", "mov esi, dword ptr [edx+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x13", "mov esi, dword ptr [ebx+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x14", "mov esi, dword ptr [esp+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x16", "mov esi, dword ptr [esi+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x17", "mov esi, dword ptr [edi+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x18", "mov esi, dword ptr [eax+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x19", "mov esi, dword ptr [ecx+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x1a", "mov esi, dword ptr [edx+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x1b", "mov esi, dword ptr [ebx+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x1c", "mov esi, dword ptr [esp+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x1e", "mov esi, dword ptr [esi+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x1f", "mov esi, dword ptr [edi+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x24", "mov esi, dword ptr [esp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x28", "mov esi, dword ptr [eax+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x29", "mov esi, dword ptr [ecx+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x2a", "mov esi, dword ptr [edx+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x2b", "mov esi, dword ptr [ebx+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x2c", "mov esi, dword ptr [esp+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x2e", "mov esi, dword ptr [esi+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x2f", "mov esi, dword ptr [edi+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x30", "mov esi, dword ptr [eax+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x31", "mov esi, dword ptr [ecx+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x32", "mov esi, dword ptr [edx+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x33", "mov esi, dword ptr [ebx+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x34", "mov esi, dword ptr [esp+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x36", "mov esi, dword ptr [esi+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x37", "mov esi, dword ptr [edi+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x38", "mov esi, dword ptr [eax+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x39", "mov esi, dword ptr [ecx+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x3a", "mov esi, dword ptr [edx+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x3b", "mov esi, dword ptr [ebx+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x3c", "mov esi, dword ptr [esp+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x3e", "mov esi, dword ptr [esi+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x3f", "mov esi, dword ptr [edi+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x40", "mov esi, dword ptr [eax+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x41", "mov esi, dword ptr [ecx+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x42", "mov esi, dword ptr [edx+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x43", "mov esi, dword ptr [ebx+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x44", "mov esi, dword ptr [esp+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x46", "mov esi, dword ptr [esi+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x47", "mov esi, dword ptr [edi+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x48", "mov esi, dword ptr [eax+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x49", "mov esi, dword ptr [ecx+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x4a", "mov esi, dword ptr [edx+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x4b", "mov esi, dword ptr [ebx+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x4c", "mov esi, dword ptr [esp+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x4e", "mov esi, dword ptr [esi+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x4f", "mov esi, dword ptr [edi+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x50", "mov esi, dword ptr [eax+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x51", "mov esi, dword ptr [ecx+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x52", "mov esi, dword ptr [edx+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x53", "mov esi, dword ptr [ebx+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x54", "mov esi, dword ptr [esp+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x56", "mov esi, dword ptr [esi+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x57", "mov esi, dword ptr [edi+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x58", "mov esi, dword ptr [eax+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x59", "mov esi, dword ptr [ecx+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x5a", "mov esi, dword ptr [edx+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x5b", "mov esi, dword ptr [ebx+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x5c", "mov esi, dword ptr [esp+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x5e", "mov esi, dword ptr [esi+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x5f", "mov esi, dword ptr [edi+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x68", "mov esi, dword ptr [eax+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x69", "mov esi, dword ptr [ecx+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x6a", "mov esi, dword ptr [edx+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x6b", "mov esi, dword ptr [ebx+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x6c", "mov esi, dword ptr [esp+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x6e", "mov esi, dword ptr [esi+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x6f", "mov esi, dword ptr [edi+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x70", "mov esi, dword ptr [eax+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x71", "mov esi, dword ptr [ecx+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x72", "mov esi, dword ptr [edx+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x73", "mov esi, dword ptr [ebx+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x74", "mov esi, dword ptr [esp+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x76", "mov esi, dword ptr [esi+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x77", "mov esi, dword ptr [edi+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x78", "mov esi, dword ptr [eax+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x79", "mov esi, dword ptr [ecx+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x7a", "mov esi, dword ptr [edx+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x7b", "mov esi, dword ptr [ebx+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x7c", "mov esi, dword ptr [esp+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x7e", "mov esi, dword ptr [esi+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x7f", "mov esi, dword ptr [edi+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x80", "mov esi, dword ptr [eax+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x81", "mov esi, dword ptr [ecx+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x82", "mov esi, dword ptr [edx+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x83", "mov esi, dword ptr [ebx+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x84", "mov esi, dword ptr [esp+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x86", "mov esi, dword ptr [esi+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x87", "mov esi, dword ptr [edi+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x88", "mov esi, dword ptr [eax+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x89", "mov esi, dword ptr [ecx+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x8a", "mov esi, dword ptr [edx+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x8b", "mov esi, dword ptr [ebx+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x8c", "mov esi, dword ptr [esp+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x8e", "mov esi, dword ptr [esi+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x8f", "mov esi, dword ptr [edi+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x90", "mov esi, dword ptr [eax+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x91", "mov esi, dword ptr [ecx+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x92", "mov esi, dword ptr [edx+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x93", "mov esi, dword ptr [ebx+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x94", "mov esi, dword ptr [esp+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x96", "mov esi, dword ptr [esi+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x97", "mov esi, dword ptr [edi+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x98", "mov esi, dword ptr [eax+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x99", "mov esi, dword ptr [ecx+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x9a", "mov esi, dword ptr [edx+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x9b", "mov esi, dword ptr [ebx+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x9c", "mov esi, dword ptr [esp+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x9e", "mov esi, dword ptr [esi+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\x9f", "mov esi, dword ptr [edi+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xa8", "mov esi, dword ptr [eax+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xa9", "mov esi, dword ptr [ecx+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xaa", "mov esi, dword ptr [edx+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xab", "mov esi, dword ptr [ebx+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xac", "mov esi, dword ptr [esp+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xae", "mov esi, dword ptr [esi+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xaf", "mov esi, dword ptr [edi+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xb0", "mov esi, dword ptr [eax+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xb1", "mov esi, dword ptr [ecx+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xb2", "mov esi, dword ptr [edx+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xb3", "mov esi, dword ptr [ebx+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xb4", "mov esi, dword ptr [esp+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xb6", "mov esi, dword ptr [esi+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xb7", "mov esi, dword ptr [edi+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xb8", "mov esi, dword ptr [eax+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xb9", "mov esi, dword ptr [ecx+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xba", "mov esi, dword ptr [edx+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xbb", "mov esi, dword ptr [ebx+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xbc", "mov esi, dword ptr [esp+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xbe", "mov esi, dword ptr [esi+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xbf", "mov esi, dword ptr [edi+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xc0", "mov esi, dword ptr [eax+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xc1", "mov esi, dword ptr [ecx+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xc2", "mov esi, dword ptr [edx+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xc3", "mov esi, dword ptr [ebx+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xc4", "mov esi, dword ptr [esp+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xc6", "mov esi, dword ptr [esi+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xc7", "mov esi, dword ptr [edi+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xc8", "mov esi, dword ptr [eax+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xc9", "mov esi, dword ptr [ecx+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xca", "mov esi, dword ptr [edx+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xcb", "mov esi, dword ptr [ebx+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xcc", "mov esi, dword ptr [esp+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xce", "mov esi, dword ptr [esi+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xcf", "mov esi, dword ptr [edi+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xd0", "mov esi, dword ptr [eax+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xd1", "mov esi, dword ptr [ecx+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xd2", "mov esi, dword ptr [edx+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xd3", "mov esi, dword ptr [ebx+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xd4", "mov esi, dword ptr [esp+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xd6", "mov esi, dword ptr [esi+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xd7", "mov esi, dword ptr [edi+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xd8", "mov esi, dword ptr [eax+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xd9", "mov esi, dword ptr [ecx+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xda", "mov esi, dword ptr [edx+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xdb", "mov esi, dword ptr [ebx+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xdc", "mov esi, dword ptr [esp+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xde", "mov esi, dword ptr [esi+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xdf", "mov esi, dword ptr [edi+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xe8", "mov esi, dword ptr [eax+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xe9", "mov esi, dword ptr [ecx+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xea", "mov esi, dword ptr [edx+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xeb", "mov esi, dword ptr [ebx+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xec", "mov esi, dword ptr [esp+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xee", "mov esi, dword ptr [esi+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xef", "mov esi, dword ptr [edi+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xf0", "mov esi, dword ptr [eax+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xf1", "mov esi, dword ptr [ecx+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xf2", "mov esi, dword ptr [edx+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xf3", "mov esi, dword ptr [ebx+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xf4", "mov esi, dword ptr [esp+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xf6", "mov esi, dword ptr [esi+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xf7", "mov esi, dword ptr [edi+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xf8", "mov esi, dword ptr [eax+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xf9", "mov esi, dword ptr [ecx+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xfa", "mov esi, dword ptr [edx+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xfb", "mov esi, dword ptr [ebx+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xfc", "mov esi, dword ptr [esp+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xfe", "mov esi, dword ptr [esi+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x34\xff", "mov esi, dword ptr [edi+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x00", "mov edi, dword ptr [eax+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x01", "mov edi, dword ptr [ecx+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x02", "mov edi, dword ptr [edx+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x03", "mov edi, dword ptr [ebx+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x04", "mov edi, dword ptr [esp+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x06", "mov edi, dword ptr [esi+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x07", "mov edi, dword ptr [edi+eax]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x08", "mov edi, dword ptr [eax+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x09", "mov edi, dword ptr [ecx+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x0a", "mov edi, dword ptr [edx+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x0b", "mov edi, dword ptr [ebx+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x0c", "mov edi, dword ptr [esp+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x0e", "mov edi, dword ptr [esi+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x0f", "mov edi, dword ptr [edi+ecx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x10", "mov edi, dword ptr [eax+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x11", "mov edi, dword ptr [ecx+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x12", "mov edi, dword ptr [edx+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x13", "mov edi, dword ptr [ebx+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x14", "mov edi, dword ptr [esp+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x16", "mov edi, dword ptr [esi+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x17", "mov edi, dword ptr [edi+edx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x18", "mov edi, dword ptr [eax+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x19", "mov edi, dword ptr [ecx+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x1a", "mov edi, dword ptr [edx+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x1b", "mov edi, dword ptr [ebx+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x1c", "mov edi, dword ptr [esp+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x1e", "mov edi, dword ptr [esi+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x1f", "mov edi, dword ptr [edi+ebx]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x24", "mov edi, dword ptr [esp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x28", "mov edi, dword ptr [eax+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x29", "mov edi, dword ptr [ecx+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x2a", "mov edi, dword ptr [edx+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x2b", "mov edi, dword ptr [ebx+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x2c", "mov edi, dword ptr [esp+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x2e", "mov edi, dword ptr [esi+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x2f", "mov edi, dword ptr [edi+ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x30", "mov edi, dword ptr [eax+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x31", "mov edi, dword ptr [ecx+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x32", "mov edi, dword ptr [edx+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x33", "mov edi, dword ptr [ebx+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x34", "mov edi, dword ptr [esp+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x36", "mov edi, dword ptr [esi+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x37", "mov edi, dword ptr [edi+esi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x38", "mov edi, dword ptr [eax+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x39", "mov edi, dword ptr [ecx+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x3a", "mov edi, dword ptr [edx+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x3b", "mov edi, dword ptr [ebx+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x3c", "mov edi, dword ptr [esp+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x3e", "mov edi, dword ptr [esi+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x3f", "mov edi, dword ptr [edi+edi]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x40", "mov edi, dword ptr [eax+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x41", "mov edi, dword ptr [ecx+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x42", "mov edi, dword ptr [edx+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x43", "mov edi, dword ptr [ebx+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x44", "mov edi, dword ptr [esp+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x46", "mov edi, dword ptr [esi+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x47", "mov edi, dword ptr [edi+eax*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x48", "mov edi, dword ptr [eax+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x49", "mov edi, dword ptr [ecx+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x4a", "mov edi, dword ptr [edx+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x4b", "mov edi, dword ptr [ebx+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x4c", "mov edi, dword ptr [esp+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x4e", "mov edi, dword ptr [esi+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x4f", "mov edi, dword ptr [edi+ecx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x50", "mov edi, dword ptr [eax+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x51", "mov edi, dword ptr [ecx+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x52", "mov edi, dword ptr [edx+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x53", "mov edi, dword ptr [ebx+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x54", "mov edi, dword ptr [esp+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x56", "mov edi, dword ptr [esi+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x57", "mov edi, dword ptr [edi+edx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x58", "mov edi, dword ptr [eax+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x59", "mov edi, dword ptr [ecx+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x5a", "mov edi, dword ptr [edx+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x5b", "mov edi, dword ptr [ebx+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x5c", "mov edi, dword ptr [esp+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x5e", "mov edi, dword ptr [esi+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x5f", "mov edi, dword ptr [edi+ebx*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x68", "mov edi, dword ptr [eax+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x69", "mov edi, dword ptr [ecx+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x6a", "mov edi, dword ptr [edx+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x6b", "mov edi, dword ptr [ebx+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x6c", "mov edi, dword ptr [esp+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x6e", "mov edi, dword ptr [esi+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x6f", "mov edi, dword ptr [edi+ebp*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x70", "mov edi, dword ptr [eax+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x71", "mov edi, dword ptr [ecx+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x72", "mov edi, dword ptr [edx+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x73", "mov edi, dword ptr [ebx+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x74", "mov edi, dword ptr [esp+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x76", "mov edi, dword ptr [esi+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x77", "mov edi, dword ptr [edi+esi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x78", "mov edi, dword ptr [eax+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x79", "mov edi, dword ptr [ecx+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x7a", "mov edi, dword ptr [edx+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x7b", "mov edi, dword ptr [ebx+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x7c", "mov edi, dword ptr [esp+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x7e", "mov edi, dword ptr [esi+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x7f", "mov edi, dword ptr [edi+edi*2]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x80", "mov edi, dword ptr [eax+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x81", "mov edi, dword ptr [ecx+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x82", "mov edi, dword ptr [edx+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x83", "mov edi, dword ptr [ebx+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x84", "mov edi, dword ptr [esp+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x86", "mov edi, dword ptr [esi+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x87", "mov edi, dword ptr [edi+eax*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x88", "mov edi, dword ptr [eax+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x89", "mov edi, dword ptr [ecx+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x8a", "mov edi, dword ptr [edx+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x8b", "mov edi, dword ptr [ebx+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x8c", "mov edi, dword ptr [esp+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x8e", "mov edi, dword ptr [esi+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x8f", "mov edi, dword ptr [edi+ecx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x90", "mov edi, dword ptr [eax+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x91", "mov edi, dword ptr [ecx+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x92", "mov edi, dword ptr [edx+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x93", "mov edi, dword ptr [ebx+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x94", "mov edi, dword ptr [esp+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x96", "mov edi, dword ptr [esi+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x97", "mov edi, dword ptr [edi+edx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x98", "mov edi, dword ptr [eax+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x99", "mov edi, dword ptr [ecx+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x9a", "mov edi, dword ptr [edx+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x9b", "mov edi, dword ptr [ebx+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x9c", "mov edi, dword ptr [esp+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x9e", "mov edi, dword ptr [esi+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\x9f", "mov edi, dword ptr [edi+ebx*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xa8", "mov edi, dword ptr [eax+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xa9", "mov edi, dword ptr [ecx+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xaa", "mov edi, dword ptr [edx+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xab", "mov edi, dword ptr [ebx+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xac", "mov edi, dword ptr [esp+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xae", "mov edi, dword ptr [esi+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xaf", "mov edi, dword ptr [edi+ebp*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xb0", "mov edi, dword ptr [eax+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xb1", "mov edi, dword ptr [ecx+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xb2", "mov edi, dword ptr [edx+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xb3", "mov edi, dword ptr [ebx+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xb4", "mov edi, dword ptr [esp+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xb6", "mov edi, dword ptr [esi+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xb7", "mov edi, dword ptr [edi+esi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xb8", "mov edi, dword ptr [eax+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xb9", "mov edi, dword ptr [ecx+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xba", "mov edi, dword ptr [edx+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xbb", "mov edi, dword ptr [ebx+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xbc", "mov edi, dword ptr [esp+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xbe", "mov edi, dword ptr [esi+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xbf", "mov edi, dword ptr [edi+edi*4]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xc0", "mov edi, dword ptr [eax+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xc1", "mov edi, dword ptr [ecx+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xc2", "mov edi, dword ptr [edx+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xc3", "mov edi, dword ptr [ebx+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xc4", "mov edi, dword ptr [esp+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xc6", "mov edi, dword ptr [esi+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xc7", "mov edi, dword ptr [edi+eax*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xc8", "mov edi, dword ptr [eax+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xc9", "mov edi, dword ptr [ecx+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xca", "mov edi, dword ptr [edx+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xcb", "mov edi, dword ptr [ebx+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xcc", "mov edi, dword ptr [esp+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xce", "mov edi, dword ptr [esi+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xcf", "mov edi, dword ptr [edi+ecx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xd0", "mov edi, dword ptr [eax+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xd1", "mov edi, dword ptr [ecx+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xd2", "mov edi, dword ptr [edx+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xd3", "mov edi, dword ptr [ebx+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xd4", "mov edi, dword ptr [esp+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xd6", "mov edi, dword ptr [esi+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xd7", "mov edi, dword ptr [edi+edx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xd8", "mov edi, dword ptr [eax+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xd9", "mov edi, dword ptr [ecx+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xda", "mov edi, dword ptr [edx+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xdb", "mov edi, dword ptr [ebx+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xdc", "mov edi, dword ptr [esp+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xde", "mov edi, dword ptr [esi+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xdf", "mov edi, dword ptr [edi+ebx*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xe8", "mov edi, dword ptr [eax+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xe9", "mov edi, dword ptr [ecx+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xea", "mov edi, dword ptr [edx+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xeb", "mov edi, dword ptr [ebx+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xec", "mov edi, dword ptr [esp+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xee", "mov edi, dword ptr [esi+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xef", "mov edi, dword ptr [edi+ebp*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xf0", "mov edi, dword ptr [eax+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xf1", "mov edi, dword ptr [ecx+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xf2", "mov edi, dword ptr [edx+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xf3", "mov edi, dword ptr [ebx+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xf4", "mov edi, dword ptr [esp+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xf6", "mov edi, dword ptr [esi+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xf7", "mov edi, dword ptr [edi+esi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xf8", "mov edi, dword ptr [eax+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xf9", "mov edi, dword ptr [ecx+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xfa", "mov edi, dword ptr [edx+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xfb", "mov edi, dword ptr [ebx+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xfc", "mov edi, dword ptr [esp+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xfe", "mov edi, dword ptr [esi+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x3c\xff", "mov edi, dword ptr [edi+edi*8]", insn32_t::FLAG_NONE},
	{3,"\x8b\x45\x00", "mov eax, dword ptr [ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x4d\x00", "mov ecx, dword ptr [ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x55\x00", "mov edx, dword ptr [ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x5d\x00", "mov ebx, dword ptr [ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x65\x00", "mov esp, dword ptr [ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x6d\x00", "mov ebp, dword ptr [ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x75\x00", "mov esi, dword ptr [ebp]", insn32_t::FLAG_NONE},
	{3,"\x8b\x7d\x00", "mov edi, dword ptr [ebp]", insn32_t::FLAG_NONE},
	{4,"\x89\x44\x05\x00", "mov dword ptr [ebp+eax], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\x0d\x00", "mov dword ptr [ebp+ecx], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\x15\x00", "mov dword ptr [ebp+edx], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\x1d\x00", "mov dword ptr [ebp+ebx], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\x2d\x00", "mov dword ptr [ebp+ebp], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\x35\x00", "mov dword ptr [ebp+esi], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\x3d\x00", "mov dword ptr [ebp+edi], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\x45\x00", "mov dword ptr [ebp+eax*2], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\x4d\x00", "mov dword ptr [ebp+ecx*2], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\x55\x00", "mov dword ptr [ebp+edx*2], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\x5d\x00", "mov dword ptr [ebp+ebx*2], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\x6d\x00", "mov dword ptr [ebp+ebp*2], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\x75\x00", "mov dword ptr [ebp+esi*2], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\x7d\x00", "mov dword ptr [ebp+edi*2], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\x85\x00", "mov dword ptr [ebp+eax*4], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\x8d\x00", "mov dword ptr [ebp+ecx*4], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\x95\x00", "mov dword ptr [ebp+edx*4], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\x9d\x00", "mov dword ptr [ebp+ebx*4], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\xad\x00", "mov dword ptr [ebp+ebp*4], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\xb5\x00", "mov dword ptr [ebp+esi*4], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\xbd\x00", "mov dword ptr [ebp+edi*4], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\xc5\x00", "mov dword ptr [ebp+eax*8], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\xcd\x00", "mov dword ptr [ebp+ecx*8], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\xd5\x00", "mov dword ptr [ebp+edx*8], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\xdd\x00", "mov dword ptr [ebp+ebx*8], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\xed\x00", "mov dword ptr [ebp+ebp*8], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\xf5\x00", "mov dword ptr [ebp+esi*8], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x44\xfd\x00", "mov dword ptr [ebp+edi*8], eax", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\x05\x00", "mov dword ptr [ebp+eax], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\x0d\x00", "mov dword ptr [ebp+ecx], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\x15\x00", "mov dword ptr [ebp+edx], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\x1d\x00", "mov dword ptr [ebp+ebx], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\x2d\x00", "mov dword ptr [ebp+ebp], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\x35\x00", "mov dword ptr [ebp+esi], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\x3d\x00", "mov dword ptr [ebp+edi], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\x45\x00", "mov dword ptr [ebp+eax*2], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\x4d\x00", "mov dword ptr [ebp+ecx*2], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\x55\x00", "mov dword ptr [ebp+edx*2], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\x5d\x00", "mov dword ptr [ebp+ebx*2], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\x6d\x00", "mov dword ptr [ebp+ebp*2], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\x75\x00", "mov dword ptr [ebp+esi*2], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\x7d\x00", "mov dword ptr [ebp+edi*2], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\x85\x00", "mov dword ptr [ebp+eax*4], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\x8d\x00", "mov dword ptr [ebp+ecx*4], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\x95\x00", "mov dword ptr [ebp+edx*4], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\x9d\x00", "mov dword ptr [ebp+ebx*4], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\xad\x00", "mov dword ptr [ebp+ebp*4], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\xb5\x00", "mov dword ptr [ebp+esi*4], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\xbd\x00", "mov dword ptr [ebp+edi*4], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\xc5\x00", "mov dword ptr [ebp+eax*8], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\xcd\x00", "mov dword ptr [ebp+ecx*8], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\xd5\x00", "mov dword ptr [ebp+edx*8], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\xdd\x00", "mov dword ptr [ebp+ebx*8], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\xed\x00", "mov dword ptr [ebp+ebp*8], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\xf5\x00", "mov dword ptr [ebp+esi*8], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x4c\xfd\x00", "mov dword ptr [ebp+edi*8], ecx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\x05\x00", "mov dword ptr [ebp+eax], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\x0d\x00", "mov dword ptr [ebp+ecx], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\x15\x00", "mov dword ptr [ebp+edx], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\x1d\x00", "mov dword ptr [ebp+ebx], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\x2d\x00", "mov dword ptr [ebp+ebp], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\x35\x00", "mov dword ptr [ebp+esi], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\x3d\x00", "mov dword ptr [ebp+edi], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\x45\x00", "mov dword ptr [ebp+eax*2], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\x4d\x00", "mov dword ptr [ebp+ecx*2], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\x55\x00", "mov dword ptr [ebp+edx*2], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\x5d\x00", "mov dword ptr [ebp+ebx*2], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\x6d\x00", "mov dword ptr [ebp+ebp*2], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\x75\x00", "mov dword ptr [ebp+esi*2], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\x7d\x00", "mov dword ptr [ebp+edi*2], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\x85\x00", "mov dword ptr [ebp+eax*4], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\x8d\x00", "mov dword ptr [ebp+ecx*4], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\x95\x00", "mov dword ptr [ebp+edx*4], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\x9d\x00", "mov dword ptr [ebp+ebx*4], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\xad\x00", "mov dword ptr [ebp+ebp*4], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\xb5\x00", "mov dword ptr [ebp+esi*4], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\xbd\x00", "mov dword ptr [ebp+edi*4], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\xc5\x00", "mov dword ptr [ebp+eax*8], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\xcd\x00", "mov dword ptr [ebp+ecx*8], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\xd5\x00", "mov dword ptr [ebp+edx*8], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\xdd\x00", "mov dword ptr [ebp+ebx*8], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\xed\x00", "mov dword ptr [ebp+ebp*8], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\xf5\x00", "mov dword ptr [ebp+esi*8], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x54\xfd\x00", "mov dword ptr [ebp+edi*8], edx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\x05\x00", "mov dword ptr [ebp+eax], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\x0d\x00", "mov dword ptr [ebp+ecx], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\x15\x00", "mov dword ptr [ebp+edx], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\x1d\x00", "mov dword ptr [ebp+ebx], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\x2d\x00", "mov dword ptr [ebp+ebp], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\x35\x00", "mov dword ptr [ebp+esi], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\x3d\x00", "mov dword ptr [ebp+edi], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\x45\x00", "mov dword ptr [ebp+eax*2], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\x4d\x00", "mov dword ptr [ebp+ecx*2], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\x55\x00", "mov dword ptr [ebp+edx*2], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\x5d\x00", "mov dword ptr [ebp+ebx*2], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\x6d\x00", "mov dword ptr [ebp+ebp*2], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\x75\x00", "mov dword ptr [ebp+esi*2], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\x7d\x00", "mov dword ptr [ebp+edi*2], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\x85\x00", "mov dword ptr [ebp+eax*4], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\x8d\x00", "mov dword ptr [ebp+ecx*4], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\x95\x00", "mov dword ptr [ebp+edx*4], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\x9d\x00", "mov dword ptr [ebp+ebx*4], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\xad\x00", "mov dword ptr [ebp+ebp*4], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\xb5\x00", "mov dword ptr [ebp+esi*4], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\xbd\x00", "mov dword ptr [ebp+edi*4], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\xc5\x00", "mov dword ptr [ebp+eax*8], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\xcd\x00", "mov dword ptr [ebp+ecx*8], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\xd5\x00", "mov dword ptr [ebp+edx*8], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\xdd\x00", "mov dword ptr [ebp+ebx*8], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\xed\x00", "mov dword ptr [ebp+ebp*8], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\xf5\x00", "mov dword ptr [ebp+esi*8], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x5c\xfd\x00", "mov dword ptr [ebp+edi*8], ebx", insn32_t::FLAG_NONE},
	{4,"\x89\x64\x05\x00", "mov dword ptr [ebp+eax], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\x0d\x00", "mov dword ptr [ebp+ecx], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\x15\x00", "mov dword ptr [ebp+edx], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\x1d\x00", "mov dword ptr [ebp+ebx], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\x2d\x00", "mov dword ptr [ebp+ebp], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\x35\x00", "mov dword ptr [ebp+esi], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\x3d\x00", "mov dword ptr [ebp+edi], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\x45\x00", "mov dword ptr [ebp+eax*2], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\x4d\x00", "mov dword ptr [ebp+ecx*2], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\x55\x00", "mov dword ptr [ebp+edx*2], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\x5d\x00", "mov dword ptr [ebp+ebx*2], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\x6d\x00", "mov dword ptr [ebp+ebp*2], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\x75\x00", "mov dword ptr [ebp+esi*2], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\x7d\x00", "mov dword ptr [ebp+edi*2], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\x85\x00", "mov dword ptr [ebp+eax*4], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\x8d\x00", "mov dword ptr [ebp+ecx*4], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\x95\x00", "mov dword ptr [ebp+edx*4], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\x9d\x00", "mov dword ptr [ebp+ebx*4], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\xad\x00", "mov dword ptr [ebp+ebp*4], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\xb5\x00", "mov dword ptr [ebp+esi*4], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\xbd\x00", "mov dword ptr [ebp+edi*4], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\xc5\x00", "mov dword ptr [ebp+eax*8], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\xcd\x00", "mov dword ptr [ebp+ecx*8], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\xd5\x00", "mov dword ptr [ebp+edx*8], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\xdd\x00", "mov dword ptr [ebp+ebx*8], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\xed\x00", "mov dword ptr [ebp+ebp*8], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\xf5\x00", "mov dword ptr [ebp+esi*8], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x64\xfd\x00", "mov dword ptr [ebp+edi*8], esp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\x05\x00", "mov dword ptr [ebp+eax], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\x0d\x00", "mov dword ptr [ebp+ecx], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\x15\x00", "mov dword ptr [ebp+edx], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\x1d\x00", "mov dword ptr [ebp+ebx], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\x2d\x00", "mov dword ptr [ebp+ebp], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\x35\x00", "mov dword ptr [ebp+esi], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\x3d\x00", "mov dword ptr [ebp+edi], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\x45\x00", "mov dword ptr [ebp+eax*2], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\x4d\x00", "mov dword ptr [ebp+ecx*2], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\x55\x00", "mov dword ptr [ebp+edx*2], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\x5d\x00", "mov dword ptr [ebp+ebx*2], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\x6d\x00", "mov dword ptr [ebp+ebp*2], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\x75\x00", "mov dword ptr [ebp+esi*2], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\x7d\x00", "mov dword ptr [ebp+edi*2], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\x85\x00", "mov dword ptr [ebp+eax*4], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\x8d\x00", "mov dword ptr [ebp+ecx*4], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\x95\x00", "mov dword ptr [ebp+edx*4], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\x9d\x00", "mov dword ptr [ebp+ebx*4], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\xad\x00", "mov dword ptr [ebp+ebp*4], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\xb5\x00", "mov dword ptr [ebp+esi*4], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\xbd\x00", "mov dword ptr [ebp+edi*4], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\xc5\x00", "mov dword ptr [ebp+eax*8], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\xcd\x00", "mov dword ptr [ebp+ecx*8], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\xd5\x00", "mov dword ptr [ebp+edx*8], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\xdd\x00", "mov dword ptr [ebp+ebx*8], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\xed\x00", "mov dword ptr [ebp+ebp*8], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\xf5\x00", "mov dword ptr [ebp+esi*8], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x6c\xfd\x00", "mov dword ptr [ebp+edi*8], ebp", insn32_t::FLAG_NONE},
	{4,"\x89\x74\x05\x00", "mov dword ptr [ebp+eax], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\x0d\x00", "mov dword ptr [ebp+ecx], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\x15\x00", "mov dword ptr [ebp+edx], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\x1d\x00", "mov dword ptr [ebp+ebx], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\x2d\x00", "mov dword ptr [ebp+ebp], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\x35\x00", "mov dword ptr [ebp+esi], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\x3d\x00", "mov dword ptr [ebp+edi], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\x45\x00", "mov dword ptr [ebp+eax*2], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\x4d\x00", "mov dword ptr [ebp+ecx*2], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\x55\x00", "mov dword ptr [ebp+edx*2], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\x5d\x00", "mov dword ptr [ebp+ebx*2], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\x6d\x00", "mov dword ptr [ebp+ebp*2], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\x75\x00", "mov dword ptr [ebp+esi*2], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\x7d\x00", "mov dword ptr [ebp+edi*2], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\x85\x00", "mov dword ptr [ebp+eax*4], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\x8d\x00", "mov dword ptr [ebp+ecx*4], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\x95\x00", "mov dword ptr [ebp+edx*4], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\x9d\x00", "mov dword ptr [ebp+ebx*4], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\xad\x00", "mov dword ptr [ebp+ebp*4], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\xb5\x00", "mov dword ptr [ebp+esi*4], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\xbd\x00", "mov dword ptr [ebp+edi*4], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\xc5\x00", "mov dword ptr [ebp+eax*8], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\xcd\x00", "mov dword ptr [ebp+ecx*8], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\xd5\x00", "mov dword ptr [ebp+edx*8], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\xdd\x00", "mov dword ptr [ebp+ebx*8], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\xed\x00", "mov dword ptr [ebp+ebp*8], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\xf5\x00", "mov dword ptr [ebp+esi*8], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x74\xfd\x00", "mov dword ptr [ebp+edi*8], esi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\x05\x00", "mov dword ptr [ebp+eax], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\x0d\x00", "mov dword ptr [ebp+ecx], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\x15\x00", "mov dword ptr [ebp+edx], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\x1d\x00", "mov dword ptr [ebp+ebx], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\x2d\x00", "mov dword ptr [ebp+ebp], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\x35\x00", "mov dword ptr [ebp+esi], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\x3d\x00", "mov dword ptr [ebp+edi], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\x45\x00", "mov dword ptr [ebp+eax*2], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\x4d\x00", "mov dword ptr [ebp+ecx*2], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\x55\x00", "mov dword ptr [ebp+edx*2], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\x5d\x00", "mov dword ptr [ebp+ebx*2], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\x6d\x00", "mov dword ptr [ebp+ebp*2], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\x75\x00", "mov dword ptr [ebp+esi*2], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\x7d\x00", "mov dword ptr [ebp+edi*2], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\x85\x00", "mov dword ptr [ebp+eax*4], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\x8d\x00", "mov dword ptr [ebp+ecx*4], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\x95\x00", "mov dword ptr [ebp+edx*4], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\x9d\x00", "mov dword ptr [ebp+ebx*4], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\xad\x00", "mov dword ptr [ebp+ebp*4], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\xb5\x00", "mov dword ptr [ebp+esi*4], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\xbd\x00", "mov dword ptr [ebp+edi*4], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\xc5\x00", "mov dword ptr [ebp+eax*8], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\xcd\x00", "mov dword ptr [ebp+ecx*8], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\xd5\x00", "mov dword ptr [ebp+edx*8], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\xdd\x00", "mov dword ptr [ebp+ebx*8], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\xed\x00", "mov dword ptr [ebp+ebp*8], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\xf5\x00", "mov dword ptr [ebp+esi*8], edi", insn32_t::FLAG_NONE},
	{4,"\x89\x7c\xfd\x00", "mov dword ptr [ebp+edi*8], edi", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\x05\x00", "mov eax, dword ptr [ebp+eax]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\x0d\x00", "mov eax, dword ptr [ebp+ecx]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\x15\x00", "mov eax, dword ptr [ebp+edx]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\x1d\x00", "mov eax, dword ptr [ebp+ebx]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\x2d\x00", "mov eax, dword ptr [ebp+ebp]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\x35\x00", "mov eax, dword ptr [ebp+esi]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\x3d\x00", "mov eax, dword ptr [ebp+edi]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\x45\x00", "mov eax, dword ptr [ebp+eax*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\x4d\x00", "mov eax, dword ptr [ebp+ecx*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\x55\x00", "mov eax, dword ptr [ebp+edx*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\x5d\x00", "mov eax, dword ptr [ebp+ebx*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\x6d\x00", "mov eax, dword ptr [ebp+ebp*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\x75\x00", "mov eax, dword ptr [ebp+esi*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\x7d\x00", "mov eax, dword ptr [ebp+edi*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\x85\x00", "mov eax, dword ptr [ebp+eax*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\x8d\x00", "mov eax, dword ptr [ebp+ecx*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\x95\x00", "mov eax, dword ptr [ebp+edx*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\x9d\x00", "mov eax, dword ptr [ebp+ebx*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\xad\x00", "mov eax, dword ptr [ebp+ebp*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\xb5\x00", "mov eax, dword ptr [ebp+esi*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\xbd\x00", "mov eax, dword ptr [ebp+edi*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\xc5\x00", "mov eax, dword ptr [ebp+eax*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\xcd\x00", "mov eax, dword ptr [ebp+ecx*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\xd5\x00", "mov eax, dword ptr [ebp+edx*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\xdd\x00", "mov eax, dword ptr [ebp+ebx*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\xed\x00", "mov eax, dword ptr [ebp+ebp*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\xf5\x00", "mov eax, dword ptr [ebp+esi*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x44\xfd\x00", "mov eax, dword ptr [ebp+edi*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\x05\x00", "mov ecx, dword ptr [ebp+eax]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\x0d\x00", "mov ecx, dword ptr [ebp+ecx]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\x15\x00", "mov ecx, dword ptr [ebp+edx]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\x1d\x00", "mov ecx, dword ptr [ebp+ebx]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\x2d\x00", "mov ecx, dword ptr [ebp+ebp]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\x35\x00", "mov ecx, dword ptr [ebp+esi]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\x3d\x00", "mov ecx, dword ptr [ebp+edi]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\x45\x00", "mov ecx, dword ptr [ebp+eax*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\x4d\x00", "mov ecx, dword ptr [ebp+ecx*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\x55\x00", "mov ecx, dword ptr [ebp+edx*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\x5d\x00", "mov ecx, dword ptr [ebp+ebx*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\x6d\x00", "mov ecx, dword ptr [ebp+ebp*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\x75\x00", "mov ecx, dword ptr [ebp+esi*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\x7d\x00", "mov ecx, dword ptr [ebp+edi*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\x85\x00", "mov ecx, dword ptr [ebp+eax*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\x8d\x00", "mov ecx, dword ptr [ebp+ecx*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\x95\x00", "mov ecx, dword ptr [ebp+edx*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\x9d\x00", "mov ecx, dword ptr [ebp+ebx*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\xad\x00", "mov ecx, dword ptr [ebp+ebp*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\xb5\x00", "mov ecx, dword ptr [ebp+esi*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\xbd\x00", "mov ecx, dword ptr [ebp+edi*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\xc5\x00", "mov ecx, dword ptr [ebp+eax*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\xcd\x00", "mov ecx, dword ptr [ebp+ecx*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\xd5\x00", "mov ecx, dword ptr [ebp+edx*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\xdd\x00", "mov ecx, dword ptr [ebp+ebx*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\xed\x00", "mov ecx, dword ptr [ebp+ebp*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\xf5\x00", "mov ecx, dword ptr [ebp+esi*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x4c\xfd\x00", "mov ecx, dword ptr [ebp+edi*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\x05\x00", "mov edx, dword ptr [ebp+eax]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\x0d\x00", "mov edx, dword ptr [ebp+ecx]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\x15\x00", "mov edx, dword ptr [ebp+edx]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\x1d\x00", "mov edx, dword ptr [ebp+ebx]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\x2d\x00", "mov edx, dword ptr [ebp+ebp]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\x35\x00", "mov edx, dword ptr [ebp+esi]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\x3d\x00", "mov edx, dword ptr [ebp+edi]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\x45\x00", "mov edx, dword ptr [ebp+eax*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\x4d\x00", "mov edx, dword ptr [ebp+ecx*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\x55\x00", "mov edx, dword ptr [ebp+edx*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\x5d\x00", "mov edx, dword ptr [ebp+ebx*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\x6d\x00", "mov edx, dword ptr [ebp+ebp*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\x75\x00", "mov edx, dword ptr [ebp+esi*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\x7d\x00", "mov edx, dword ptr [ebp+edi*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\x85\x00", "mov edx, dword ptr [ebp+eax*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\x8d\x00", "mov edx, dword ptr [ebp+ecx*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\x95\x00", "mov edx, dword ptr [ebp+edx*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\x9d\x00", "mov edx, dword ptr [ebp+ebx*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\xad\x00", "mov edx, dword ptr [ebp+ebp*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\xb5\x00", "mov edx, dword ptr [ebp+esi*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\xbd\x00", "mov edx, dword ptr [ebp+edi*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\xc5\x00", "mov edx, dword ptr [ebp+eax*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\xcd\x00", "mov edx, dword ptr [ebp+ecx*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\xd5\x00", "mov edx, dword ptr [ebp+edx*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\xdd\x00", "mov edx, dword ptr [ebp+ebx*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\xed\x00", "mov edx, dword ptr [ebp+ebp*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\xf5\x00", "mov edx, dword ptr [ebp+esi*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x54\xfd\x00", "mov edx, dword ptr [ebp+edi*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\x05\x00", "mov ebx, dword ptr [ebp+eax]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\x0d\x00", "mov ebx, dword ptr [ebp+ecx]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\x15\x00", "mov ebx, dword ptr [ebp+edx]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\x1d\x00", "mov ebx, dword ptr [ebp+ebx]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\x2d\x00", "mov ebx, dword ptr [ebp+ebp]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\x35\x00", "mov ebx, dword ptr [ebp+esi]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\x3d\x00", "mov ebx, dword ptr [ebp+edi]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\x45\x00", "mov ebx, dword ptr [ebp+eax*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\x4d\x00", "mov ebx, dword ptr [ebp+ecx*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\x55\x00", "mov ebx, dword ptr [ebp+edx*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\x5d\x00", "mov ebx, dword ptr [ebp+ebx*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\x6d\x00", "mov ebx, dword ptr [ebp+ebp*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\x75\x00", "mov ebx, dword ptr [ebp+esi*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\x7d\x00", "mov ebx, dword ptr [ebp+edi*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\x85\x00", "mov ebx, dword ptr [ebp+eax*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\x8d\x00", "mov ebx, dword ptr [ebp+ecx*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\x95\x00", "mov ebx, dword ptr [ebp+edx*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\x9d\x00", "mov ebx, dword ptr [ebp+ebx*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\xad\x00", "mov ebx, dword ptr [ebp+ebp*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\xb5\x00", "mov ebx, dword ptr [ebp+esi*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\xbd\x00", "mov ebx, dword ptr [ebp+edi*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\xc5\x00", "mov ebx, dword ptr [ebp+eax*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\xcd\x00", "mov ebx, dword ptr [ebp+ecx*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\xd5\x00", "mov ebx, dword ptr [ebp+edx*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\xdd\x00", "mov ebx, dword ptr [ebp+ebx*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\xed\x00", "mov ebx, dword ptr [ebp+ebp*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\xf5\x00", "mov ebx, dword ptr [ebp+esi*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x5c\xfd\x00", "mov ebx, dword ptr [ebp+edi*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\x05\x00", "mov esp, dword ptr [ebp+eax]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\x0d\x00", "mov esp, dword ptr [ebp+ecx]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\x15\x00", "mov esp, dword ptr [ebp+edx]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\x1d\x00", "mov esp, dword ptr [ebp+ebx]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\x2d\x00", "mov esp, dword ptr [ebp+ebp]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\x35\x00", "mov esp, dword ptr [ebp+esi]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\x3d\x00", "mov esp, dword ptr [ebp+edi]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\x45\x00", "mov esp, dword ptr [ebp+eax*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\x4d\x00", "mov esp, dword ptr [ebp+ecx*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\x55\x00", "mov esp, dword ptr [ebp+edx*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\x5d\x00", "mov esp, dword ptr [ebp+ebx*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\x6d\x00", "mov esp, dword ptr [ebp+ebp*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\x75\x00", "mov esp, dword ptr [ebp+esi*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\x7d\x00", "mov esp, dword ptr [ebp+edi*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\x85\x00", "mov esp, dword ptr [ebp+eax*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\x8d\x00", "mov esp, dword ptr [ebp+ecx*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\x95\x00", "mov esp, dword ptr [ebp+edx*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\x9d\x00", "mov esp, dword ptr [ebp+ebx*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\xad\x00", "mov esp, dword ptr [ebp+ebp*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\xb5\x00", "mov esp, dword ptr [ebp+esi*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\xbd\x00", "mov esp, dword ptr [ebp+edi*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\xc5\x00", "mov esp, dword ptr [ebp+eax*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\xcd\x00", "mov esp, dword ptr [ebp+ecx*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\xd5\x00", "mov esp, dword ptr [ebp+edx*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\xdd\x00", "mov esp, dword ptr [ebp+ebx*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\xed\x00", "mov esp, dword ptr [ebp+ebp*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\xf5\x00", "mov esp, dword ptr [ebp+esi*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x64\xfd\x00", "mov esp, dword ptr [ebp+edi*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\x05\x00", "mov ebp, dword ptr [ebp+eax]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\x0d\x00", "mov ebp, dword ptr [ebp+ecx]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\x15\x00", "mov ebp, dword ptr [ebp+edx]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\x1d\x00", "mov ebp, dword ptr [ebp+ebx]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\x2d\x00", "mov ebp, dword ptr [ebp+ebp]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\x35\x00", "mov ebp, dword ptr [ebp+esi]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\x3d\x00", "mov ebp, dword ptr [ebp+edi]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\x45\x00", "mov ebp, dword ptr [ebp+eax*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\x4d\x00", "mov ebp, dword ptr [ebp+ecx*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\x55\x00", "mov ebp, dword ptr [ebp+edx*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\x5d\x00", "mov ebp, dword ptr [ebp+ebx*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\x6d\x00", "mov ebp, dword ptr [ebp+ebp*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\x75\x00", "mov ebp, dword ptr [ebp+esi*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\x7d\x00", "mov ebp, dword ptr [ebp+edi*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\x85\x00", "mov ebp, dword ptr [ebp+eax*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\x8d\x00", "mov ebp, dword ptr [ebp+ecx*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\x95\x00", "mov ebp, dword ptr [ebp+edx*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\x9d\x00", "mov ebp, dword ptr [ebp+ebx*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\xad\x00", "mov ebp, dword ptr [ebp+ebp*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\xb5\x00", "mov ebp, dword ptr [ebp+esi*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\xbd\x00", "mov ebp, dword ptr [ebp+edi*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\xc5\x00", "mov ebp, dword ptr [ebp+eax*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\xcd\x00", "mov ebp, dword ptr [ebp+ecx*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\xd5\x00", "mov ebp, dword ptr [ebp+edx*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\xdd\x00", "mov ebp, dword ptr [ebp+ebx*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\xed\x00", "mov ebp, dword ptr [ebp+ebp*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\xf5\x00", "mov ebp, dword ptr [ebp+esi*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x6c\xfd\x00", "mov ebp, dword ptr [ebp+edi*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\x05\x00", "mov esi, dword ptr [ebp+eax]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\x0d\x00", "mov esi, dword ptr [ebp+ecx]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\x15\x00", "mov esi, dword ptr [ebp+edx]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\x1d\x00", "mov esi, dword ptr [ebp+ebx]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\x2d\x00", "mov esi, dword ptr [ebp+ebp]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\x35\x00", "mov esi, dword ptr [ebp+esi]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\x3d\x00", "mov esi, dword ptr [ebp+edi]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\x45\x00", "mov esi, dword ptr [ebp+eax*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\x4d\x00", "mov esi, dword ptr [ebp+ecx*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\x55\x00", "mov esi, dword ptr [ebp+edx*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\x5d\x00", "mov esi, dword ptr [ebp+ebx*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\x6d\x00", "mov esi, dword ptr [ebp+ebp*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\x75\x00", "mov esi, dword ptr [ebp+esi*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\x7d\x00", "mov esi, dword ptr [ebp+edi*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\x85\x00", "mov esi, dword ptr [ebp+eax*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\x8d\x00", "mov esi, dword ptr [ebp+ecx*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\x95\x00", "mov esi, dword ptr [ebp+edx*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\x9d\x00", "mov esi, dword ptr [ebp+ebx*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\xad\x00", "mov esi, dword ptr [ebp+ebp*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\xb5\x00", "mov esi, dword ptr [ebp+esi*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\xbd\x00", "mov esi, dword ptr [ebp+edi*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\xc5\x00", "mov esi, dword ptr [ebp+eax*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\xcd\x00", "mov esi, dword ptr [ebp+ecx*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\xd5\x00", "mov esi, dword ptr [ebp+edx*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\xdd\x00", "mov esi, dword ptr [ebp+ebx*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\xed\x00", "mov esi, dword ptr [ebp+ebp*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\xf5\x00", "mov esi, dword ptr [ebp+esi*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x74\xfd\x00", "mov esi, dword ptr [ebp+edi*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\x05\x00", "mov edi, dword ptr [ebp+eax]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\x0d\x00", "mov edi, dword ptr [ebp+ecx]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\x15\x00", "mov edi, dword ptr [ebp+edx]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\x1d\x00", "mov edi, dword ptr [ebp+ebx]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\x2d\x00", "mov edi, dword ptr [ebp+ebp]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\x35\x00", "mov edi, dword ptr [ebp+esi]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\x3d\x00", "mov edi, dword ptr [ebp+edi]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\x45\x00", "mov edi, dword ptr [ebp+eax*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\x4d\x00", "mov edi, dword ptr [ebp+ecx*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\x55\x00", "mov edi, dword ptr [ebp+edx*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\x5d\x00", "mov edi, dword ptr [ebp+ebx*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\x6d\x00", "mov edi, dword ptr [ebp+ebp*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\x75\x00", "mov edi, dword ptr [ebp+esi*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\x7d\x00", "mov edi, dword ptr [ebp+edi*2]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\x85\x00", "mov edi, dword ptr [ebp+eax*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\x8d\x00", "mov edi, dword ptr [ebp+ecx*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\x95\x00", "mov edi, dword ptr [ebp+edx*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\x9d\x00", "mov edi, dword ptr [ebp+ebx*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\xad\x00", "mov edi, dword ptr [ebp+ebp*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\xb5\x00", "mov edi, dword ptr [ebp+esi*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\xbd\x00", "mov edi, dword ptr [ebp+edi*4]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\xc5\x00", "mov edi, dword ptr [ebp+eax*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\xcd\x00", "mov edi, dword ptr [ebp+ecx*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\xd5\x00", "mov edi, dword ptr [ebp+edx*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\xdd\x00", "mov edi, dword ptr [ebp+ebx*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\xed\x00", "mov edi, dword ptr [ebp+ebp*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\xf5\x00", "mov edi, dword ptr [ebp+esi*8]", insn32_t::FLAG_NONE},
	{4,"\x8b\x7c\xfd\x00", "mov edi, dword ptr [ebp+edi*8]", insn32_t::FLAG_NONE},
	{7,"\x89\x04\x85\x00\x00\x00\x00", "mov dword ptr [eax*4], eax", insn32_t::FLAG_NONE},
	{7,"\x89\x04\x8d\x00\x00\x00\x00", "mov dword ptr [ecx*4], eax", insn32_t::FLAG_NONE},
	{7,"\x89\x04\x95\x00\x00\x00\x00", "mov dword ptr [edx*4], eax", insn32_t::FLAG_NONE},
	{7,"\x89\x04\x9d\x00\x00\x00\x00", "mov dword ptr [ebx*4], eax", insn32_t::FLAG_NONE},
	{7,"\x89\x04\xad\x00\x00\x00\x00", "mov dword ptr [ebp*4], eax", insn32_t::FLAG_NONE},
	{7,"\x89\x04\xb5\x00\x00\x00\x00", "mov dword ptr [esi*4], eax", insn32_t::FLAG_NONE},
	{7,"\x89\x04\xbd\x00\x00\x00\x00", "mov dword ptr [edi*4], eax", insn32_t::FLAG_NONE},
	{7,"\x89\x04\xc5\x00\x00\x00\x00", "mov dword ptr [eax*8], eax", insn32_t::FLAG_NONE},
	{7,"\x89\x04\xcd\x00\x00\x00\x00", "mov dword ptr [ecx*8], eax", insn32_t::FLAG_NONE},
	{7,"\x89\x04\xd5\x00\x00\x00\x00", "mov dword ptr [edx*8], eax", insn32_t::FLAG_NONE},
	{7,"\x89\x04\xdd\x00\x00\x00\x00", "mov dword ptr [ebx*8], eax", insn32_t::FLAG_NONE},
	{7,"\x89\x04\xed\x00\x00\x00\x00", "mov dword ptr [ebp*8], eax", insn32_t::FLAG_NONE},
	{7,"\x89\x04\xf5\x00\x00\x00\x00", "mov dword ptr [esi*8], eax", insn32_t::FLAG_NONE},
	{7,"\x89\x04\xfd\x00\x00\x00\x00", "mov dword ptr [edi*8], eax", insn32_t::FLAG_NONE},
	{7,"\x89\x0c\x85\x00\x00\x00\x00", "mov dword ptr [eax*4], ecx", insn32_t::FLAG_NONE},
	{7,"\x89\x0c\x8d\x00\x00\x00\x00", "mov dword ptr [ecx*4], ecx", insn32_t::FLAG_NONE},
	{7,"\x89\x0c\x95\x00\x00\x00\x00", "mov dword ptr [edx*4], ecx", insn32_t::FLAG_NONE},
	{7,"\x89\x0c\x9d\x00\x00\x00\x00", "mov dword ptr [ebx*4], ecx", insn32_t::FLAG_NONE},
	{7,"\x89\x0c\xad\x00\x00\x00\x00", "mov dword ptr [ebp*4], ecx", insn32_t::FLAG_NONE},
	{7,"\x89\x0c\xb5\x00\x00\x00\x00", "mov dword ptr [esi*4], ecx", insn32_t::FLAG_NONE},
	{7,"\x89\x0c\xbd\x00\x00\x00\x00", "mov dword ptr [edi*4], ecx", insn32_t::FLAG_NONE},
	{7,"\x89\x0c\xc5\x00\x00\x00\x00", "mov dword ptr [eax*8], ecx", insn32_t::FLAG_NONE},
	{7,"\x89\x0c\xcd\x00\x00\x00\x00", "mov dword ptr [ecx*8], ecx", insn32_t::FLAG_NONE},
	{7,"\x89\x0c\xd5\x00\x00\x00\x00", "mov dword ptr [edx*8], ecx", insn32_t::FLAG_NONE},
	{7,"\x89\x0c\xdd\x00\x00\x00\x00", "mov dword ptr [ebx*8], ecx", insn32_t::FLAG_NONE},
	{7,"\x89\x0c\xed\x00\x00\x00\x00", "mov dword ptr [ebp*8], ecx", insn32_t::FLAG_NONE},
	{7,"\x89\x0c\xf5\x00\x00\x00\x00", "mov dword ptr [esi*8], ecx", insn32_t::FLAG_NONE},
	{7,"\x89\x0c\xfd\x00\x00\x00\x00", "mov dword ptr [edi*8], ecx", insn32_t::FLAG_NONE},
	{7,"\x89\x14\x85\x00\x00\x00\x00", "mov dword ptr [eax*4], edx", insn32_t::FLAG_NONE},
	{7,"\x89\x14\x8d\x00\x00\x00\x00", "mov dword ptr [ecx*4], edx", insn32_t::FLAG_NONE},
	{7,"\x89\x14\x95\x00\x00\x00\x00", "mov dword ptr [edx*4], edx", insn32_t::FLAG_NONE},
	{7,"\x89\x14\x9d\x00\x00\x00\x00", "mov dword ptr [ebx*4], edx", insn32_t::FLAG_NONE},
	{7,"\x89\x14\xad\x00\x00\x00\x00", "mov dword ptr [ebp*4], edx", insn32_t::FLAG_NONE},
	{7,"\x89\x14\xb5\x00\x00\x00\x00", "mov dword ptr [esi*4], edx", insn32_t::FLAG_NONE},
	{7,"\x89\x14\xbd\x00\x00\x00\x00", "mov dword ptr [edi*4], edx", insn32_t::FLAG_NONE},
	{7,"\x89\x14\xc5\x00\x00\x00\x00", "mov dword ptr [eax*8], edx", insn32_t::FLAG_NONE},
	{7,"\x89\x14\xcd\x00\x00\x00\x00", "mov dword ptr [ecx*8], edx", insn32_t::FLAG_NONE},
	{7,"\x89\x14\xd5\x00\x00\x00\x00", "mov dword ptr [edx*8], edx", insn32_t::FLAG_NONE},
	{7,"\x89\x14\xdd\x00\x00\x00\x00", "mov dword ptr [ebx*8], edx", insn32_t::FLAG_NONE},
	{7,"\x89\x14\xed\x00\x00\x00\x00", "mov dword ptr [ebp*8], edx", insn32_t::FLAG_NONE},
	{7,"\x89\x14\xf5\x00\x00\x00\x00", "mov dword ptr [esi*8], edx", insn32_t::FLAG_NONE},
	{7,"\x89\x14\xfd\x00\x00\x00\x00", "mov dword ptr [edi*8], edx", insn32_t::FLAG_NONE},
	{7,"\x89\x1c\x85\x00\x00\x00\x00", "mov dword ptr [eax*4], ebx", insn32_t::FLAG_NONE},
	{7,"\x89\x1c\x8d\x00\x00\x00\x00", "mov dword ptr [ecx*4], ebx", insn32_t::FLAG_NONE},
	{7,"\x89\x1c\x95\x00\x00\x00\x00", "mov dword ptr [edx*4], ebx", insn32_t::FLAG_NONE},
	{7,"\x89\x1c\x9d\x00\x00\x00\x00", "mov dword ptr [ebx*4], ebx", insn32_t::FLAG_NONE},
	{7,"\x89\x1c\xad\x00\x00\x00\x00", "mov dword ptr [ebp*4], ebx", insn32_t::FLAG_NONE},
	{7,"\x89\x1c\xb5\x00\x00\x00\x00", "mov dword ptr [esi*4], ebx", insn32_t::FLAG_NONE},
	{7,"\x89\x1c\xbd\x00\x00\x00\x00", "mov dword ptr [edi*4], ebx", insn32_t::FLAG_NONE},
	{7,"\x89\x1c\xc5\x00\x00\x00\x00", "mov dword ptr [eax*8], ebx", insn32_t::FLAG_NONE},
	{7,"\x89\x1c\xcd\x00\x00\x00\x00", "mov dword ptr [ecx*8], ebx", insn32_t::FLAG_NONE},
	{7,"\x89\x1c\xd5\x00\x00\x00\x00", "mov dword ptr [edx*8], ebx", insn32_t::FLAG_NONE},
	{7,"\x89\x1c\xdd\x00\x00\x00\x00", "mov dword ptr [ebx*8], ebx", insn32_t::FLAG_NONE},
	{7,"\x89\x1c\xed\x00\x00\x00\x00", "mov dword ptr [ebp*8], ebx", insn32_t::FLAG_NONE},
	{7,"\x89\x1c\xf5\x00\x00\x00\x00", "mov dword ptr [esi*8], ebx", insn32_t::FLAG_NONE},
	{7,"\x89\x1c\xfd\x00\x00\x00\x00", "mov dword ptr [edi*8], ebx", insn32_t::FLAG_NONE},
	{7,"\x89\x24\x85\x00\x00\x00\x00", "mov dword ptr [eax*4], esp", insn32_t::FLAG_NONE},
	{7,"\x89\x24\x8d\x00\x00\x00\x00", "mov dword ptr [ecx*4], esp", insn32_t::FLAG_NONE},
	{7,"\x89\x24\x95\x00\x00\x00\x00", "mov dword ptr [edx*4], esp", insn32_t::FLAG_NONE},
	{7,"\x89\x24\x9d\x00\x00\x00\x00", "mov dword ptr [ebx*4], esp", insn32_t::FLAG_NONE},
	{7,"\x89\x24\xad\x00\x00\x00\x00", "mov dword ptr [ebp*4], esp", insn32_t::FLAG_NONE},
	{7,"\x89\x24\xb5\x00\x00\x00\x00", "mov dword ptr [esi*4], esp", insn32_t::FLAG_NONE},
	{7,"\x89\x24\xbd\x00\x00\x00\x00", "mov dword ptr [edi*4], esp", insn32_t::FLAG_NONE},
	{7,"\x89\x24\xc5\x00\x00\x00\x00", "mov dword ptr [eax*8], esp", insn32_t::FLAG_NONE},
	{7,"\x89\x24\xcd\x00\x00\x00\x00", "mov dword ptr [ecx*8], esp", insn32_t::FLAG_NONE},
	{7,"\x89\x24\xd5\x00\x00\x00\x00", "mov dword ptr [edx*8], esp", insn32_t::FLAG_NONE},
	{7,"\x89\x24\xdd\x00\x00\x00\x00", "mov dword ptr [ebx*8], esp", insn32_t::FLAG_NONE},
	{7,"\x89\x24\xed\x00\x00\x00\x00", "mov dword ptr [ebp*8], esp", insn32_t::FLAG_NONE},
	{7,"\x89\x24\xf5\x00\x00\x00\x00", "mov dword ptr [esi*8], esp", insn32_t::FLAG_NONE},
	{7,"\x89\x24\xfd\x00\x00\x00\x00", "mov dword ptr [edi*8], esp", insn32_t::FLAG_NONE},
	{7,"\x89\x2c\x85\x00\x00\x00\x00", "mov dword ptr [eax*4], ebp", insn32_t::FLAG_NONE},
	{7,"\x89\x2c\x8d\x00\x00\x00\x00", "mov dword ptr [ecx*4], ebp", insn32_t::FLAG_NONE},
	{7,"\x89\x2c\x95\x00\x00\x00\x00", "mov dword ptr [edx*4], ebp", insn32_t::FLAG_NONE},
	{7,"\x89\x2c\x9d\x00\x00\x00\x00", "mov dword ptr [ebx*4], ebp", insn32_t::FLAG_NONE},
	{7,"\x89\x2c\xad\x00\x00\x00\x00", "mov dword ptr [ebp*4], ebp", insn32_t::FLAG_NONE},
	{7,"\x89\x2c\xb5\x00\x00\x00\x00", "mov dword ptr [esi*4], ebp", insn32_t::FLAG_NONE},
	{7,"\x89\x2c\xbd\x00\x00\x00\x00", "mov dword ptr [edi*4], ebp", insn32_t::FLAG_NONE},
	{7,"\x89\x2c\xc5\x00\x00\x00\x00", "mov dword ptr [eax*8], ebp", insn32_t::FLAG_NONE},
	{7,"\x89\x2c\xcd\x00\x00\x00\x00", "mov dword ptr [ecx*8], ebp", insn32_t::FLAG_NONE},
	{7,"\x89\x2c\xd5\x00\x00\x00\x00", "mov dword ptr [edx*8], ebp", insn32_t::FLAG_NONE},
	{7,"\x89\x2c\xdd\x00\x00\x00\x00", "mov dword ptr [ebx*8], ebp", insn32_t::FLAG_NONE},
	{7,"\x89\x2c\xed\x00\x00\x00\x00", "mov dword ptr [ebp*8], ebp", insn32_t::FLAG_NONE},
	{7,"\x89\x2c\xf5\x00\x00\x00\x00", "mov dword ptr [esi*8], ebp", insn32_t::FLAG_NONE},
	{7,"\x89\x2c\xfd\x00\x00\x00\x00", "mov dword ptr [edi*8], ebp", insn32_t::FLAG_NONE},
	{7,"\x89\x34\x85\x00\x00\x00\x00", "mov dword ptr [eax*4], esi", insn32_t::FLAG_NONE},
	{7,"\x89\x34\x8d\x00\x00\x00\x00", "mov dword ptr [ecx*4], esi", insn32_t::FLAG_NONE},
	{7,"\x89\x34\x95\x00\x00\x00\x00", "mov dword ptr [edx*4], esi", insn32_t::FLAG_NONE},
	{7,"\x89\x34\x9d\x00\x00\x00\x00", "mov dword ptr [ebx*4], esi", insn32_t::FLAG_NONE},
	{7,"\x89\x34\xad\x00\x00\x00\x00", "mov dword ptr [ebp*4], esi", insn32_t::FLAG_NONE},
	{7,"\x89\x34\xb5\x00\x00\x00\x00", "mov dword ptr [esi*4], esi", insn32_t::FLAG_NONE},
	{7,"\x89\x34\xbd\x00\x00\x00\x00", "mov dword ptr [edi*4], esi", insn32_t::FLAG_NONE},
	{7,"\x89\x34\xc5\x00\x00\x00\x00", "mov dword ptr [eax*8], esi", insn32_t::FLAG_NONE},
	{7,"\x89\x34\xcd\x00\x00\x00\x00", "mov dword ptr [ecx*8], esi", insn32_t::FLAG_NONE},
	{7,"\x89\x34\xd5\x00\x00\x00\x00", "mov dword ptr [edx*8], esi", insn32_t::FLAG_NONE},
	{7,"\x89\x34\xdd\x00\x00\x00\x00", "mov dword ptr [ebx*8], esi", insn32_t::FLAG_NONE},
	{7,"\x89\x34\xed\x00\x00\x00\x00", "mov dword ptr [ebp*8], esi", insn32_t::FLAG_NONE},
	{7,"\x89\x34\xf5\x00\x00\x00\x00", "mov dword ptr [esi*8], esi", insn32_t::FLAG_NONE},
	{7,"\x89\x34\xfd\x00\x00\x00\x00", "mov dword ptr [edi*8], esi", insn32_t::FLAG_NONE},
	{7,"\x89\x3c\x85\x00\x00\x00\x00", "mov dword ptr [eax*4], edi", insn32_t::FLAG_NONE},
	{7,"\x89\x3c\x8d\x00\x00\x00\x00", "mov dword ptr [ecx*4], edi", insn32_t::FLAG_NONE},
	{7,"\x89\x3c\x95\x00\x00\x00\x00", "mov dword ptr [edx*4], edi", insn32_t::FLAG_NONE},
	{7,"\x89\x3c\x9d\x00\x00\x00\x00", "mov dword ptr [ebx*4], edi", insn32_t::FLAG_NONE},
	{7,"\x89\x3c\xad\x00\x00\x00\x00", "mov dword ptr [ebp*4], edi", insn32_t::FLAG_NONE},
	{7,"\x89\x3c\xb5\x00\x00\x00\x00", "mov dword ptr [esi*4], edi", insn32_t::FLAG_NONE},
	{7,"\x89\x3c\xbd\x00\x00\x00\x00", "mov dword ptr [edi*4], edi", insn32_t::FLAG_NONE},
	{7,"\x89\x3c\xc5\x00\x00\x00\x00", "mov dword ptr [eax*8], edi", insn32_t::FLAG_NONE},
	{7,"\x89\x3c\xcd\x00\x00\x00\x00", "mov dword ptr [ecx*8], edi", insn32_t::FLAG_NONE},
	{7,"\x89\x3c\xd5\x00\x00\x00\x00", "mov dword ptr [edx*8], edi", insn32_t::FLAG_NONE},
	{7,"\x89\x3c\xdd\x00\x00\x00\x00", "mov dword ptr [ebx*8], edi", insn32_t::FLAG_NONE},
	{7,"\x89\x3c\xed\x00\x00\x00\x00", "mov dword ptr [ebp*8], edi", insn32_t::FLAG_NONE},
	{7,"\x89\x3c\xf5\x00\x00\x00\x00", "mov dword ptr [esi*8], edi", insn32_t::FLAG_NONE},
	{7,"\x89\x3c\xfd\x00\x00\x00\x00", "mov dword ptr [edi*8], edi", insn32_t::FLAG_NONE},
	{7,"\x8b\x04\x85\x00\x00\x00\x00", "mov eax, dword ptr [eax*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x04\x8d\x00\x00\x00\x00", "mov eax, dword ptr [ecx*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x04\x95\x00\x00\x00\x00", "mov eax, dword ptr [edx*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x04\x9d\x00\x00\x00\x00", "mov eax, dword ptr [ebx*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x04\xad\x00\x00\x00\x00", "mov eax, dword ptr [ebp*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x04\xb5\x00\x00\x00\x00", "mov eax, dword ptr [esi*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x04\xbd\x00\x00\x00\x00", "mov eax, dword ptr [edi*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x04\xc5\x00\x00\x00\x00", "mov eax, dword ptr [eax*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x04\xcd\x00\x00\x00\x00", "mov eax, dword ptr [ecx*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x04\xd5\x00\x00\x00\x00", "mov eax, dword ptr [edx*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x04\xdd\x00\x00\x00\x00", "mov eax, dword ptr [ebx*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x04\xed\x00\x00\x00\x00", "mov eax, dword ptr [ebp*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x04\xf5\x00\x00\x00\x00", "mov eax, dword ptr [esi*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x04\xfd\x00\x00\x00\x00", "mov eax, dword ptr [edi*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x0c\x85\x00\x00\x00\x00", "mov ecx, dword ptr [eax*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x0c\x8d\x00\x00\x00\x00", "mov ecx, dword ptr [ecx*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x0c\x95\x00\x00\x00\x00", "mov ecx, dword ptr [edx*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x0c\x9d\x00\x00\x00\x00", "mov ecx, dword ptr [ebx*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x0c\xad\x00\x00\x00\x00", "mov ecx, dword ptr [ebp*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x0c\xb5\x00\x00\x00\x00", "mov ecx, dword ptr [esi*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x0c\xbd\x00\x00\x00\x00", "mov ecx, dword ptr [edi*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x0c\xc5\x00\x00\x00\x00", "mov ecx, dword ptr [eax*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x0c\xcd\x00\x00\x00\x00", "mov ecx, dword ptr [ecx*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x0c\xd5\x00\x00\x00\x00", "mov ecx, dword ptr [edx*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x0c\xdd\x00\x00\x00\x00", "mov ecx, dword ptr [ebx*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x0c\xed\x00\x00\x00\x00", "mov ecx, dword ptr [ebp*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x0c\xf5\x00\x00\x00\x00", "mov ecx, dword ptr [esi*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x0c\xfd\x00\x00\x00\x00", "mov ecx, dword ptr [edi*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x14\x85\x00\x00\x00\x00", "mov edx, dword ptr [eax*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x14\x8d\x00\x00\x00\x00", "mov edx, dword ptr [ecx*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x14\x95\x00\x00\x00\x00", "mov edx, dword ptr [edx*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x14\x9d\x00\x00\x00\x00", "mov edx, dword ptr [ebx*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x14\xad\x00\x00\x00\x00", "mov edx, dword ptr [ebp*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x14\xb5\x00\x00\x00\x00", "mov edx, dword ptr [esi*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x14\xbd\x00\x00\x00\x00", "mov edx, dword ptr [edi*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x14\xc5\x00\x00\x00\x00", "mov edx, dword ptr [eax*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x14\xcd\x00\x00\x00\x00", "mov edx, dword ptr [ecx*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x14\xd5\x00\x00\x00\x00", "mov edx, dword ptr [edx*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x14\xdd\x00\x00\x00\x00", "mov edx, dword ptr [ebx*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x14\xed\x00\x00\x00\x00", "mov edx, dword ptr [ebp*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x14\xf5\x00\x00\x00\x00", "mov edx, dword ptr [esi*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x14\xfd\x00\x00\x00\x00", "mov edx, dword ptr [edi*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x1c\x85\x00\x00\x00\x00", "mov ebx, dword ptr [eax*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x1c\x8d\x00\x00\x00\x00", "mov ebx, dword ptr [ecx*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x1c\x95\x00\x00\x00\x00", "mov ebx, dword ptr [edx*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x1c\x9d\x00\x00\x00\x00", "mov ebx, dword ptr [ebx*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x1c\xad\x00\x00\x00\x00", "mov ebx, dword ptr [ebp*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x1c\xb5\x00\x00\x00\x00", "mov ebx, dword ptr [esi*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x1c\xbd\x00\x00\x00\x00", "mov ebx, dword ptr [edi*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x1c\xc5\x00\x00\x00\x00", "mov ebx, dword ptr [eax*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x1c\xcd\x00\x00\x00\x00", "mov ebx, dword ptr [ecx*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x1c\xd5\x00\x00\x00\x00", "mov ebx, dword ptr [edx*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x1c\xdd\x00\x00\x00\x00", "mov ebx, dword ptr [ebx*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x1c\xed\x00\x00\x00\x00", "mov ebx, dword ptr [ebp*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x1c\xf5\x00\x00\x00\x00", "mov ebx, dword ptr [esi*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x1c\xfd\x00\x00\x00\x00", "mov ebx, dword ptr [edi*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x24\x85\x00\x00\x00\x00", "mov esp, dword ptr [eax*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x24\x8d\x00\x00\x00\x00", "mov esp, dword ptr [ecx*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x24\x95\x00\x00\x00\x00", "mov esp, dword ptr [edx*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x24\x9d\x00\x00\x00\x00", "mov esp, dword ptr [ebx*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x24\xad\x00\x00\x00\x00", "mov esp, dword ptr [ebp*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x24\xb5\x00\x00\x00\x00", "mov esp, dword ptr [esi*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x24\xbd\x00\x00\x00\x00", "mov esp, dword ptr [edi*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x24\xc5\x00\x00\x00\x00", "mov esp, dword ptr [eax*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x24\xcd\x00\x00\x00\x00", "mov esp, dword ptr [ecx*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x24\xd5\x00\x00\x00\x00", "mov esp, dword ptr [edx*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x24\xdd\x00\x00\x00\x00", "mov esp, dword ptr [ebx*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x24\xed\x00\x00\x00\x00", "mov esp, dword ptr [ebp*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x24\xf5\x00\x00\x00\x00", "mov esp, dword ptr [esi*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x24\xfd\x00\x00\x00\x00", "mov esp, dword ptr [edi*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x2c\x85\x00\x00\x00\x00", "mov ebp, dword ptr [eax*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x2c\x8d\x00\x00\x00\x00", "mov ebp, dword ptr [ecx*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x2c\x95\x00\x00\x00\x00", "mov ebp, dword ptr [edx*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x2c\x9d\x00\x00\x00\x00", "mov ebp, dword ptr [ebx*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x2c\xad\x00\x00\x00\x00", "mov ebp, dword ptr [ebp*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x2c\xb5\x00\x00\x00\x00", "mov ebp, dword ptr [esi*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x2c\xbd\x00\x00\x00\x00", "mov ebp, dword ptr [edi*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x2c\xc5\x00\x00\x00\x00", "mov ebp, dword ptr [eax*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x2c\xcd\x00\x00\x00\x00", "mov ebp, dword ptr [ecx*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x2c\xd5\x00\x00\x00\x00", "mov ebp, dword ptr [edx*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x2c\xdd\x00\x00\x00\x00", "mov ebp, dword ptr [ebx*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x2c\xed\x00\x00\x00\x00", "mov ebp, dword ptr [ebp*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x2c\xf5\x00\x00\x00\x00", "mov ebp, dword ptr [esi*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x2c\xfd\x00\x00\x00\x00", "mov ebp, dword ptr [edi*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x34\x85\x00\x00\x00\x00", "mov esi, dword ptr [eax*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x34\x8d\x00\x00\x00\x00", "mov esi, dword ptr [ecx*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x34\x95\x00\x00\x00\x00", "mov esi, dword ptr [edx*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x34\x9d\x00\x00\x00\x00", "mov esi, dword ptr [ebx*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x34\xad\x00\x00\x00\x00", "mov esi, dword ptr [ebp*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x34\xb5\x00\x00\x00\x00", "mov esi, dword ptr [esi*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x34\xbd\x00\x00\x00\x00", "mov esi, dword ptr [edi*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x34\xc5\x00\x00\x00\x00", "mov esi, dword ptr [eax*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x34\xcd\x00\x00\x00\x00", "mov esi, dword ptr [ecx*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x34\xd5\x00\x00\x00\x00", "mov esi, dword ptr [edx*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x34\xdd\x00\x00\x00\x00", "mov esi, dword ptr [ebx*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x34\xed\x00\x00\x00\x00", "mov esi, dword ptr [ebp*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x34\xf5\x00\x00\x00\x00", "mov esi, dword ptr [esi*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x34\xfd\x00\x00\x00\x00", "mov esi, dword ptr [edi*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x3c\x85\x00\x00\x00\x00", "mov edi, dword ptr [eax*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x3c\x8d\x00\x00\x00\x00", "mov edi, dword ptr [ecx*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x3c\x95\x00\x00\x00\x00", "mov edi, dword ptr [edx*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x3c\x9d\x00\x00\x00\x00", "mov edi, dword ptr [ebx*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x3c\xad\x00\x00\x00\x00", "mov edi, dword ptr [ebp*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x3c\xb5\x00\x00\x00\x00", "mov edi, dword ptr [esi*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x3c\xbd\x00\x00\x00\x00", "mov edi, dword ptr [edi*4]", insn32_t::FLAG_NONE},
	{7,"\x8b\x3c\xc5\x00\x00\x00\x00", "mov edi, dword ptr [eax*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x3c\xcd\x00\x00\x00\x00", "mov edi, dword ptr [ecx*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x3c\xd5\x00\x00\x00\x00", "mov edi, dword ptr [edx*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x3c\xdd\x00\x00\x00\x00", "mov edi, dword ptr [ebx*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x3c\xed\x00\x00\x00\x00", "mov edi, dword ptr [ebp*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x3c\xf5\x00\x00\x00\x00", "mov edi, dword ptr [esi*8]", insn32_t::FLAG_NONE},
	{7,"\x8b\x3c\xfd\x00\x00\x00\x00", "mov edi, dword ptr [edi*8]", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xc0", "mov ax, ax", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xe8", "mov ax, bp", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xd8", "mov ax, bx", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xc8", "mov ax, cx", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xf8", "mov ax, di", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xd0", "mov ax, dx", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xf0", "mov ax, si", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xe0", "mov ax, sp", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xc3", "mov bx, ax", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xeb", "mov bx, bp", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xdb", "mov bx, bx", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xcb", "mov bx, cx", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xfb", "mov bx, di", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xd3", "mov bx, dx", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xf3", "mov bx, si", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xe3", "mov bx, sp", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xc1", "mov cx, ax", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xe9", "mov cx, bp", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xd9", "mov cx, bx", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xc9", "mov cx, cx", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xf9", "mov cx, di", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xd1", "mov cx, dx", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xf1", "mov cx, si", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xe1", "mov cx, sp", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xc7", "mov di, ax", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xef", "mov di, bp", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xdf", "mov di, bx", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xcf", "mov di, cx", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xff", "mov di, di", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xd7", "mov di, dx", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xf7", "mov di, si", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xe7", "mov di, sp", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xc2", "mov dx, ax", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xea", "mov dx, bp", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xda", "mov dx, bx", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xca", "mov dx, cx", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xfa", "mov dx, di", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xd2", "mov dx, dx", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xf2", "mov dx, si", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xe2", "mov dx, sp", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xc6", "mov si, ax", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xee", "mov si, bp", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xde", "mov si, bx", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xce", "mov si, cx", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xfe", "mov si, di", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xd6", "mov si, dx", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xf6", "mov si, si", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xe6", "mov si, sp", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xc4", "mov sp, ax", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xec", "mov sp, bp", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xdc", "mov sp, bx", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xcc", "mov sp, cx", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xfc", "mov sp, di", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xd4", "mov sp, dx", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xf4", "mov sp, si", insn32_t::FLAG_NONE},
	{3,"\x66\x89\xe4", "mov sp, sp", insn32_t::FLAG_NONE},


	// test opcodes with no operands (or implicit ones, which we consider to be "none")
	{1, "\x27", "daa", insn32_t::FLAG_32BIT_ONLY | insn32_t::FLAG_RW_FLAGS},
	{1, "\x2f", "das", insn32_t::FLAG_32BIT_ONLY | insn32_t::FLAG_RW_FLAGS},
	{1, "\x37", "aaa", insn32_t::FLAG_32BIT_ONLY | insn32_t::FLAG_RW_FLAGS},
	{1, "\x3f", "aas", insn32_t::FLAG_32BIT_ONLY | insn32_t::FLAG_RW_FLAGS},
	{1, "\x60", "pushad", insn32_t::FLAG_32BIT_ONLY | insn32_t::FLAG_STACK},
	{1, "\x61", "popad", insn32_t::FLAG_32BIT_ONLY | insn32_t::FLAG_STACK},
	{1, "\x6c", "insb", insn32_t::FLAG_R_FLAGS},
	{1, "\x6d", "insd", insn32_t::FLAG_R_FLAGS},
	{1, "\x6e", "outsb", insn32_t::FLAG_R_FLAGS},
	{1, "\x6f", "outsd", insn32_t::FLAG_R_FLAGS},
	{1, "\x90", "nop", insn32_t::FLAG_NONE},
	{1, "\x98", "cwde", insn32_t::FLAG_NONE},
	{1, "\x99", "cdq", insn32_t::FLAG_NONE},
	{1, "\x9c", "pushfd", insn32_t::FLAG_STACK | insn32_t::FLAG_R_FLAGS},
	{1, "\x9d", "popfd", insn32_t::FLAG_STACK | insn32_t::FLAG_W_FLAGS},
	{1, "\x9e", "sahf", insn32_t::FLAG_NONE},
	{1, "\x9f", "lahf", insn32_t::FLAG_R_FLAGS},
	{1, "\xa4", "movsb", insn32_t::FLAG_R_FLAGS},
	{1, "\xa5", "movsd", insn32_t::FLAG_R_FLAGS},
	{1, "\xa6", "cmpsb", insn32_t::FLAG_RW_FLAGS},
	{1, "\xa7", "cmpsd", insn32_t::FLAG_RW_FLAGS},
	{1, "\xaa", "stosb", insn32_t::FLAG_R_FLAGS},
	{1, "\xab", "stosd", insn32_t::FLAG_R_FLAGS},
	{1, "\xac", "lodsb", insn32_t::FLAG_R_FLAGS},
	{1, "\xad", "lodsd", insn32_t::FLAG_R_FLAGS},
	{1, "\xae", "scasb", insn32_t::FLAG_RW_FLAGS},
	{1, "\xaf", "scasd", insn32_t::FLAG_RW_FLAGS},
	{1, "\xc3", "ret", insn32_t::FLAG_STACK},
	{1, "\xc9", "leave", insn32_t::FLAG_STACK},
	{1, "\xcb", "retf", insn32_t::FLAG_STACK},
	{1, "\xcc", "int3", insn32_t::FLAG_W_FLAGS},
	{1, "\xce", "into", insn32_t::FLAG_RW_FLAGS | insn32_t::FLAG_32BIT_ONLY},
	{1, "\xcf", "iret", insn32_t::FLAG_STACK | insn32_t::FLAG_W_FLAGS},
	{1, "\xcf", "iret", insn32_t::FLAG_STACK | insn32_t::FLAG_W_FLAGS},
	{1, "\xd6", "salc", insn32_t::FLAG_UNDOCUMENTED | insn32_t::FLAG_R_FLAGS},
	{1, "\xf4", "hlt", insn32_t::FLAG_RING0},
	{1, "\xf5", "cmc", insn32_t::FLAG_RW_FLAGS},
	{1, "\xf8", "clc", insn32_t::FLAG_W_FLAGS},
	{1, "\xf9", "stc", insn32_t::FLAG_W_FLAGS},
	{1, "\xfa", "cli", insn32_t::FLAG_W_FLAGS},
	{1, "\xfb", "sti", insn32_t::FLAG_W_FLAGS},
	{1, "\xfc", "cld", insn32_t::FLAG_W_FLAGS},
	{1, "\xfd", "std", insn32_t::FLAG_W_FLAGS},
//	{2, "\x0f\x05", "syscall", insn32_t::FLAG_NONE}, // is this actually valid on i386?
	{2, "\x0f\x06", "clts", insn32_t::FLAG_RING0},
//	{2, "\x0f\x07", "sysret", insn32_t::FLAG_NONE}, // is this actually valid on i386?
	{2, "\x0f\x08", "invd", insn32_t::FLAG_NONE},
	{2, "\x0f\x09", "wbinvd", insn32_t::FLAG_NONE},
	{2, "\x0f\x0b", "ud2", insn32_t::FLAG_PENTIUM_PRO},
	{2, "\x0f\x30", "wrmsr", insn32_t::FLAG_NONE},
	{2, "\x0f\x31", "rdtsc", insn32_t::FLAG_NONE},
	{2, "\x0f\x32", "rdmsr", insn32_t::FLAG_NONE},
	{2, "\x0f\x33", "rdpmc", insn32_t::FLAG_NONE},
	{2, "\x0f\x34", "sysenter", insn32_t::FLAG_PENTIUM_PRO | insn32_t::FLAG_W_FLAGS},
	{2, "\x0f\x35", "sysexit", insn32_t::FLAG_PENTIUM_PRO | insn32_t::FLAG_W_FLAGS},
	{2, "\x0f\x37", "getsec", insn32_t::FLAG_NONE},
	{2, "\x0f\x77", "emms", insn32_t::FLAG_MMX},
	{2, "\x0f\xa2", "cpuid", insn32_t::FLAG_NONE},
	{2, "\x0f\xaa", "rsm", insn32_t::FLAG_NONE},
	{2, "\x66\x60", "pushaw", insn32_t::FLAG_32BIT_ONLY | insn32_t::FLAG_STACK},
	{2, "\x66\x61", "popaw", insn32_t::FLAG_32BIT_ONLY | insn32_t::FLAG_STACK},
	{2, "\x66\x6d", "insw", insn32_t::FLAG_R_FLAGS},
	{2, "\x66\x6f", "outsw", insn32_t::FLAG_R_FLAGS},
	{2, "\x66\x98", "cbw", insn32_t::FLAG_NONE},
	{2, "\x66\x99", "cwd", insn32_t::FLAG_NONE},
	{2, "\x66\x9c", "pushfw", insn32_t::FLAG_STACK | insn32_t::FLAG_R_FLAGS},
	{2, "\x66\x9d", "popfw", insn32_t::FLAG_STACK | insn32_t::FLAG_W_FLAGS},
	{2, "\x66\xa5", "movsw", insn32_t::FLAG_R_FLAGS},
	{2, "\x66\xa7", "cmpsw", insn32_t::FLAG_RW_FLAGS},
	{2, "\x66\xab", "stosw", insn32_t::FLAG_R_FLAGS},
	{2, "\x66\xad", "lodsw", insn32_t::FLAG_R_FLAGS},
	{2, "\x66\xaf", "scasw", insn32_t::FLAG_RW_FLAGS},
	{2, "\xd9\xd0", "fnop", insn32_t::FLAG_FPU},
	{2, "\xd9\xe0", "fchs", insn32_t::FLAG_FPU},
	{2, "\xd9\xe1", "fabs", insn32_t::FLAG_FPU},
	{2, "\xd9\xe4", "ftst", insn32_t::FLAG_FPU},
	{2, "\xd9\xe5", "fxam", insn32_t::FLAG_FPU},
	{2, "\xd9\xe8", "fld1", insn32_t::FLAG_FPU},
	{2, "\xd9\xe9", "fldl2t", insn32_t::FLAG_FPU},
	{2, "\xd9\xea", "fldl2e", insn32_t::FLAG_FPU},
	{2, "\xd9\xeb", "fldpi", insn32_t::FLAG_FPU},
	{2, "\xd9\xec", "fldlg2", insn32_t::FLAG_FPU},
	{2, "\xd9\xed", "fldln2", insn32_t::FLAG_FPU},
	{2, "\xd9\xee", "fldz", insn32_t::FLAG_FPU},
	{2, "\xd9\xf0", "f2xm1", insn32_t::FLAG_FPU},
	{2, "\xd9\xf1", "fyl2x", insn32_t::FLAG_FPU},
	{2, "\xd9\xf2", "fptan", insn32_t::FLAG_FPU},
	{2, "\xd9\xf3", "fpatan", insn32_t::FLAG_FPU},
	{2, "\xd9\xf4", "fxtract", insn32_t::FLAG_FPU},
	{2, "\xd9\xf5", "fprem1", insn32_t::FLAG_FPU},
	{2, "\xd9\xf6", "fdecstp", insn32_t::FLAG_FPU},
	{2, "\xd9\xf7", "fincstp", insn32_t::FLAG_FPU},
	{2, "\xd9\xf8", "fprem", insn32_t::FLAG_FPU},
	{2, "\xd9\xf9", "fyl2xp1", insn32_t::FLAG_FPU},
	{2, "\xd9\xfa", "fsqrt", insn32_t::FLAG_FPU},
	{2, "\xd9\xfb", "fsincos", insn32_t::FLAG_FPU},
	{2, "\xd9\xfc", "frndint", insn32_t::FLAG_FPU},
	{2, "\xd9\xfd", "fscale", insn32_t::FLAG_FPU},
	{2, "\xd9\xfe", "fsin", insn32_t::FLAG_FPU},
	{2, "\xd9\xff", "fcos", insn32_t::FLAG_FPU},
	{2, "\xda\xe9", "fucompp", insn32_t::FLAG_FPU},
	{2, "\xde\xd9", "fcompp", insn32_t::FLAG_FPU},
	{2, "\xf3\x90", "pause", insn32_t::FLAG_SSE2},
	{3, "\x0f\x01\xc1", "vmcall", insn32_t::FLAG_VMX | insn32_t::FLAG_W_FLAGS},
	{3, "\x0f\x01\xc2", "vmlaunch", insn32_t::FLAG_VMX | insn32_t::FLAG_W_FLAGS},
	{3, "\x0f\x01\xc3", "vmresume", insn32_t::FLAG_VMX | insn32_t::FLAG_W_FLAGS},
	{3, "\x0f\x01\xc4", "vmxoff", insn32_t::FLAG_VMX | insn32_t::FLAG_W_FLAGS},
	{3, "\x0f\x01\xc8", "monitor", insn32_t::FLAG_SSE3},
	{3, "\x0f\x01\xc9", "mwait", insn32_t::FLAG_SSE3},
	{3, "\x0f\x01\xd0", "xgetbv", insn32_t::FLAG_NONE},
	{3, "\x0f\x01\xd1", "xsetbv", insn32_t::FLAG_NONE},
	{3, "\x0f\x01\xd8", "vmrun", insn32_t::FLAG_AMD},
	{3, "\x0f\x01\xd9", "vmmcall", insn32_t::FLAG_AMD},
	{3, "\x0f\x01\xda", "vmload", insn32_t::FLAG_AMD},
	{3, "\x0f\x01\xdb", "vmsave", insn32_t::FLAG_AMD},
	{3, "\x0f\x01\xdc", "stgi", insn32_t::FLAG_AMD},
	{3, "\x0f\x01\xdd", "clgi", insn32_t::FLAG_AMD},
	{3, "\x0f\x01\xde", "skinit", insn32_t::FLAG_AMD},
	{3, "\x0f\x01\xdf", "invlpga", insn32_t::FLAG_AMD},
	{3, "\x0f\x01\xf9", "rdtscp", insn32_t::FLAG_NONE},
	{3, "\x0f\xae\xe8", "lfence", insn32_t::FLAG_SSE2},
	{3, "\x0f\xae\xf0", "mfence", insn32_t::FLAG_SSE2},
	{3, "\x0f\xae\xf8", "sfence", insn32_t::FLAG_SSE},

	// stupid wait prefix stuff...
	{2, "\xdb\xe1", "fndisi", insn32_t::FLAG_FPU},
	{2, "\xdb\xe2", "fnclex", insn32_t::FLAG_FPU},
	{2, "\xdb\xe3", "fninit", insn32_t::FLAG_FPU},
	{2, "\xdb\xe4", "fnsetpm", insn32_t::FLAG_FPU | insn32_t::FLAG_286_ONLY},

	{3, "\x9b\xdb\xe1", "fdisi", insn32_t::FLAG_FPU},
	{3, "\x9b\xdb\xe2", "fclex", insn32_t::FLAG_FPU | insn32_t::FLAG_W_FLAGS},
	{3, "\x9b\xdb\xe3", "finit", insn32_t::FLAG_FPU},
	{3, "\x9b\xdb\xe4", "fsetpm", insn32_t::FLAG_FPU},

	{2, "\xdd\x30",         "fnsave [eax]", insn32_t::FLAG_FPU},
	{2, "\xdd\x38",         "fnstsw word ptr [eax]", insn32_t::FLAG_FPU},
	{2, "\xdf\xe0",         "fnstsw ax", insn32_t::FLAG_FPU},
	{3, "\x67\xdd\x30",     "fnsave [bx+si]", insn32_t::FLAG_FPU},
	{3, "\x67\xdd\x38",     "fnstsw word ptr [bx+si]", insn32_t::FLAG_FPU},
	{3, "\x9b\xdb\xe2",     "fclex", insn32_t::FLAG_FPU | insn32_t::FLAG_W_FLAGS},
	{3, "\x9b\xdb\xe3",     "finit", insn32_t::FLAG_FPU},
	{3, "\x9b\xdd\x30",     "fsave [eax]", insn32_t::FLAG_FPU},
	{3, "\x9b\xdd\x31",     "fsave [ecx]", insn32_t::FLAG_FPU},
	{3, "\x9b\xdd\x33",     "fsave [ebx]", insn32_t::FLAG_FPU},
	{3, "\x9b\xdd\x38",     "fstsw word ptr [eax]", insn32_t::FLAG_FPU},
	{3, "\x9b\xdf\xe0",     "fstsw ax", insn32_t::FLAG_FPU},
	{4, "\x67\x9b\xdd\x30", "fsave [bx+si]", insn32_t::FLAG_FPU},
	{4, "\x67\x9b\xdd\x38", "fstsw word ptr [bx+si]", insn32_t::FLAG_FPU},
	{4, "\x9b\xdd\x75\x94", "fsave [ebp-108]", insn32_t::FLAG_FPU},

	{2, "\xd9\x30", "fnstenv [eax]", insn32_t::FLAG_FPU},
	{2, "\xd9\x38", "fnstcw word ptr [eax]", insn32_t::FLAG_FPU},
	{3, "\x9b\xd9\x30", "fstenv [eax]", insn32_t::FLAG_FPU},
	{3, "\x9b\xd9\x31", "fstenv [ecx]", insn32_t::FLAG_FPU},
	{3, "\x9b\xd9\x38", "fstcw word ptr [eax]", insn32_t::FLAG_FPU},
	{3, "\x9b\xd9\x39", "fstcw word ptr [ecx]", insn32_t::FLAG_FPU},
	{4, "\x66\x0f\xfe\x08", "paddd xmm1, xmmword ptr [eax]", insn32_t::FLAG_MMX},
	{3, "\x0f\xfe\x08", "paddd mm1, qword ptr [eax]", insn32_t::FLAG_MMX},
	{5, "\x66\x0F\x38\x10\x11", "pblendvb xmm2, xmmword ptr [ecx]", insn32_t::FLAG_SSE4_1}
};

int main() {

	printf("[\n");
	for(size_t i = 0; i < sizeof(test32_data) / sizeof(test32_data[0]); ++i) {
	
		test_data_t *p = &test32_data[i];
		
		printf("{\n");
		printf("\t\"mnemonic\":\"%s\",\n", p->result);
		printf("\t\"bytes\":[");
		for(unsigned z = 0; z < p->size; ++z) {
			if(z != p->size - 1) {
				printf("\"%02x\",", p->bytes[z] & 0xff);
			} else {
				printf("\"%02x\"", p->bytes[z] & 0xff);
			}
		}
		printf("]\n");
		printf("}");
		
		if(i != (sizeof(test32_data) / sizeof(test32_data[0])) - 1) {
			printf(",");
		}
		
		printf("\n");
	
		/*
			unsigned size;
			const char *bytes;
			const char *result;
			unsigned int flags;
		*/
	
	}
	printf("]\n");
	
	return 0;


	for(size_t i = 0; i < sizeof(test32_data) / sizeof(test32_data[0]); ++i) {
		test_data_t *p = &test32_data[i];

		std::cout << "performing test #" << i << "...";
		insn32_t insn(p->bytes, p->bytes + p->size, 0x00000000, std::nothrow);

		if(!insn.valid() || edisassm::to_string(insn) != p->result) {
			std::cout << "\n----------\n";
			std::cout << edisassm::to_string(insn) << " != " << p->result << std::endl;
			std::cout << "FAIL" << std::endl;
			return -1;
		}

		if(insn.size() != p->size) {
			std::cout << "\n----------\n";
			std::cout << edisassm::to_byte_string(insn) << " incorrect size" << std::endl;
			std::cout << "FAIL" << std::endl;
			return -1;
		}

		if(insn.flags() != p->flags) {
			std::cout << "\n----------\n";
			std::cout << edisassm::to_byte_string(insn) << " wrong flags" << std::endl;
			std::cout << "FAIL" << std::endl;
			return -1;
		}

		std::cout << " " << edisassm::to_byte_string(insn) << " '" << edisassm::to_string(insn) << "' " << "OK" << std::endl;
	}
}
